@@ -12,35 +12,45 @@
 #include "ft2_gui.h"
 #include "ft2_midi.h"
 #include "ft2_wav_renderer.h"
-#include "ft2_module_loader.h"
 #include "ft2_mix.h"
-#include "ft2_audio.h"
+#include "ft2_tables.h"
+#include "ft2_structs.h"
 
 #define INITIAL_DITHER_SEED 0x12345000
 
+// globalized
+audio_t audio;
+pattSyncData_t *pattSyncEntry;
+chSyncData_t *chSyncEntry;
+chSync_t chSync;
+pattSync_t pattSync;
+volatile bool pattQueueReading, pattQueueClearing, chQueueReading, chQueueClearing;
+
+
 static int8_t pmpCountDiv, pmpChannels = 2;
 static uint16_t smpBuffSize;
-static int32_t masterVol, oldAudioFreq, speedVal, pmpLeft, randSeed = INITIAL_DITHER_SEED;
+static int32_t masterVol, oldAudioFreq, pmpLeft, randSeed = INITIAL_DITHER_SEED;
 static int32_t prngStateL, prngStateR;
-static uint32_t tickTimeLen, tickTimeLenFrac, oldSFrq, oldSFrqRev = 0xFFFFFFFF;
+static uint32_t tickTimeLen, tickTimeLenFrac;
 static float fAudioAmpMul;
 static voice_t voice[MAX_VOICES * 2];
 static void (*sendAudSamplesFunc)(uint8_t *, uint32_t, uint8_t); // "send mixed samples" routines
 
-pattSyncData_t *pattSyncEntry;
-chSyncData_t *chSyncEntry;
+#if !defined __amd64__ && !defined _WIN64
+static int32_t oldPeriod;
+static uint32_t oldSFrq, oldSFrqRev;
+#endif
 
-volatile bool pattQueueReading, pattQueueClearing, chQueueReading, chQueueClearing;
-
-extern const uint32_t panningTab[257]; // defined at the bottom of this file
-
-void resetOldRevFreqs(void)
+#if !defined __amd64__ && !defined _WIN64
+void resetCachedMixerVars(void)
 {
+	oldPeriod = -1;
 	oldSFrq = 0;
 	oldSFrqRev = 0xFFFFFFFF;
 }
+#endif
 
-void stopVoice(uint8_t i)
+void stopVoice(int32_t i)
 {
 	voice_t *v;
 
@@ -66,7 +76,7 @@ bool setNewAudioSettings(void) // only call this from the main input/video threa
 		// set back old known working settings
 
 		config.audioFreq = audio.lastWorkingAudioFreq;
-		config.specialFlags &= ~(BITDEPTH_16 + BITDEPTH_24 + BUFFSIZE_512 + BUFFSIZE_1024 + BUFFSIZE_2048);
+		config.specialFlags &= ~(BITDEPTH_16 + BITDEPTH_32 + BUFFSIZE_512 + BUFFSIZE_1024 + BUFFSIZE_2048);
 		config.specialFlags |= audio.lastWorkingAudioBits;
 
 		if (audio.lastWorkingAudioDeviceName != NULL)
@@ -88,7 +98,7 @@ bool setNewAudioSettings(void) // only call this from the main input/video threa
 		}
 
 		// also update config audio radio buttons if we're on that screen at the moment
-		if (editor.ui.configScreenShown && editor.currConfigScreen == CONFIG_SCREEN_IO_DEVICES)
+		if (ui.configScreenShown && editor.currConfigScreen == CONFIG_SCREEN_IO_DEVICES)
 			setConfigIORadioButtonStates();
 
 		// if it didn't work to use the old settings again, then something is seriously wrong...
@@ -114,7 +124,7 @@ void setAudioAmp(int16_t ampFactor, int16_t master, bool bitDepth32Flag)
 
 	if (bitDepth32Flag)
 	{
-		// 32-bit floating point (24-bit)
+		// 32-bit floating point
 		fAudioAmpMul = fAudioNorm * (master / 256.0f) * (ampFactor / 32.0f);
 	}
 	else
@@ -148,11 +158,11 @@ void setSpeed(uint16_t bpm)
 	if (bpm == 0)
 		return;
 
-	speedVal = ((audio.freq + audio.freq) + (audio.freq >> 1)) / bpm; // (audio.freq * 2.5) / BPM
-	if (speedVal > 0) // calculate tick time length for audio/video sync timestamp
+	audio.speedVal = ((audio.freq + audio.freq) + (audio.freq >> 1)) / bpm; // (audio.freq * 2.5) / BPM
+	if (audio.speedVal > 0) // calculate tick time length for audio/video sync timestamp
 	{
 		// number of samples per tick -> tick length for performance counter
-		dFrac = modf(speedVal * audio.dSpeedValMul, &dInt);
+		dFrac = modf(audio.speedVal * audio.dSpeedValMul, &dInt);
 
 		/* - integer part -
 		** Cast to int32_t so that the compiler will use fast SSE2 float->int instructions.
@@ -162,7 +172,13 @@ void setSpeed(uint16_t bpm)
 
 		// - fractional part (scaled to 0..2^32-1) -
 		dFrac *= UINT32_MAX;
-		tickTimeLenFrac = (uint32_t)(dFrac + 0.5);
+		dFrac += 0.5;
+		if (dFrac > UINT32_MAX)
+			dFrac = UINT32_MAX;
+
+		tickTimeLenFrac = (uint32_t)dFrac;
+
+		audio.rampSpeedValMul = 0xFFFFFFFF / audio.speedVal;
 	}
 }
 
@@ -180,9 +196,9 @@ void audioSetInterpolation(bool interpolation)
 	unlockMixerCallback();
 }
 
-static inline void voiceUpdateVolumes(uint8_t i, uint8_t status)
+static void voiceUpdateVolumes(int32_t i, uint8_t status)
 {
-	int32_t volL, volR;
+	int32_t volL, volR, destVolL, destVolR;
 	uint32_t vol;
 	voice_t *v, *f;
 
@@ -216,8 +232,12 @@ static inline void voiceUpdateVolumes(uint8_t i, uint8_t status)
 				*f = *v; // copy voice
 
 				f->SVolIPLen = audio.quickVolSizeVal;
-				f->SLVolIP = -f->SLVol2 / f->SVolIPLen;
-				f->SRVolIP = -f->SRVol2 / f->SVolIPLen;
+
+				destVolL = -f->SLVol2;
+				destVolR = -f->SRVol2;
+
+				f->SLVolIP = ((int64_t)destVolL * audio.rampQuickVolMul) >> 32;
+				f->SRVolIP = ((int64_t)destVolR * audio.rampQuickVolMul) >> 32;
 
 				f->isFadeOutVoice = true;
 			}
@@ -240,14 +260,26 @@ static inline void voiceUpdateVolumes(uint8_t i, uint8_t status)
 		}
 		else
 		{
-			v->SVolIPLen = (status & IS_QuickVol) ? audio.quickVolSizeVal : speedVal;
-			v->SLVolIP = (volL - v->SLVol2) / v->SVolIPLen;
-			v->SRVolIP = (volR - v->SRVol2) / v->SVolIPLen;
+			destVolL = volL - v->SLVol2;
+			destVolR = volR - v->SRVol2;
+
+			if (status & IS_QuickVol)
+			{
+				v->SVolIPLen = audio.quickVolSizeVal;
+				v->SLVolIP = ((int64_t)destVolL * audio.rampQuickVolMul) >> 32;
+				v->SRVolIP = ((int64_t)destVolR * audio.rampQuickVolMul) >> 32;
+			}
+			else
+			{
+				v->SVolIPLen = audio.speedVal;
+				v->SLVolIP = ((int64_t)destVolL * audio.rampSpeedValMul) >> 32;
+				v->SRVolIP = ((int64_t)destVolR * audio.rampSpeedValMul) >> 32;
+			}
 		}
 	}
 }
 
-static void voiceTrigger(uint8_t i, sampleTyp *s, int32_t position)
+static void voiceTrigger(int32_t i, sampleTyp *s, int32_t position)
 {
 	bool sampleIs16Bit;
 	uint8_t loopType;
@@ -314,7 +346,7 @@ static void voiceTrigger(uint8_t i, sampleTyp *s, int32_t position)
 void mix_SaveIPVolumes(void) // for volume ramping
 {
 	voice_t *v = voice;
-	for (uint32_t i = 0; i < MAX_VOICES; i++, v++)
+	for (int32_t i = 0; i < song.antChn; i++, v++)
 	{
 		v->SLVol2 = v->SLVol1;
 		v->SRVol2 = v->SRVol1;
@@ -331,44 +363,43 @@ void mix_UpdateChannelVolPanFrq(void)
 	ch = stm;
 	v = voice;
 
-	for (uint8_t i = 0; i < song.antChn; i++, ch++, v++)
+	for (int32_t i = 0; i < song.antChn; i++, ch++, v++)
 	{
 		status = ch->tmpStatus = ch->status; // ch->tmpStatus is used for audio/video sync queue
-		if (status == 0)
-			continue;
-
+		if (status == 0) continue;
 		ch->status = 0;
 
-		// volume change
 		if (status & IS_Vol)
 			v->SVol = ch->finalVol;
 
-		// panning change
 		if (status & IS_Pan)
 			v->SPan = ch->finalPan;
 
-		// update mixing volumes if vol/pan change
 		if (status & (IS_Vol + IS_Pan))
 			voiceUpdateVolumes(i, status);
 
-		// frequency change
 		if (status & IS_Period)
 		{
+#if defined __amd64__ || defined _WIN64
 			v->SFrq = getFrequenceValue(ch->finalPeriod);
-
-			if (v->SFrq != oldSFrq)
+#else
+			// use cached values to prevent a 32-bit divsion all the time
+			if (ch->finalPeriod != oldPeriod)
 			{
-				oldSFrq = v->SFrq;
+				oldPeriod = ch->finalPeriod;
+
+				oldSFrq = getFrequenceValue(ch->finalPeriod);
 
 				oldSFrqRev = 0xFFFFFFFF;
 				if (oldSFrq != 0)
 					oldSFrqRev /= oldSFrq;
 			}
 
+			v->SFrq = oldSFrq;
 			v->SFrqRev = oldSFrqRev;
+#endif
 		}
 
-		// sample trigger (note)
 		if (status & IS_NyTon)
 			voiceTrigger(i, ch->smpPtr, ch->smpStartPos);
 	}
@@ -545,7 +576,7 @@ static void mixAudio(uint8_t *stream, uint32_t sampleBlockLength, uint8_t numAud
 	v = voice; // normal voices
 	r = &voice[MAX_VOICES]; // volume ramp voices
 
-	for (uint32_t i = 0; i < song.antChn; i++, v++, r++)
+	for (int32_t i = 0; i < song.antChn; i++, v++, r++)
 	{
 		// call the mixing routine currently set for the voice
 		if (v->mixRoutine != NULL) v->mixRoutine(v, sampleBlockLength); // mix normal voice
@@ -561,35 +592,35 @@ uint32_t mixReplayerTickToBuffer(uint8_t *stream, uint8_t bitDepth)
 {
 	voice_t *v, *r;
 
-	assert(speedVal <= MAX_WAV_RENDER_SAMPLES_PER_TICK);
-	memset(audio.mixBufferL, 0, speedVal * sizeof (int32_t));
-	memset(audio.mixBufferR, 0, speedVal * sizeof (int32_t));
+	assert(audio.speedVal <= MAX_WAV_RENDER_SAMPLES_PER_TICK);
+	memset(audio.mixBufferL, 0, audio.speedVal * sizeof (int32_t));
+	memset(audio.mixBufferR, 0, audio.speedVal * sizeof (int32_t));
 
 	// mix channels
 	v = voice; // normal voices
 	r = &voice[MAX_VOICES]; // volume ramp voices
 
-	for (uint32_t i = 0; i < song.antChn; i++, v++, r++)
+	for (int32_t i = 0; i < song.antChn; i++, v++, r++)
 	{
 		// call the mixing routine currently set for the voice
-		if (v->mixRoutine != NULL) v->mixRoutine(v, speedVal); // mix normal voice
-		if (r->mixRoutine != NULL) r->mixRoutine(r, speedVal); // mix volume ramp voice
+		if (v->mixRoutine != NULL) v->mixRoutine(v, audio.speedVal); // mix normal voice
+		if (r->mixRoutine != NULL) r->mixRoutine(r, audio.speedVal); // mix volume ramp voice
 	}
 
 	// normalize mix buffer and send to audio stream
 	if (bitDepth == 16)
 	{
 		if (config.specialFlags2 & DITHERED_AUDIO)
-			sendSamples16BitDitherStereo(stream, speedVal, 2);
+			sendSamples16BitDitherStereo(stream, audio.speedVal, 2);
 		else
-			sendSamples16BitStereo(stream, speedVal, 2);
+			sendSamples16BitStereo(stream, audio.speedVal, 2);
 	}
 	else
 	{
-		sendSamples24BitStereo(stream, speedVal, 2);
+		sendSamples24BitStereo(stream, audio.speedVal, 2);
 	}
 
-	return speedVal;
+	return audio.speedVal;
 }
 
 int32_t pattQueueReadSize(void)
@@ -895,7 +926,7 @@ static void SDLCALL mixCallback(void *userdata, Uint8 *stream, int len)
 				pattSyncData.patternPos = song.curReplayerPattPos;
 				pattSyncData.pattern = song.curReplayerPattNr;
 				pattSyncData.songPos = song.curReplayerSongPos;
-				pattSyncData.speed = (uint8_t)song.speed;
+				pattSyncData.speed = song.speed;
 				pattSyncData.tempo = (uint8_t)song.tempo;
 				pattSyncData.globalVol = (uint8_t)song.globVol;
 				pattSyncData.timestamp = audio.tickTime64;
@@ -903,12 +934,12 @@ static void SDLCALL mixCallback(void *userdata, Uint8 *stream, int len)
 			}
 
 			// push channel variables to sync queue
-			for (uint32_t i = 0; i < song.antChn; i++)
-			{
-				c = &chSyncData.channels[i];
-				s = &stm[i];
 
-				c->voiceDelta = voice[i].SFrq;
+			c = chSyncData.channels;
+			s = stm;
+
+			for (int32_t i = 0; i < song.antChn; i++, c++, s++)
+			{
 				c->finalPeriod = s->finalPeriod;
 				c->fineTune = s->fineTune;
 				c->relTonNr = s->relTonNr;
@@ -931,7 +962,7 @@ static void SDLCALL mixCallback(void *userdata, Uint8 *stream, int len)
 				audio.tickTime64++;
 			}
 
-			pmpLeft = speedVal;
+			pmpLeft = audio.speedVal;
 			replayerBusy = false;
 		}
 
@@ -987,8 +1018,8 @@ void updateSendAudSamplesRoutine(bool lockMixer)
 	if (lockMixer)
 		lockMixerCallback();
 
-	// force dither off if somehow set with 24-bit float (illegal)
-	if ((config.specialFlags2 & DITHERED_AUDIO) && (config.specialFlags & BITDEPTH_24))
+	// force dither off if somehow set with 32-bit float (illegal)
+	if ((config.specialFlags2 & DITHERED_AUDIO) && (config.specialFlags & BITDEPTH_32))
 		config.specialFlags2 &= ~DITHERED_AUDIO;
 
 	if (config.specialFlags2 & DITHERED_AUDIO)
@@ -1033,15 +1064,17 @@ static void calcAudioLatencyVars(uint16_t haveSamples, int32_t haveFreq)
 
 	dAudioLatencySecs = haveSamples / dHaveFreq;
 
-	// XXX: haveSamples and haveFreq better not be bogus values...
 	dFrac = modf(dAudioLatencySecs * editor.dPerfFreq, &dInt);
 
 	// integer part
-	audio.audLatencyPerfValInt = (uint32_t)dInt;
+	audio.audLatencyPerfValInt = (int32_t)dInt;
 
 	// fractional part (scaled to 0..2^32-1)
 	dFrac *= UINT32_MAX;
-	audio.audLatencyPerfValFrac = (uint32_t)(dFrac + 0.5);
+	dFrac += 0.5;
+	if (dFrac > UINT32_MAX)
+		dFrac = UINT32_MAX;
+	audio.audLatencyPerfValFrac = (uint32_t)dFrac;
 
 	audio.dAudioLatencyMs = dAudioLatencySecs * 1000.0;
 }
@@ -1079,14 +1112,7 @@ bool setupAudio(bool showErrorMsg)
 	closeAudio();
 
 	if (config.audioFreq < MIN_AUDIO_FREQ || config.audioFreq > MAX_AUDIO_FREQ)
-	{
-		// set default rate
-#ifdef __APPLE__
-		config.audioFreq = 44100;
-#else
-		config.audioFreq = 48000;
-#endif
-	}
+		config.audioFreq = 48000; // set default rate
 
 	// get audio buffer size from config special flags
 
@@ -1105,7 +1131,7 @@ bool setupAudio(bool showErrorMsg)
 
 	// these three may change after opening a device, but our mixer is dealing with it
 	want.freq = config.audioFreq;
-	want.format = (config.specialFlags & BITDEPTH_24) ? AUDIO_F32 : AUDIO_S16;
+	want.format = (config.specialFlags & BITDEPTH_32) ? AUDIO_F32 : AUDIO_S16;
 	want.channels = 2;
 	// -------------------------------------------------------------------------------
 	want.callback = mixCallback;
@@ -1124,7 +1150,7 @@ bool setupAudio(bool showErrorMsg)
 	if (have.format != AUDIO_S16 && have.format != AUDIO_F32)
 	{
 		if (showErrorMsg)
-			showErrorMsgBox("Couldn't open audio device:\nThe program doesn't support an SDL_AudioFormat of '%d' (not 16-bit or 24-bit float).",
+			showErrorMsgBox("Couldn't open audio device:\nThe program doesn't support an SDL_AudioFormat of '%d' (not 16-bit or 32-bit float).",
 				(uint32_t)have.format);
 
 		closeAudio();
@@ -1132,7 +1158,12 @@ bool setupAudio(bool showErrorMsg)
 	}
 
 	// test if the received audio rate is compatible
-	if (have.freq != 44100 && have.freq != 48000 && have.freq != 96000)
+
+#if defined _WIN64 || defined __amd64__
+	if (have.freq != 44100 && have.freq != 48000 && have.freq != 96000 && have.freq != 192000)
+#else
+	if (have.freq != 44100 && have.freq != 48000)
+#endif
 	{
 		if (showErrorMsg)
 			showErrorMsgBox("Couldn't open audio device:\nThe program doesn't support an audio output rate of %dHz. Sorry!", have.freq);
@@ -1153,14 +1184,14 @@ bool setupAudio(bool showErrorMsg)
 	// set new bit depth flag
 
 	newBitDepth = 16;
-	config.specialFlags &= ~BITDEPTH_24;
+	config.specialFlags &= ~BITDEPTH_32;
 	config.specialFlags |=  BITDEPTH_16;
 
 	if (have.format == AUDIO_F32)
 	{
 		newBitDepth = 24;
 		config.specialFlags &= ~BITDEPTH_16;
-		config.specialFlags |=  BITDEPTH_24;
+		config.specialFlags |=  BITDEPTH_32;
 	}
 
 	audio.haveFreq = have.freq;
@@ -1184,18 +1215,18 @@ bool setupAudio(bool showErrorMsg)
 	// make a copy of the new known working audio settings
 
 	audio.lastWorkingAudioFreq = config.audioFreq;
-	audio.lastWorkingAudioBits = config.specialFlags & (BITDEPTH_16 + BITDEPTH_24 + BUFFSIZE_512 + BUFFSIZE_1024 + BUFFSIZE_2048);
+	audio.lastWorkingAudioBits = config.specialFlags & (BITDEPTH_16 + BITDEPTH_32 + BUFFSIZE_512 + BUFFSIZE_1024 + BUFFSIZE_2048);
 	setLastWorkingAudioDevName();
 
 	// update config audio radio buttons if we're on that screen at the moment
-	if (editor.ui.configScreenShown && editor.currConfigScreen == CONFIG_SCREEN_IO_DEVICES)
+	if (ui.configScreenShown && editor.currConfigScreen == CONFIG_SCREEN_IO_DEVICES)
 		showConfigScreen();
 
 	updateWavRendererSettings();
-	setAudioAmp(config.boostLevel, config.masterVol, (config.specialFlags & BITDEPTH_24) ? true : false);
+	setAudioAmp(config.boostLevel, config.masterVol, (config.specialFlags & BITDEPTH_32) ? true : false);
 
 	// don't call stopVoices() in this routine
-	for (uint8_t i = 0; i < MAX_VOICES; i++)
+	for (int32_t i = 0; i < MAX_VOICES; i++)
 		stopVoice(i);
 
 	stopAllScopes();
@@ -1226,24 +1257,3 @@ void closeAudio(void)
 
 	freeAudioBuffers();
 }
-
-const uint32_t panningTab[257] = // panning table from FT2 code
-{
-		0, 4096, 5793, 7094, 8192, 9159,10033,10837,11585,12288,12953,13585,14189,14768,15326,15864,
-	16384,16888,17378,17854,18318,18770,19212,19644,20066,20480,20886,21283,21674,22058,22435,22806,
-	23170,23530,23884,24232,24576,24915,25249,25580,25905,26227,26545,26859,27170,27477,27780,28081,
-	28378,28672,28963,29251,29537,29819,30099,30377,30652,30924,31194,31462,31727,31991,32252,32511,
-	32768,33023,33276,33527,33776,34024,34270,34514,34756,34996,35235,35472,35708,35942,36175,36406,
-	36636,36864,37091,37316,37540,37763,37985,38205,38424,38642,38858,39073,39287,39500,39712,39923,
-	40132,40341,40548,40755,40960,41164,41368,41570,41771,41972,42171,42369,42567,42763,42959,43154,
-	43348,43541,43733,43925,44115,44305,44494,44682,44869,45056,45242,45427,45611,45795,45977,46160,
-	46341,46522,46702,46881,47059,47237,47415,47591,47767,47942,48117,48291,48465,48637,48809,48981,
-	49152,49322,49492,49661,49830,49998,50166,50332,50499,50665,50830,50995,51159,51323,51486,51649,
-	51811,51972,52134,52294,52454,52614,52773,52932,53090,53248,53405,53562,53719,53874,54030,54185,
-	54340,54494,54647,54801,54954,55106,55258,55410,55561,55712,55862,56012,56162,56311,56459,56608,
-	56756,56903,57051,57198,57344,57490,57636,57781,57926,58071,58215,58359,58503,58646,58789,58931,
-	59073,59215,59357,59498,59639,59779,59919,60059,60199,60338,60477,60615,60753,60891,61029,61166,
-	61303,61440,61576,61712,61848,61984,62119,62254,62388,62523,62657,62790,62924,63057,63190,63323,
-	63455,63587,63719,63850,63982,64113,64243,64374,64504,64634,64763,64893,65022,65151,65279,65408,
-	65536
-};
