@@ -45,7 +45,7 @@ enum
 	PAL_JUNGLE = 5,
 	PAL_LITHE_DARK = 6,
 	PAL_ROSE = 7,
-	PAL_SPACE_PIGS = 8,
+	PAL_DARK_MODE = 8,
 	PAL_VIOLENT = 9,
 	PAL_WHY_COLORS = 10, // default
 	PAL_USER_DEFINED = 11,
@@ -67,7 +67,7 @@ enum
 	// specialFlags
 	NO_VOLRAMP_FLAG = 1,
 	BITDEPTH_16 = 2,
-	BITDEPTH_24 = 4,
+	BITDEPTH_32 = 4,
 	BUFFSIZE_512 = 8,
 	BUFFSIZE_1024 = 16,
 	BUFFSIZE_2048 = 32,
@@ -208,6 +208,7 @@ void rbConfigAudio24bit(void);
 void rbConfigAudio44kHz(void);
 void rbConfigAudio48kHz(void);
 void rbConfigAudio96kHz(void);
+void rbConfigAudio192kHz(void);
 void rbConfigAudioInput44kHz(void);
 void rbConfigAudioInput48kHz(void);
 void rbConfigAudioInput96kHz(void);
