@@ -38,8 +38,9 @@
 #include "ft2_events.h"
 #include "ft2_video.h"
 #include "ft2_inst_ed.h"
+#include "ft2_structs.h"
 
-// hide POSIX warnings
+// hide POSIX warnings for chdir()
 #ifdef _MSC_VER
 #pragma warning(disable: 4996)
 #endif
@@ -75,7 +76,7 @@ static char FReq_SysReqText[196], *FReq_FileName, *FReq_NameTemp;
 static char *modTmpFName, *insTmpFName, *smpTmpFName, *patTmpFName, *trkTmpFName;
 static char *modTmpFNameUTF8; // for window title
 static uint8_t FReq_Item;
-static bool FReq_ShowAllFiles;
+static bool FReq_ShowAllFiles, insPathSet, smpPathSet, patPathSet, trkPathSet, firstTimeOpeningDiskOp = true;
 static int32_t FReq_EntrySelected = -1, FReq_FileCount, FReq_DirPos, lastMouseY;
 static UNICHAR *FReq_CurPathU, *FReq_ModCurPathU, *FReq_InsCurPathU, *FReq_SmpCurPathU, *FReq_PatCurPathU, *FReq_TrkCurPathU;
 static DirRec *FReq_Buffer;
@@ -104,7 +105,7 @@ int32_t getFileSize(UNICHAR *fileNameU) // returning -1 = filesize over 2GB
 	if (stat(fileNameU, &st) != 0)
 		return 0;
 
-	fSize = (int64_t)(st.st_size);
+	fSize = (int64_t)st.st_size;
 #endif
 	if (fSize < 0)
 		fSize = 0;
@@ -172,44 +173,63 @@ const UNICHAR *getDiskOpSmpPath(void)
 
 static void setupInitialPaths(void)
 {
-#ifdef _WIN32
-	UNICHAR pathU[PATH_MAX + 2];
-#endif
-
 	// the UNICHAR paths are already zeroed out
 
 #ifdef _WIN32
-	if (config.modulesPath[0]  != '\0') MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.modulesPath, -1, FReq_ModCurPathU, 80);
-	if (config.instrPath[0]    != '\0') MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.instrPath, -1, FReq_InsCurPathU, 80);
-	if (config.samplesPath[0]  != '\0') MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.samplesPath, -1, FReq_SmpCurPathU, 80);
-	if (config.patternsPath[0] != '\0') MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.patternsPath, -1, FReq_PatCurPathU, 80);
-	if (config.tracksPath[0]   != '\0') MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.tracksPath, -1, FReq_TrkCurPathU, 80);
+	if (config.modulesPath[0] != '\0')
+		MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.modulesPath, -1, FReq_ModCurPathU, 80);
+
+	if (config.instrPath[0] != '\0')
+	{
+		MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.instrPath, -1, FReq_InsCurPathU, 80);
+		insPathSet = true;
+	}
+
+	if (config.samplesPath[0] != '\0')
+	{
+		MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.samplesPath, -1, FReq_SmpCurPathU, 80);
+		smpPathSet = true;
+	}
+
+	if (config.patternsPath[0] != '\0')
+	{
+		MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.patternsPath, -1, FReq_PatCurPathU, 80);
+		patPathSet = true;
+	}
+
+	if (config.tracksPath[0] != '\0')
+	{
+		MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config.tracksPath, -1, FReq_TrkCurPathU, 80);
+		trkPathSet = true;
+	}
 #else
-	if (config.modulesPath[0]  != '\0') strncpy(FReq_ModCurPathU, config.modulesPath, 80);
-	if (config.instrPath[0]    != '\0') strncpy(FReq_InsCurPathU, config.instrPath, 80);
-	if (config.samplesPath[0]  != '\0') strncpy(FReq_SmpCurPathU, config.samplesPath, 80);
-	if (config.patternsPath[0] != '\0') strncpy(FReq_PatCurPathU, config.patternsPath, 80);
-	if (config.tracksPath[0]   != '\0') strncpy(FReq_TrkCurPathU, config.tracksPath, 80);
+	if (config.modulesPath[0] != '\0')
+		strncpy(FReq_ModCurPathU, config.modulesPath, 80);
+
+	if (config.instrPath[0] != '\0')
+	{
+		strncpy(FReq_InsCurPathU, config.instrPath, 80);
+		insPathSet = true;
+	}
+
+	if (config.samplesPath[0] != '\0')
+	{
+		strncpy(FReq_SmpCurPathU, config.samplesPath, 80);
+		smpPathSet = true;
+	}
+
+	if (config.patternsPath[0] != '\0')
+	{
+		strncpy(FReq_PatCurPathU, config.patternsPath, 80);
+		patPathSet = true;
+	}
+
+	if (config.tracksPath[0] != '\0')
+	{
+		strncpy(FReq_TrkCurPathU, config.tracksPath, 80);
+		trkPathSet = true;
+	}
 #endif
-
-	// set initial path to user directory
-#ifdef _WIN32
-	if (SHGetFolderPathW(NULL, CSIDL_PROFILE, NULL, 0, pathU) >= 0)
-		UNICHAR_CHDIR(pathU);
-#else
-	UNICHAR_CHDIR(getenv("HOME"));
-#endif
-
-	// if present in config, set custom "modules" path
-	if (UNICHAR_CHDIR(FReq_ModCurPathU) != 0)
-		UNICHAR_GETCWD(FReq_ModCurPathU, PATH_MAX);
-
-	if (UNICHAR_CHDIR(FReq_InsCurPathU) != 0) UNICHAR_STRCPY(FReq_InsCurPathU, FReq_ModCurPathU);
-	if (UNICHAR_CHDIR(FReq_SmpCurPathU) != 0) UNICHAR_STRCPY(FReq_SmpCurPathU, FReq_ModCurPathU);
-	if (UNICHAR_CHDIR(FReq_PatCurPathU) != 0) UNICHAR_STRCPY(FReq_PatCurPathU, FReq_ModCurPathU);
-	if (UNICHAR_CHDIR(FReq_TrkCurPathU) != 0) UNICHAR_STRCPY(FReq_TrkCurPathU, FReq_ModCurPathU);
-
-	UNICHAR_CHDIR(FReq_ModCurPathU); // set back after testing
 }
 
 static void freeDirRecBuffer(void)
@@ -650,7 +670,7 @@ void diskOpSetFilename(uint8_t type, UNICHAR *pathU)
 
 	free(ansiPath);
 
-	if (editor.ui.diskOpShown)
+	if (ui.diskOpShown)
 		drawTextBox(TB_DISKOP_FILENAME);
 }
 
@@ -751,14 +771,14 @@ void changeFilenameExt(char *name, char *ext, int32_t nameMaxLen)
 
 	 strcat(name, ext);
 
-	 if (editor.ui.diskOpShown)
+	 if (ui.diskOpShown)
 		diskOp_DrawDirectory();
 }
 
 void diskOpChangeFilenameExt(char *ext)
 {
 	changeFilenameExt(FReq_FileName, ext, PATH_MAX);
-	if (editor.ui.diskOpShown)
+	if (ui.diskOpShown)
 		diskOp_DrawDirectory();
 }
 
@@ -1117,7 +1137,7 @@ bool testDiskOpMouseDown(bool mouseHeldDlown)
 {
 	int32_t tmpEntry, max;
 
-	if (!editor.ui.diskOpShown || FReq_FileCount == 0)
+	if (!ui.diskOpShown || FReq_FileCount == 0)
 		return false;
 
 	max = FReq_FileCount - FReq_DirPos;
@@ -1181,7 +1201,7 @@ bool testDiskOpMouseDown(bool mouseHeldDlown)
 
 void testDiskOpMouseRelease(void)
 {
-	if (editor.ui.diskOpShown && FReq_EntrySelected != -1)
+	if (ui.diskOpShown && FReq_EntrySelected != -1)
 	{
 		if (mouse.x >= 169 && mouse.x <= 329 && mouse.y >= 4 && mouse.y <= 168)
 			fileListPressed((mouse.y - 4) / (FONT1_CHAR_H + 1));
@@ -1432,6 +1452,9 @@ static int8_t findFirst(DirRec *searchRec)
 	}
 #endif
 
+	if (searchRec->filesize < -1)
+		searchRec->filesize = -1;
+
 	if (handleEntrySkip(searchRec->nameU, searchRec->isDir))
 	{
 		// skip file
@@ -1512,6 +1535,9 @@ static int8_t findNext(DirRec *searchRec)
 	}
 #endif
 
+	if (searchRec->filesize < -1)
+		searchRec->filesize = -1;
+
 	if (handleEntrySkip(searchRec->nameU, searchRec->isDir))
 	{
 		// skip file
@@ -1891,8 +1917,6 @@ static int32_t SDLCALL diskOp_ReadDirectoryThread(void *ptr)
 	uint8_t lastFindFileFlag;
 	DirRec tmpBuffer, *newPtr;
 
-	(void)ptr;
-
 	FReq_DirPos = 0;
 
 	// free old buffer
@@ -1966,10 +1990,11 @@ static int32_t SDLCALL diskOp_ReadDirectoryThread(void *ptr)
 
 	setMouseBusy(false);
 
+	(void)ptr;
 	return true;
 }
 
-void startDiskOpFillThread(void)
+void diskOp_StartDirReadThread(void)
 {
 	editor.diskOpReadDone = false;
 
@@ -2047,7 +2072,7 @@ static void setDiskOpItemRadioButtons(void)
 	if (FReq_Item == DISKOP_ITEM_PATTERN) radioButtons[RB_DISKOP_PAT_SAVEAS_XP].state = RADIOBUTTON_CHECKED;
 	if (FReq_Item == DISKOP_ITEM_TRACK)   radioButtons[RB_DISKOP_TRK_SAVEAS_XT].state = RADIOBUTTON_CHECKED;
 
-	if (editor.ui.diskOpShown)
+	if (ui.diskOpShown)
 	{
 		switch (FReq_Item)
 		{
@@ -2076,11 +2101,82 @@ static void setDiskOpItem(uint8_t item)
 	FReq_Item = item;
 	switch (FReq_Item)
 	{
-		default: case DISKOP_ITEM_MODULE:  FReq_FileName = modTmpFName; FReq_CurPathU = FReq_ModCurPathU; break;
-		         case DISKOP_ITEM_INSTR:   FReq_FileName = insTmpFName; FReq_CurPathU = FReq_InsCurPathU; break;
-		         case DISKOP_ITEM_SAMPLE:  FReq_FileName = smpTmpFName; FReq_CurPathU = FReq_SmpCurPathU; break;
-		         case DISKOP_ITEM_PATTERN: FReq_FileName = patTmpFName; FReq_CurPathU = FReq_PatCurPathU; break;
-		         case DISKOP_ITEM_TRACK:   FReq_FileName = trkTmpFName; FReq_CurPathU = FReq_TrkCurPathU; break;
+		default:
+		case DISKOP_ITEM_MODULE:
+		{
+			FReq_FileName = modTmpFName;
+
+			// FReq_ModCurPathU is always set at this point
+
+			FReq_CurPathU = FReq_ModCurPathU;
+			if (FReq_CurPathU != NULL)
+				UNICHAR_CHDIR(FReq_CurPathU);
+		}
+		break;
+
+		case DISKOP_ITEM_INSTR:
+		{
+			FReq_FileName = insTmpFName;
+
+			if (!insPathSet)
+			{
+				UNICHAR_STRCPY(FReq_InsCurPathU, FReq_CurPathU);
+				insPathSet = true;
+			}
+
+			FReq_CurPathU = FReq_InsCurPathU;
+			if (FReq_CurPathU != NULL)
+				UNICHAR_CHDIR(FReq_CurPathU);
+		}
+		break;
+
+		case DISKOP_ITEM_SAMPLE:
+		{
+			FReq_FileName = smpTmpFName;
+
+			if (!smpPathSet)
+			{
+				UNICHAR_STRCPY(FReq_SmpCurPathU, FReq_CurPathU);
+				smpPathSet = true;
+			}
+
+			FReq_CurPathU = FReq_SmpCurPathU;
+			if (FReq_CurPathU != NULL)
+				UNICHAR_CHDIR(FReq_CurPathU);
+		}
+		break;
+
+		case DISKOP_ITEM_PATTERN:
+		{
+			FReq_FileName = patTmpFName;
+
+			if (!patPathSet)
+			{
+				UNICHAR_STRCPY(FReq_PatCurPathU, FReq_CurPathU);
+				patPathSet = true;
+			}
+
+			FReq_CurPathU = FReq_PatCurPathU;
+			if (FReq_CurPathU != NULL)
+				UNICHAR_CHDIR(FReq_CurPathU);
+		}
+		break;
+
+		case DISKOP_ITEM_TRACK:
+		{
+			FReq_FileName = trkTmpFName;
+
+			if (!trkPathSet)
+			{
+				UNICHAR_STRCPY(FReq_TrkCurPathU, FReq_CurPathU);
+				trkPathSet = true;
+			}
+
+			FReq_CurPathU = FReq_TrkCurPathU;
+			if (FReq_CurPathU != NULL)
+				UNICHAR_CHDIR(FReq_CurPathU);
+		}
+		break;
 	}
 
 	pathLen = (int32_t)UNICHAR_STRLEN(FReq_CurPathU);
@@ -2089,16 +2185,12 @@ static void setDiskOpItem(uint8_t item)
 		memset(FReq_CurPathU, 0, (PATH_MAX + 2) * sizeof (UNICHAR));
 		UNICHAR_STRCPY(FReq_CurPathU, FReq_ModCurPathU);
 	}
-	else
-	{
-		UNICHAR_CHDIR(FReq_CurPathU);
-	}
 
 	textBoxes[TB_DISKOP_FILENAME].textPtr = FReq_FileName;
 
 	FReq_ShowAllFiles = false;
 
-	if (editor.ui.diskOpShown)
+	if (ui.diskOpShown)
 	{
 		editor.diskOpReadDir = true;
 
@@ -2175,12 +2267,34 @@ static void drawDiskOpScreen(void)
 
 void showDiskOpScreen(void)
 {
-	if (editor.ui.extended)
+	// if first time opening Disk Op., set initial directory
+	if (firstTimeOpeningDiskOp)
+	{
+		assert(FReq_ModCurPathU != NULL);
+
+		// first test if we can change the dir to the one stored in the config (if present)
+		if (UNICHAR_STRLEN(FReq_ModCurPathU) == 0 || UNICHAR_CHDIR(FReq_ModCurPathU) != 0)
+		{
+			// nope, couldn't do that, set Disk Op. path to user/home directory
+#ifdef _WIN32
+			SHGetFolderPathW(NULL, CSIDL_PROFILE, NULL, 0, FReq_ModCurPathU);
+#else
+			if (getenv("HOME") != NULL)
+				UNICHAR_STRCPY(FReq_ModCurPathU, getenv("HOME"));
+#endif
+			UNICHAR_CHDIR(FReq_ModCurPathU);
+		}
+
+		UNICHAR_GETCWD(FReq_ModCurPathU, PATH_MAX);
+		firstTimeOpeningDiskOp = false;
+	}
+
+	if (ui.extended)
 		exitPatternEditorExtended();
 
 	hideTopScreen();
-	editor.ui.diskOpShown = true;
-	editor.ui.scopesShown = false;
+	ui.diskOpShown = true;
+	ui.scopesShown = false;
 
 	showTopRightMainScreen();
 	drawDiskOpScreen();
@@ -2189,7 +2303,7 @@ void showDiskOpScreen(void)
 	if (editor.diskOpReadOnOpen)
 	{
 		editor.diskOpReadOnOpen = false;
-		startDiskOpFillThread();
+		diskOp_StartDirReadThread();
 	}
 }
 
@@ -2222,19 +2336,19 @@ void hideDiskOpScreen(void)
 	hideRadioButtonGroup(RB_GROUP_DISKOP_PAT_SAVEAS);
 	hideRadioButtonGroup(RB_GROUP_DISKOP_TRK_SAVEAS);
 
-	editor.ui.diskOpShown = false;
+	ui.diskOpShown = false;
 }
 
 void exitDiskOpScreen(void)
 {
 	hideDiskOpScreen();
-	editor.ui.oldTopLeftScreen = 0; // disk op. ignores previously opened top screens
+	ui.oldTopLeftScreen = 0; // disk op. ignores previously opened top screens
 	showTopScreen(true);
 }
 
 void toggleDiskOpScreen(void)
 {
-	if (editor.ui.diskOpShown)
+	if (ui.diskOpShown)
 		exitDiskOpScreen();
 	else
 		showDiskOpScreen();
