@@ -13,6 +13,8 @@
 #include "ft2_pattern_ed.h"
 #include "ft2_sysreqs.h"
 #include "ft2_textboxes.h"
+#include "ft2_tables.h"
+#include "ft2_structs.h"
 
 enum
 {
@@ -39,30 +41,6 @@ static tonTyp trackCopyBuff[MAX_PATT_LEN];
 
 static const int8_t tickArr[16] = { 16, 8, 0, 4, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 1 };
 
-static const SDL_Keycode key2VolTab[] = 
-{
-	SDLK_0, SDLK_1, SDLK_2, SDLK_3, SDLK_4, SDLK_MINUS, SDLK_PLUS, SDLK_d,
-	SDLK_u, SDLK_s, SDLK_v, SDLK_p, SDLK_l, SDLK_r, SDLK_m
-};
-#define KEY2VOL_ENTRIES (signed)(sizeof (key2VolTab) / sizeof (SDL_Keycode))
-
-static const SDL_Keycode key2EfxTab[] = 
-{
-	SDLK_0, SDLK_1, SDLK_2, SDLK_3, SDLK_4, SDLK_5, SDLK_6, SDLK_7,
-	SDLK_8, SDLK_9, SDLK_a, SDLK_b, SDLK_c, SDLK_d, SDLK_e, SDLK_f,
-	SDLK_g, SDLK_h, SDLK_i, SDLK_j, SDLK_k, SDLK_l, SDLK_m, SDLK_n,
-	SDLK_o, SDLK_p, SDLK_q, SDLK_r, SDLK_s, SDLK_t, SDLK_u, SDLK_v,
-	SDLK_w, SDLK_x, SDLK_y, SDLK_z
-};
-#define KEY2EFX_ENTRIES (signed)(sizeof (key2EfxTab) / sizeof (SDL_Keycode))
-
-static const SDL_Keycode key2HexTab[] = 
-{
-	SDLK_0, SDLK_1, SDLK_2, SDLK_3, SDLK_4, SDLK_5, SDLK_6, SDLK_7,
-	SDLK_8, SDLK_9, SDLK_a, SDLK_b, SDLK_c, SDLK_d, SDLK_e, SDLK_f
-};
-#define KEY2HEX_ENTRIES (signed)(sizeof (key2HexTab) / sizeof (SDL_Keycode))
-
 void recordNote(uint8_t note, int8_t vol);
 
 // when the cursor is at the note slot
@@ -93,7 +71,7 @@ static bool testEditKeys(SDL_Scancode scancode, SDL_Keycode keycode)
 	uint16_t pattLen;
 	tonTyp *ton;
 
-	if (editor.cursor.object == CURSOR_NOTE)
+	if (cursor.object == CURSOR_NOTE)
 	{
 		// the edit cursor is at the note slot
 
@@ -111,7 +89,7 @@ static bool testEditKeys(SDL_Scancode scancode, SDL_Keycode keycode)
 
 	// convert key to slot data
 
-	if (editor.cursor.object == CURSOR_VOL1)
+	if (cursor.object == CURSOR_VOL1)
 	{
 		// volume column effect type (mixed keys)
 
@@ -124,7 +102,7 @@ static bool testEditKeys(SDL_Scancode scancode, SDL_Keycode keycode)
 		if (i == KEY2VOL_ENTRIES)
 			i = -1; // invalid key for slot
 	}
-	else if (editor.cursor.object == CURSOR_EFX0)
+	else if (cursor.object == CURSOR_EFX0)
 	{
 		// effect type (mixed keys)
 
@@ -156,8 +134,8 @@ static bool testEditKeys(SDL_Scancode scancode, SDL_Keycode keycode)
 
 	// insert slot data
 
-	ton = &patt[editor.editPattern][(editor.pattPos * MAX_VOICES) + editor.cursor.ch];
-	switch (editor.cursor.object)
+	ton = &patt[editor.editPattern][(editor.pattPos * MAX_VOICES) + cursor.ch];
+	switch (cursor.object)
 	{
 		case CURSOR_INST1:
 		{
@@ -257,7 +235,7 @@ static bool testEditKeys(SDL_Scancode scancode, SDL_Keycode keycode)
 	if (i == 0) // if we inserted a zero, check if pattern is empty, for killing
 		killPatternIfUnused(editor.editPattern);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	return true;
 }
 
@@ -375,7 +353,7 @@ void recordNote(uint8_t note, int8_t vol)
 		}
 		else
 		{
-			c = editor.cursor.ch;
+			c = cursor.ch;
 		}
 
 		for (i = 0; i < song.antChn; i++)
@@ -418,7 +396,7 @@ void recordNote(uint8_t note, int8_t vol)
 		}
 		else
 		{
-			c = editor.cursor.ch;
+			c = cursor.ch;
 		}
 
 		for (i = 0; i < song.antChn; i++)
@@ -477,7 +455,7 @@ void recordNote(uint8_t note, int8_t vol)
 					}
 				}
 
-				editor.ui.updatePatternEditor = true;
+				ui.updatePatternEditor = true;
 				setSongModifiedFlag();
 			}
 		}
@@ -548,7 +526,7 @@ void recordNote(uint8_t note, int8_t vol)
 					}
 				}
 
-				editor.ui.updatePatternEditor = true;
+				ui.updatePatternEditor = true;
 				setSongModifiedFlag();
 			}
 		}
@@ -570,7 +548,7 @@ bool handleEditKeys(SDL_Keycode keycode, SDL_Scancode scancode)
 		if (patt[editor.editPattern] == NULL)
 			return true;
 
-		note = &patt[editor.editPattern][(editor.pattPos * MAX_VOICES) + editor.cursor.ch];
+		note = &patt[editor.editPattern][(editor.pattPos * MAX_VOICES) + cursor.ch];
 
 		if (keyb.leftShiftPressed)
 		{
@@ -592,7 +570,7 @@ bool handleEditKeys(SDL_Keycode keycode, SDL_Scancode scancode)
 		}
 		else
 		{
-			if (editor.cursor.object == CURSOR_VOL1 || editor.cursor.object == CURSOR_VOL2)
+			if (cursor.object == CURSOR_VOL1 || cursor.object == CURSOR_VOL2)
 			{
 				// delete volume column
 				note->vol = 0;
@@ -612,7 +590,7 @@ bool handleEditKeys(SDL_Keycode keycode, SDL_Scancode scancode)
 		if (playMode == PLAYMODE_EDIT && pattLen >= 1)
 			setPos(-1, (editor.pattPos + editor.ID_Add) % pattLen, true);
 
-		editor.ui.updatePatternEditor = true;
+		ui.updatePatternEditor = true;
 		setSongModifiedFlag();
 
 		return true;
@@ -638,12 +616,12 @@ void writeToMacroSlot(uint8_t slot)
 
 	if (patt[editor.editPattern] != NULL)
 	{
-		note = &patt[editor.editPattern][(editor.pattPos * MAX_VOICES) + editor.cursor.ch];
+		note = &patt[editor.editPattern][(editor.pattPos * MAX_VOICES) + cursor.ch];
 		writeVol = note->vol;
 		writeEff = (note->effTyp << 8) | note->eff;
 	}
 
-	if (editor.cursor.object == CURSOR_VOL1 || editor.cursor.object == CURSOR_VOL2)
+	if (cursor.object == CURSOR_VOL1 || cursor.object == CURSOR_VOL2)
 		config.volMacro[slot] = writeVol;
 	else
 		config.comMacro[slot] = writeEff;
@@ -662,9 +640,9 @@ void writeFromMacroSlot(uint8_t slot)
 		return;
 
 	pattLen = pattLens[editor.editPattern];
-	note = &patt[editor.editPattern][(editor.pattPos * MAX_VOICES) + editor.cursor.ch];
+	note = &patt[editor.editPattern][(editor.pattPos * MAX_VOICES) + cursor.ch];
 
-	if (editor.cursor.object == CURSOR_VOL1 || editor.cursor.object == CURSOR_VOL2)
+	if (cursor.object == CURSOR_VOL1 || cursor.object == CURSOR_VOL2)
 	{
 		note->vol = (uint8_t)config.volMacro[slot];
 	}
@@ -689,7 +667,7 @@ void writeFromMacroSlot(uint8_t slot)
 
 	killPatternIfUnused(editor.editPattern);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -714,14 +692,14 @@ void insertPatternNote(void)
 	if (pattLen > 1)
 	{
 		for (int32_t i = pattLen-2; i >= pattPos; i--)
-			pattPtr[((i + 1) * MAX_VOICES) + editor.cursor.ch] = pattPtr[(i * MAX_VOICES) + editor.cursor.ch];
+			pattPtr[((i + 1) * MAX_VOICES) + cursor.ch] = pattPtr[(i * MAX_VOICES) + cursor.ch];
 	}
 
-	memset(&pattPtr[(pattPos * MAX_VOICES) + editor.cursor.ch], 0, sizeof (tonTyp));
+	memset(&pattPtr[(pattPos * MAX_VOICES) + cursor.ch], 0, sizeof (tonTyp));
 
 	killPatternIfUnused(nr);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -757,7 +735,7 @@ void insertPatternLine(void)
 		killPatternIfUnused(nr);
 	}
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -783,9 +761,9 @@ void deletePatternNote(void)
 			editor.pattPos = song.pattPos = pattPos;
 
 			for (int32_t i = pattPos; i < pattLen-1; i++)
-				pattPtr[(i * MAX_VOICES) + editor.cursor.ch] = pattPtr[((i + 1) * MAX_VOICES) + editor.cursor.ch];
+				pattPtr[(i * MAX_VOICES) + cursor.ch] = pattPtr[((i + 1) * MAX_VOICES) + cursor.ch];
 
-			memset(&pattPtr[((pattLen - 1) * MAX_VOICES) + editor.cursor.ch], 0, sizeof (tonTyp));
+			memset(&pattPtr[((pattLen - 1) * MAX_VOICES) + cursor.ch], 0, sizeof (tonTyp));
 		}
 	}
 	else
@@ -799,7 +777,7 @@ void deletePatternNote(void)
 
 	killPatternIfUnused(nr);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -847,7 +825,7 @@ void deletePatternLine(void)
 
 	killPatternIfUnused(nr);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -868,7 +846,7 @@ static void countOverflowingNotes(uint8_t currInsOnly, uint8_t transpMode, int8_
 			if (pattPtr == NULL)
 				return; // empty pattern
 
-			pattPtr += editor.cursor.ch;
+			pattPtr += cursor.ch;
 
 			pattLen = pattLens[editor.editPattern];
 			for (row = 0; row < pattLen; row++)
@@ -999,7 +977,7 @@ void doTranspose(void)
 			if (pattPtr == NULL)
 				return; // empty pattern
 
-			pattPtr += editor.cursor.ch;
+			pattPtr += cursor.ch;
 
 			pattLen = pattLens[editor.editPattern];
 			for (row = 0; row < pattLen; row++)
@@ -1117,7 +1095,7 @@ void doTranspose(void)
 		default: break;
 	}
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1424,19 +1402,19 @@ void cutTrack(void)
 	{
 		memset(trackCopyBuff, 0, MAX_PATT_LEN * sizeof (tonTyp));
 		for (i = 0; i < pattLen; i++)
-			copyNote(&pattPtr[(i * MAX_VOICES) + editor.cursor.ch], &trackCopyBuff[i]);
+			copyNote(&pattPtr[(i * MAX_VOICES) + cursor.ch], &trackCopyBuff[i]);
 
 		trkBufLen = pattLen;
 	}
 
 	pauseMusic();
 	for (i = 0; i < pattLen; i++)
-		pasteNote(&clearNote, &pattPtr[(i * MAX_VOICES) + editor.cursor.ch]);
+		pasteNote(&clearNote, &pattPtr[(i * MAX_VOICES) + cursor.ch]);
 	resumeMusic();
 
 	killPatternIfUnused(editor.editPattern);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1453,7 +1431,7 @@ void copyTrack(void)
 
 	memset(trackCopyBuff, 0, MAX_PATT_LEN * sizeof (tonTyp));
 	for (i = 0; i < pattLen; i++)
-		copyNote(&pattPtr[(i * MAX_VOICES) + editor.cursor.ch], &trackCopyBuff[i]);
+		copyNote(&pattPtr[(i * MAX_VOICES) + cursor.ch], &trackCopyBuff[i]);
 
 	trkBufLen = pattLen;
 }
@@ -1471,12 +1449,12 @@ void pasteTrack(void)
 
 	pauseMusic();
 	for (i = 0; i < pattLen; i++)
-		pasteNote(&trackCopyBuff[i], &pattPtr[(i * MAX_VOICES) + editor.cursor.ch]);
+		pasteNote(&trackCopyBuff[i], &pattPtr[(i * MAX_VOICES) + cursor.ch]);
 	resumeMusic();
 
 	killPatternIfUnused(editor.editPattern);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1513,7 +1491,7 @@ void cutPattern(void)
 
 	killPatternIfUnused(editor.editPattern);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1537,7 +1515,7 @@ void copyPattern(void)
 
 	ptnBufLen = pattLen;
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 }
 
 void pastePattern(void)
@@ -1570,7 +1548,7 @@ void pastePattern(void)
 
 	killPatternIfUnused(editor.editPattern);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1613,7 +1591,7 @@ void cutBlock(void)
 
 	killPatternIfUnused(editor.editPattern);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1646,7 +1624,7 @@ void copyBlock(void)
 
 void pasteBlock(void)
 {
-	uint16_t xpos, ypos, j, k, pattLen;
+	int32_t xpos, ypos, j, k, pattLen;
 	tonTyp *pattPtr;
 
 	if (!blockCopied || !allocatePattern(editor.editPattern))
@@ -1654,7 +1632,7 @@ void pasteBlock(void)
 
 	pattLen = pattLens[editor.editPattern];
 
-	xpos = editor.cursor.ch;
+	xpos = cursor.ch;
 	ypos = editor.pattPos;
 
 	j = markXSize;
@@ -1668,9 +1646,9 @@ void pasteBlock(void)
 	pattPtr = patt[editor.editPattern];
 
 	pauseMusic();
-	for (uint16_t x = xpos; x <= xpos+j; x++)
+	for (int32_t x = xpos; x <= xpos+j; x++)
 	{
-		for (uint16_t y = ypos; y < ypos+k; y++)
+		for (int32_t y = ypos; y < ypos+k; y++)
 		{
 			assert(x < song.antChn && y < pattLen);
 			pasteNote(&blkCopyBuff[((y - ypos) * MAX_VOICES) + (x - xpos)], &pattPtr[(y * MAX_VOICES) + x]);
@@ -1680,7 +1658,7 @@ void pasteBlock(void)
 
 	killPatternIfUnused(editor.editPattern);
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1725,7 +1703,7 @@ void remapBlock(void)
 	             editor.srcInstr, editor.curInstr);
 	resumeMusic();
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1736,12 +1714,12 @@ void remapTrack(void)
 
 	pauseMusic();
 	remapInstrXY(editor.editPattern,
-	             editor.cursor.ch, 0,
-	             editor.cursor.ch, pattLens[editor.editPattern] - 1,
+	             cursor.ch, 0,
+	             cursor.ch, pattLens[editor.editPattern] - 1,
 	             editor.srcInstr, editor.curInstr);
 	resumeMusic();
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1753,11 +1731,11 @@ void remapPattern(void)
 	pauseMusic();
 	remapInstrXY(editor.editPattern,
 	             0, 0,
-	             song.antChn - 1, pattLens[editor.editPattern] - 1,
+	             (uint16_t)(song.antChn - 1), pattLens[editor.editPattern] - 1,
 	             editor.srcInstr, editor.curInstr);
 	resumeMusic();
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1769,18 +1747,18 @@ void remapSong(void)
 		return;
 
 	pauseMusic();
-	for (uint16_t i = 0; i < MAX_PATTERNS; i++)
+	for (int32_t i = 0; i < MAX_PATTERNS; i++)
 	{
 		pattNr = (uint8_t)i;
 
 		remapInstrXY(pattNr,
 		             0, 0,
-		             song.antChn - 1, pattLens[pattNr] - 1,
+		             (uint16_t)(song.antChn - 1), pattLens[pattNr] - 1,
 		             editor.srcInstr, editor.curInstr);
 	}
 	resumeMusic();
 
-	editor.ui.updatePatternEditor = true;
+	ui.updatePatternEditor = true;
 	setSongModifiedFlag();
 }
 
@@ -1905,7 +1883,7 @@ void scaleFadeVolumeTrack(void)
 	pauseMusic();
 	for (uint16_t row = 0; row < pattLen; row++)
 	{
-		scaleNote(editor.editPattern, editor.cursor.ch, row, dVol);
+		scaleNote(editor.editPattern, cursor.ch, row, dVol);
 		dVol += dIPy;
 	}
 	resumeMusic();
