@@ -14,12 +14,14 @@
 #include "ft2_gui.h"
 #include "ft2_scopes.h"
 #include "ft2_sample_ed.h"
-#include "ft2_gfxdata.h"
 #include "ft2_mouse.h"
 #include "ft2_video.h"
 #include "ft2_sample_loader.h"
 #include "ft2_diskop.h"
 #include "ft2_module_loader.h"
+#include "ft2_tables.h"
+#include "ft2_bmp.h"
+#include "ft2_structs.h"
 
 #ifdef _MSC_VER
 #pragma pack(push)
@@ -108,7 +110,7 @@ static int32_t lastMouseX, lastMouseY, saveMouseX, saveMouseY;
 static uint16_t saveInstrNr;
 static SDL_Thread *thread;
 
-extern int16_t *note2Period; // ft2_replayer.c
+extern const uint16_t *note2Period; // ft2_replayer.c
 
 void updateInstEditor(void);
 void updateNewInstrument(void);
@@ -126,7 +128,7 @@ static int32_t SDLCALL copyInstrThread(void *ptr)
 	bool error;
 	int8_t *p;
 	int16_t destIns, sourceIns;
-
+	sampleTyp *src, *dst;
 	(void)ptr;
 
 	error = false;
@@ -143,16 +145,24 @@ static int32_t SDLCALL copyInstrThread(void *ptr)
 		if (allocateInstr(destIns))
 		{
 			memcpy(instr[destIns], instr[sourceIns], sizeof (instrTyp));
+
 			for (int16_t i = 0; i < MAX_SMP_PER_INST; i++)
 			{
-				instr[destIns]->samp[i].pek = NULL;
-				if (instr[sourceIns]->samp[i].pek != NULL)
+				src = &instr[sourceIns]->samp[i];
+				dst = &instr[destIns]->samp[i];
+
+				dst->origPek = NULL;
+				dst->pek = NULL;
+
+				if (src->origPek != NULL)
 				{
-					p = (int8_t *)malloc(instr[sourceIns]->samp[i].len + LOOP_FIX_LEN);
+					p = (int8_t *)malloc(src->len + LOOP_FIX_LEN);
 					if (p != NULL)
 					{
-						memcpy(p, instr[sourceIns]->samp[i].pek, instr[sourceIns]->samp[i].len + LOOP_FIX_LEN);
-						instr[destIns]->samp[i].pek = p;
+						dst->origPek = p;
+						dst->pek = dst->origPek + SMP_DAT_OFFSET;
+
+						memcpy(dst->origPek, src->origPek, src->len + LOOP_FIX_LEN);
 					}
 					else error = true;
 				}
@@ -218,63 +228,60 @@ void xchgInstr(void) // dstInstr <-> srcInstr
 
 static void drawMIDICh(void)
 {
-	char str[8];
 	instrTyp *ins = getCurDispInstr();
-
 	assert(ins->midiChannel <= 15);
-	uint8_t disp = ins->midiChannel + 1;
-	sprintf(str, "%02d", disp);
-	textOutFixed(156, 132, PAL_FORGRND, PAL_DESKTOP, str);
+	const uint8_t val = ins->midiChannel + 1;
+	textOutFixed(156, 132, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[val]);
 }
 
 static void drawMIDIPrg(void)
 {
-	char str[8];
 	instrTyp *ins = getCurDispInstr();
-
 	assert(ins->midiProgram <= 127);
-	sprintf(str, "%03d", ins->midiProgram);
-	textOutFixed(149, 146, PAL_FORGRND, PAL_DESKTOP, str);
+	textOutFixed(149, 146, PAL_FORGRND, PAL_DESKTOP, dec3StrTab[ins->midiProgram]);
 }
 
 static void drawMIDIBend(void)
 {
-	char str[8];
 	instrTyp *ins = getCurDispInstr();
-
 	assert(ins->midiBend <= 36);
-	sprintf(str, "%02d", ins->midiBend);
-	textOutFixed(156, 160, PAL_FORGRND, PAL_DESKTOP, str);
+	textOutFixed(156, 160, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[ins->midiBend]);
 }
 
 void midiChDown(void)
 {
-	scrollBarScrollLeft(SB_INST_EXT_MIDI_CH, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_EXT_MIDI_CH, 1);
 }
 
 void midiChUp(void)
 {
-	scrollBarScrollRight(SB_INST_EXT_MIDI_CH, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_EXT_MIDI_CH, 1);
 }
 
 void midiPrgDown(void)
 {
-	scrollBarScrollLeft(SB_INST_EXT_MIDI_PRG, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_EXT_MIDI_PRG, 1);
 }
 
 void midiPrgUp(void)
 {
-	scrollBarScrollRight(SB_INST_EXT_MIDI_PRG, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_EXT_MIDI_PRG, 1);
 }
 
 void midiBendDown(void)
 {
-	scrollBarScrollLeft(SB_INST_EXT_MIDI_BEND, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_EXT_MIDI_BEND, 1);
 }
 
 void midiBendUp(void)
 {
-	scrollBarScrollRight(SB_INST_EXT_MIDI_BEND, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_EXT_MIDI_BEND, 1);
 }
 
 void sbMidiChPos(uint32_t pos)
@@ -333,15 +340,15 @@ void sbMidiBendPos(uint32_t pos)
 
 void updateNewSample(void)
 {
-	if (editor.ui.instrSwitcherShown)
+	if (ui.instrSwitcherShown)
 		updateInstrumentSwitcher();
 
 	updateSampleEditorSample();
 
-	if (editor.ui.sampleEditorShown)
+	if (ui.sampleEditorShown)
 		updateSampleEditor();
 
-	if (editor.ui.instEditorShown || editor.ui.instEditorExtShown)
+	if (ui.instEditorShown || ui.instEditorExtShown)
 		updateInstEditor();
 }
 
@@ -349,7 +356,7 @@ void updateNewInstrument(void)
 {
 	updateTextBoxPointers();
 
-	if (editor.ui.instrSwitcherShown)
+	if (ui.instrSwitcherShown)
 		updateInstrumentSwitcher();
 
 	editor.currVolEnvPoint = 0;
@@ -357,68 +364,56 @@ void updateNewInstrument(void)
 
 	updateSampleEditorSample();
 
-	if (editor.ui.sampleEditorShown)
+	if (ui.sampleEditorShown)
 		updateSampleEditor();
 
-	if (editor.ui.instEditorShown || editor.ui.instEditorExtShown)
+	if (ui.instEditorShown || ui.instEditorExtShown)
 		updateInstEditor();
 
-	if (editor.ui.advEditShown)
+	if (ui.advEditShown)
 		updateAdvEdit();
 }
 
 static void drawVolEnvSus(void)
 {
-	char str[8];
 	instrTyp *ins = getCurDispInstr();
-
-	sprintf(str, "%02d", ins->envVSust);
-	textOutFixed(382, 206, PAL_FORGRND, PAL_DESKTOP, str);
+	assert(ins->envVSust < 100);
+	textOutFixed(382, 206, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[ins->envVSust]);
 }
 
 static void drawVolEnvRepS(void)
 {
-	char str[8];
 	instrTyp *ins = getCurDispInstr();
-
-	sprintf(str, "%02d", ins->envVRepS);
-	textOutFixed(382, 234, PAL_FORGRND, PAL_DESKTOP, str);
+	assert(ins->envVRepS < 100);
+	textOutFixed(382, 233, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[ins->envVRepS]);
 }
 
 static void drawVolEnvRepE(void)
 {
-	char str[8];
 	instrTyp *ins = getCurDispInstr();
-
-	sprintf(str, "%02d", ins->envVRepE);
-	textOutFixed(382, 247, PAL_FORGRND, PAL_DESKTOP, str);
+	assert(ins->envVRepE < 100);
+	textOutFixed(382, 247, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[ins->envVRepE]);
 }
 
 static void drawPanEnvSus(void)
 {
-	char str[8];
 	instrTyp *ins = getCurDispInstr();
-
-	sprintf(str, "%02d", ins->envPSust);
-	textOutFixed(382, 294, PAL_FORGRND, PAL_DESKTOP, str);
+	assert(ins->envPSust < 100);
+	textOutFixed(382, 293, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[ins->envPSust]);
 }
 
 static void drawPanEnvRepS(void)
 {
-	char str[8];
 	instrTyp *ins = getCurDispInstr();
-
-	sprintf(str, "%02d", ins->envPRepS);
-	textOutFixed(382, 321, PAL_FORGRND, PAL_DESKTOP, str);
+	assert(ins->envPRepS < 100);
+	textOutFixed(382, 320, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[ins->envPRepS]);
 }
 
 static void drawPanEnvRepE(void)
 {
-	char str[8];
 	instrTyp *ins = getCurDispInstr();
-
-	sprintf(str, "%02d", ins->envPRepE);
-	textOutFixed(382, 335, PAL_FORGRND, PAL_DESKTOP, str);
+	assert(ins->envPRepE < 100);
+	textOutFixed(382, 334, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[ins->envPRepE]);
 }
 
 static void drawVolume(void)
@@ -430,7 +425,7 @@ static void drawVolume(void)
 	else
 		s = &instr[editor.curInstr]->samp[editor.curSmp];
 
-	hexOutBg(505, 178, PAL_FORGRND, PAL_DESKTOP, s->vol, 2);
+	hexOutBg(505, 177, PAL_FORGRND, PAL_DESKTOP, s->vol, 2);
 }
 
 static void drawPanning(void)
@@ -442,7 +437,7 @@ static void drawPanning(void)
 	else
 		s = &instr[editor.curInstr]->samp[editor.curSmp];
 
-	hexOutBg(505, 192, PAL_FORGRND, PAL_DESKTOP, s->pan, 2);
+	hexOutBg(505, 191, PAL_FORGRND, PAL_DESKTOP, s->pan, 2);
 }
 
 static void drawFineTune(void)
@@ -545,7 +540,7 @@ static void drawRelTone(void)
 
 static void setStdVolEnvelope(instrTyp *ins, uint8_t num)
 {
-	if (editor.curInstr == 0)
+	if (editor.curInstr == 0 || ins == NULL)
 		return;
 
 	pauseMusic();
@@ -568,7 +563,7 @@ static void setStdVolEnvelope(instrTyp *ins, uint8_t num)
 
 static void setStdPanEnvelope(instrTyp *ins, uint8_t num)
 {
-	if (editor.curInstr == 0)
+	if (editor.curInstr == 0 || ins == NULL)
 		return;
 
 	pauseMusic();
@@ -652,73 +647,73 @@ static void setOrStorePanEnvPreset(uint8_t num)
 
 void volPreDef1(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStoreVolEnvPreset(1 - 1);
 }
 
 void volPreDef2(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStoreVolEnvPreset(2 - 1);
 }
 
 void volPreDef3(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStoreVolEnvPreset(3 - 1);
 }
 
 void volPreDef4(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStoreVolEnvPreset(4 - 1);
 }
 
 void volPreDef5(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStoreVolEnvPreset(5 - 1);
 }
 
 void volPreDef6(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStoreVolEnvPreset(6 - 1);
 }
 
 void panPreDef1(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStorePanEnvPreset(1 - 1);
 }
 
 void panPreDef2(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStorePanEnvPreset(2 - 1);
 }
 
 void panPreDef3(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStorePanEnvPreset(3 - 1);
 }
 
 void panPreDef4(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStorePanEnvPreset(4 - 1);
 }
 
 void panPreDef5(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStorePanEnvPreset(5 - 1);
 }
 
 void panPreDef6(void)
 {
-	if (editor.curInstr > 0)
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
 		setOrStorePanEnvPreset(6 - 1);
 }
 
@@ -792,9 +787,11 @@ void volEnvAdd(void)
 {
 	int16_t i, ant;
 	instrTyp *ins = instr[editor.curInstr];
+	if (editor.curInstr == 0 || ins == NULL)
+		return;
 
 	ant = ins->envVPAnt;
-	if (ins == NULL || editor.curInstr == 0 || ant >= 12)
+	if (ant >= 12)
 		return;
 
 	i = (int16_t)editor.currVolEnvPoint;
@@ -981,9 +978,11 @@ void panEnvAdd(void)
 {
 	int16_t i, ant;
 	instrTyp *ins = instr[editor.curInstr];
+	if (ins == NULL || editor.curInstr == 0)
+		return;
 
 	ant = ins->envPPAnt;
-	if (ins == NULL || editor.curInstr == 0 || ant >= 12)
+	if (ant >= 12)
 		return;
 
 	i = (int16_t)editor.currPanEnvPoint;
@@ -1168,72 +1167,86 @@ void panEnvRepEDown(void)
 
 void volDown(void)
 {
-	scrollBarScrollLeft(SB_INST_VOL, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_VOL, 1);
 }
 
 void volUp(void)
 {
-	scrollBarScrollRight(SB_INST_VOL, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_VOL, 1);
 }
 
 void panDown(void)
 {
-	scrollBarScrollLeft(SB_INST_PAN, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_PAN, 1);
 }
 
 void panUp(void)
 {
-	scrollBarScrollRight(SB_INST_PAN, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_PAN, 1);
 }
 
 void ftuneDown(void)
 {
-	scrollBarScrollLeft(SB_INST_FTUNE, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_FTUNE, 1);
 }
 
 void ftuneUp(void)
 {
-	scrollBarScrollRight(SB_INST_FTUNE, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_FTUNE, 1);
 }
 
 void fadeoutDown(void)
 {
-	scrollBarScrollLeft(SB_INST_FADEOUT, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_FADEOUT, 1);
 }
 
 void fadeoutUp(void)
 {
-	scrollBarScrollRight(SB_INST_FADEOUT, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_FADEOUT, 1);
 }
 
 void vibSpeedDown(void)
 {
-	scrollBarScrollLeft(SB_INST_VIBSPEED, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_VIBSPEED, 1);
 }
 
 void vibSpeedUp(void)
 {
-	scrollBarScrollRight(SB_INST_VIBSPEED, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_VIBSPEED, 1);
 }
 
 void vibDepthDown(void)
 {
-	scrollBarScrollLeft(SB_INST_VIBDEPTH, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_VIBDEPTH, 1);
 }
 
 void vibDepthUp(void)
 {
-	scrollBarScrollRight(SB_INST_VIBDEPTH, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_VIBDEPTH, 1);
 }
 
 void vibSweepDown(void)
 {
-	scrollBarScrollLeft(SB_INST_VIBSWEEP, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollLeft(SB_INST_VIBSWEEP, 1);
 }
 
 void vibSweepUp(void)
 {
-	scrollBarScrollRight(SB_INST_VIBSWEEP, 1);
+	if (editor.curInstr != 0 && instr[editor.curInstr] != NULL)
+		scrollBarScrollRight(SB_INST_VIBSWEEP, 1);
 }
 
 void setVolumeScroll(uint32_t pos)
@@ -1517,7 +1530,7 @@ void cbPEnvLoop(void)
 	setSongModifiedFlag();
 }
 
-static void smallHexOutBg(uint16_t xPos, uint16_t yPos, uint8_t fgPalette, uint8_t bgPalette, uint8_t val)
+static void pinoaNumberOut(uint16_t xPos, uint16_t yPos, uint8_t fgPalette, uint8_t bgPalette, uint8_t val)
 {
 	const uint8_t *srcPtr;
 	uint32_t *dstPtr, fg, bg;
@@ -1527,7 +1540,7 @@ static void smallHexOutBg(uint16_t xPos, uint16_t yPos, uint8_t fgPalette, uint8
 	fg = video.palette[fgPalette];
 	bg = video.palette[bgPalette];
 	dstPtr = &video.frameBuffer[(yPos * SCREEN_W) + xPos];
-	srcPtr = &smallHexBitmap[val * 5];
+	srcPtr = &bmp.font8[val * 5];
 
 	for (uint32_t y = 0; y < 7; y++)
 	{
@@ -1545,27 +1558,27 @@ static void writePianoNumber(uint8_t note, uint8_t key, uint8_t octave)
 	uint16_t x;
 
 	number = 0;
-	if (instr[editor.curInstr] != NULL && editor.curInstr > 0)
+	if (instr[editor.curInstr] != NULL && editor.curInstr != 0)
 		number = instr[editor.curInstr]->ta[note];
 
 	x = keyDigitXPos[key] + (octave * 77);
 
 	if (keyIsBlackTab[key])
-		smallHexOutBg(x, 361, PAL_FORGRND, PAL_BCKGRND, number);
+		pinoaNumberOut(x, 361, PAL_FORGRND, PAL_BCKGRND, number);
 	else
-		smallHexOutBg(x, 385, PAL_BCKGRND, PAL_FORGRND, number);
+		pinoaNumberOut(x, 385, PAL_BCKGRND, PAL_FORGRND, number);
 }
 
 static void drawBlackPianoKey(uint8_t key, uint8_t octave, bool keyDown)
 {
 	uint16_t x = keyXPos[key] + (octave * 77);
-	blit(x, 351, &blackPianoKeysBitmap[keyDown * (7*27)], 7, 27);
+	blit(x, 351, &bmp.blackPianoKeys[keyDown * (7*27)], 7, 27);
 }
 
 static void drawWhitePianoKey(uint8_t key, uint8_t octave,  bool keyDown)
 {
 	uint16_t x = keyXPos[key] + (octave * 77);
-	blit(x, 351, &whitePianoKeysBitmap[(keyDown * (11*46*3)) + whiteKeysBmpOrder[key]], 11, 46);
+	blit(x, 351, &bmp.whitePianoKeys[(keyDown * (11*46*3)) + whiteKeysBmpOrder[key]], 11, 46);
 }
 
 void redrawPiano(void)
@@ -1575,8 +1588,8 @@ void redrawPiano(void)
 	memset(pianoKeyStatus, 0, sizeof (pianoKeyStatus));
 	for (uint8_t i = 0; i < 96; i++)
 	{
-		key = i % 12;
-		octave = i / 12;
+		key = noteTab1[i];
+		octave = noteTab2[i];
 
 		if (keyIsBlackTab[key])
 			drawBlackPianoKey(key, octave, false);
@@ -1590,11 +1603,13 @@ void redrawPiano(void)
 bool testPianoKeysMouseDown(bool mouseButtonDown)
 {
 	uint8_t key, note, octave;
-	int32_t mx, my;
-	instrTyp *ins;
+	int32_t mx, my, quotient, remainder;
 
-	if (!editor.ui.instEditorShown)
-		return false;
+	if (!ui.instEditorShown)
+		return false; // area not clicked
+
+	if (editor.curInstr == 0 || instr[editor.curInstr] == NULL)
+		return true; // area clicked, but don't do anything
 
 	mx = mouse.x;
 	my = mouse.y;
@@ -1612,199 +1627,98 @@ bool testPianoKeysMouseDown(bool mouseButtonDown)
 		mx = CLAMP(mx, 8, 623);
 	}
 
-	ins = instr[editor.curInstr];
-	if (ins == NULL)
-		return true;
-
 	mx -= 8;
+
+	quotient = mx / 77;
+	remainder = mx % 77;
+
 	if (my < 378)
 	{
 		// white keys and black keys (top)
 
-		octave = (uint8_t)(mx / 77);
-		mx %= 77; // width of all keys in one octave
+		octave = (uint8_t)(quotient);
+		mx = remainder; // width of all keys in one octave
 
 		// this is pretty disgusting...
 		     if (mx >= 69) key = 11;
 		else if (mx >= 62) key = 10;
-		else if (mx >= 58) key =  9;
-		else if (mx >= 51) key =  8;
-		else if (mx >= 47) key =  7;
-		else if (mx >= 40) key =  6;
-		else if (mx >= 33) key =  5;
-		else if (mx >= 25) key =  4;
-		else if (mx >= 18) key =  3;
-		else if (mx >= 14) key =  2;
-		else if (mx >=  7) key =  1;
-		else               key =  0;
-
-		note = (octave * 12) + key;
-		if (ins->ta[note] != editor.curSmp)
-		{
-			ins->ta[note] = editor.curSmp;
-			writePianoNumber(note, key, octave);
-			setSongModifiedFlag();
-		}
+		else if (mx >= 58) key = 9;
+		else if (mx >= 51) key = 8;
+		else if (mx >= 47) key = 7;
+		else if (mx >= 40) key = 6;
+		else if (mx >= 33) key = 5;
+		else if (mx >= 25) key = 4;
+		else if (mx >= 18) key = 3;
+		else if (mx >= 14) key = 2;
+		else if (mx >=  7) key = 1;
+		else               key = 0;
 	}
 	else
 	{
 		// white keys only (bottom)
+		const int32_t whiteKeyWidth = 11;
 
-		octave = (uint8_t)(mx / 77);
-		key = (uint8_t)(mx % 77) / 11;
+		octave = (uint8_t)(quotient);
+		key = whiteKeyIndex[remainder / whiteKeyWidth];
+	}
 
-		note = (octave * 12) + whiteKeyIndex[key];
-		if (ins->ta[note] != editor.curSmp)
-		{
-			ins->ta[note] = editor.curSmp;
-			writePianoNumber(note, note % 12, octave);
-			setSongModifiedFlag();
-		}
+	note = (octave * 12) + key;
+	if (instr[editor.curInstr]->ta[note] != editor.curSmp)
+	{
+		instr[editor.curInstr]->ta[note] = editor.curSmp;
+		writePianoNumber(note, key, octave);
+		setSongModifiedFlag();
 	}
 
 	return true;
 }
 
-static uint8_t getNote(uint8_t i) // returns 1..96
+void drawPiano(chSyncData_t *chSyncData)
 {
-	int8_t fineTune;
-	uint8_t note;
-	int32_t period, loPeriod, hiPeriod, tmpPeriod, tableIndex;
-	stmTyp *ch;
-
-	ch = &stm[i];
-
-	fineTune = (ch->fineTune >> 3) + 16;
-	hiPeriod = 8 * 12 * 16;
-	loPeriod = 0;
-	period = ch->finalPeriod;
-
-	for (i = 0; i < 8; i++)
-	{
-		tmpPeriod = (((loPeriod + hiPeriod) >> 1) & 0xFFFFFFF0) + fineTune;
-
-		tableIndex = tmpPeriod - 8;
-		if (tableIndex < 0) // added security check
-			tableIndex = 0;
-
-		if (period >= note2Period[tableIndex])
-			hiPeriod = tmpPeriod - fineTune;
-		else
-			loPeriod = tmpPeriod - fineTune;
-	}
-
-	if (loPeriod >= ((8*12*16) + 15) - 1) // FT2 bug: off-by-one error
-		loPeriod = (8*12*16) + 15;
-
-	note = (uint8_t)(((loPeriod + 8) >> 4) - ch->relTonNr) + 1;
-	return note;
-}
-
-void drawPiano(void) // draw piano in idle mode
-{
-	bool keyDown, newStatus[96];
-	uint8_t key, note, octave;
-	stmTyp *ch;
+	bool newStatus[96];
+	int32_t i, note;
 
 	memset(newStatus, 0, sizeof (newStatus));
 
 	// find active notes
 	if (editor.curInstr > 0)
 	{
-		for (uint8_t i = 0; i < song.antChn; i++)
+		if (chSyncData != NULL) // song is playing, use replayer channel state
 		{
-			ch = &stm[i];
-			if (ch->instrNr == editor.curInstr)
+			syncedChannel_t *c = chSyncData->channels;
+			for (i = 0; i < song.antChn; i++, c++)
 			{
-				note = getNote(i);
-				if (ch->envSustainActive)
-					newStatus[(note - 1) % 96] = true;
+				if (c->instrNr == editor.curInstr && c->envSustainActive)
+				{
+					note = getPianoKey(c->finalPeriod, c->fineTune, c->relTonNr);
+					if (note >= 0 && note <= 95)
+						newStatus[note] = true;
+				}
+			}
+		}
+		else // song is not playing (jamming from keyboard/MIDI)
+		{
+			stmTyp *c = stm;
+			for (i = 0; i < song.antChn; i++, c++)
+			{
+				if (c->instrNr == editor.curInstr && c->envSustainActive)
+				{
+					note = getPianoKey(c->finalPeriod, c->fineTune, c->relTonNr);
+					if (note >= 0 && note <= 95)
+						newStatus[note] = true;
+				}
 			}
 		}
 	}
 
 	// draw keys
-	for (uint8_t i = 0; i < 96; i++)
+	for (i = 0; i < 96; i++)
 	{
-		keyDown = newStatus[i];
+		const bool keyDown = newStatus[i];
 		if (pianoKeyStatus[i] ^ keyDown)
 		{
-			key = i % 12;
-			octave = i / 12;
-
-			if (keyIsBlackTab[key])
-				drawBlackPianoKey(key, octave, keyDown);
-			else
-				drawWhitePianoKey(key, octave, keyDown);
-
-			pianoKeyStatus[i] = keyDown;
-		}
-	}
-}
-
-static uint8_t getNoteReplayer(syncedChannel_t *ch) // returns 1..96
-{
-	int8_t fineTune;
-	uint8_t note;
-	int32_t period, loPeriod, hiPeriod, tmpPeriod, tableIndex;
-
-	fineTune = (ch->fineTune >> 3) + 16;
-	hiPeriod = 8 * 12 * 16;
-	loPeriod = 0;
-	period = ch->finalPeriod;
-
-	for (uint8_t i = 0; i < 8; i++)
-	{
-		tmpPeriod = (((loPeriod + hiPeriod) >> 1) & 0xFFFFFFF0) + fineTune;
-
-		tableIndex = tmpPeriod - 8;
-		if (tableIndex < 0) // added security check
-			tableIndex = 0;
-
-		if (period >= note2Period[tableIndex])
-			hiPeriod = tmpPeriod - fineTune;
-		else
-			loPeriod = tmpPeriod - fineTune;
-	}
-
-	if (loPeriod >= ((8*12*16) + 15) - 1) // FT2 bug: off-by-one error
-		loPeriod = (8*12*16) + 15;
-
-	note = (uint8_t)(((loPeriod + 8) >> 4) - ch->relTonNr) + 1;
-	return note;
-}
-
-void drawPianoReplayer(chSyncData_t *chSyncData) // draw piano with synced replayer datas
-{
-	bool keyDown, newStatus[96];
-	uint8_t key, note, octave;
-	syncedChannel_t *ch;
-
-	memset(newStatus, 0, sizeof (newStatus));
-
-	// find active notes
-	if (editor.curInstr > 0)
-	{
-		for (uint8_t i = 0; i < song.antChn; i++)
-		{
-			ch = &chSyncData->channels[i];
-			if (ch->instrNr == editor.curInstr)
-			{
-				note = getNoteReplayer(ch);
-				if (ch->envSustainActive)
-					newStatus[(note - 1) % 96] = true;
-			}
-		}
-	}
-
-	// draw keys
-	for (uint8_t i = 0; i < 96; i++)
-	{
-		keyDown = newStatus[i];
-		if (pianoKeyStatus[i] ^ keyDown)
-		{
-			key = i % 12;
-			octave = i / 12;
+			uint8_t key = noteTab1[i];
+			uint8_t octave = noteTab2[i];
 
 			if (keyIsBlackTab[key])
 				drawBlackPianoKey(key, octave, keyDown);
@@ -1841,10 +1755,10 @@ static void envelopeLine(int32_t nr, int16_t x1, int16_t y1, int16_t x2, int16_t
 
 	// get coefficients
 	dx = x2 - x1;
-	ax = ABS(dx) * 2;
+	ax = ABS(dx) << 1;
 	sx = SGN(dx);
 	dy = y2 - y1;
-	ay = ABS(dy) * 2;
+	ay = ABS(dy) << 1;
 	sy = SGN(dy);
 	x  = x1;
 	y  = y1;
@@ -1859,7 +1773,7 @@ static void envelopeLine(int32_t nr, int16_t x1, int16_t y1, int16_t x2, int16_t
 	// draw line
 	if (ax > ay)
 	{
-		d = ay - (ax / 2);
+		d = ay - (ax >> 1);
 
 		while (true)
 		{
@@ -1888,7 +1802,7 @@ static void envelopeLine(int32_t nr, int16_t x1, int16_t y1, int16_t x2, int16_t
 	}
 	else
 	{
-		d = ax - (ay / 2);
+		d = ax - (ay >> 1);
 
 		while (true)
 		{
@@ -2111,7 +2025,7 @@ void handleInstEditorRedrawing(void)
 
 void hideInstEditor(void)
 {
-	editor.ui.instEditorShown = false;
+	ui.instEditorShown = false;
 
 	hideScrollBar(SB_INST_VOL);
 	hideScrollBar(SB_INST_PAN);
@@ -2197,7 +2111,7 @@ void updateInstEditor(void)
 		s = &ins->samp[editor.curSmp];
 
 	// update instrument editor extension
-	if (editor.ui.instEditorExtShown)
+	if (ui.instEditorExtShown)
 	{
 		checkBoxes[CB_INST_EXT_MIDI].checked = ins->midiOn ? true : false;
 		checkBoxes[CB_INST_EXT_MUTE].checked = ins->mute ? true : false;
@@ -2214,7 +2128,7 @@ void updateInstEditor(void)
 		drawMIDIBend();
 	}
 
-	if (!editor.ui.instEditorShown)
+	if (!ui.instEditorShown)
 		return;
 
 	drawVolEnvSus();
@@ -2284,12 +2198,12 @@ void updateInstEditor(void)
 
 void showInstEditor(void)
 {
-	if (editor.ui.extended) exitPatternEditorExtended();
-	if (editor.ui.sampleEditorShown) hideSampleEditor();
-	if (editor.ui.sampleEditorExtShown) hideSampleEditorExt();
+	if (ui.extended) exitPatternEditorExtended();
+	if (ui.sampleEditorShown) hideSampleEditor();
+	if (ui.sampleEditorExtShown) hideSampleEditorExt();
 
 	hidePatternEditor();
-	editor.ui.instEditorShown = true;
+	ui.instEditorShown = true;
 
 	drawFramework(0,   173, 438,  87, FRAMEWORK_TYPE1);
 	drawFramework(0,   260, 438,  87, FRAMEWORK_TYPE1);
@@ -2304,17 +2218,17 @@ void showInstEditor(void)
 
 	textOutShadow(20,  176, PAL_FORGRND, PAL_DSKTOP2, "Volume envelope:");
 	textOutShadow(153, 176, PAL_FORGRND, PAL_DSKTOP2, "Predef.");
-	textOutShadow(358, 193, PAL_FORGRND, PAL_DSKTOP2, "Sustain:");
+	textOutShadow(358, 194, PAL_FORGRND, PAL_DSKTOP2, "Sustain:");
 	textOutShadow(342, 206, PAL_FORGRND, PAL_DSKTOP2, "Point");
 	textOutShadow(358, 219, PAL_FORGRND, PAL_DSKTOP2, "Env.loop:");
-	textOutShadow(342, 234, PAL_FORGRND, PAL_DSKTOP2, "Start");
+	textOutShadow(342, 233, PAL_FORGRND, PAL_DSKTOP2, "Start");
 	textOutShadow(342, 247, PAL_FORGRND, PAL_DSKTOP2, "End");
 	textOutShadow(20,  263, PAL_FORGRND, PAL_DSKTOP2, "Panning envelope:");
 	textOutShadow(152, 263, PAL_FORGRND, PAL_DSKTOP2, "Predef.");
-	textOutShadow(358, 280, PAL_FORGRND, PAL_DSKTOP2, "Sustain:");
+	textOutShadow(358, 281, PAL_FORGRND, PAL_DSKTOP2, "Sustain:");
 	textOutShadow(342, 293, PAL_FORGRND, PAL_DSKTOP2, "Point");
 	textOutShadow(358, 306, PAL_FORGRND, PAL_DSKTOP2, "Env.loop:");
-	textOutShadow(342, 321, PAL_FORGRND, PAL_DSKTOP2, "Start");
+	textOutShadow(342, 320, PAL_FORGRND, PAL_DSKTOP2, "Start");
 	textOutShadow(342, 334, PAL_FORGRND, PAL_DSKTOP2, "End");
 	textOutShadow(443, 177, PAL_FORGRND, PAL_DSKTOP2, "Volume");
 	textOutShadow(443, 191, PAL_FORGRND, PAL_DSKTOP2, "Panning");
@@ -2389,10 +2303,10 @@ void showInstEditor(void)
 	showCheckBox(CB_INST_PENV_LOOP);
 
 	// draw auto-vibrato waveforms
-	blitFast(455, 279, &vibWaveformBitmap[(12*10)*0], 12, 10);
-	blitFast(485, 279, &vibWaveformBitmap[(12*10)*1], 12, 10);
-	blitFast(515, 279, &vibWaveformBitmap[(12*10)*2], 12, 10);
-	blitFast(545, 279, &vibWaveformBitmap[(12*10)*3], 12, 10);
+	blitFast(455, 279, &bmp.vibratoWaveforms[0*(12*10)], 12, 10);
+	blitFast(485, 279, &bmp.vibratoWaveforms[1*(12*10)], 12, 10);
+	blitFast(515, 279, &bmp.vibratoWaveforms[2*(12*10)], 12, 10);
+	blitFast(545, 279, &bmp.vibratoWaveforms[3*(12*10)], 12, 10);
 
 	showRadioButtonGroup(RB_GROUP_INST_WAVEFORM);
 
@@ -2402,10 +2316,10 @@ void showInstEditor(void)
 
 void toggleInstEditor(void)
 {
-	if (editor.ui.sampleEditorShown)
+	if (ui.sampleEditorShown)
 		hideSampleEditor();
 
-	if (editor.ui.instEditorShown)
+	if (ui.instEditorShown)
 	{
 		exitInstEditor();
 	}
@@ -2422,7 +2336,7 @@ bool testInstrVolEnvMouseDown(bool mouseButtonDown)
 	int32_t x, y, mx, my, minX, maxX;
 	instrTyp *ins;
 
-	if (!editor.ui.instEditorShown || editor.curInstr == 0 || instr[editor.curInstr] == NULL)
+	if (!ui.instEditorShown || editor.curInstr == 0 || instr[editor.curInstr] == NULL)
 		return false;
 
 	ins = instr[editor.curInstr];
@@ -2521,7 +2435,7 @@ bool testInstrPanEnvMouseDown(bool mouseButtonDown)
 	int32_t x, y, mx, my, minX, maxX;
 	instrTyp *ins;
 
-	if (!editor.ui.instEditorShown || editor.curInstr == 0 || instr[editor.curInstr] == NULL)
+	if (!ui.instEditorShown || editor.curInstr == 0 || instr[editor.curInstr] == NULL)
 		return false;
 
 	ins = instr[editor.curInstr];
@@ -2651,8 +2565,8 @@ void drawInstEditorExt(void)
 	textOutShadow(4,   96,  PAL_FORGRND, PAL_DSKTOP2, "Instrument Editor Extension:");
 	textOutShadow(20,  114, PAL_FORGRND, PAL_DSKTOP2, "Instrument MIDI enable");
 	textOutShadow(189, 114, PAL_FORGRND, PAL_DSKTOP2, "Mute computer");
-	textOutShadow(4,   133, PAL_FORGRND, PAL_DSKTOP2, "MIDI transmit channel");
-	textOutShadow(4,   147, PAL_FORGRND, PAL_DSKTOP2, "MIDI program");
+	textOutShadow(4,   132, PAL_FORGRND, PAL_DSKTOP2, "MIDI transmit channel");
+	textOutShadow(4,   146, PAL_FORGRND, PAL_DSKTOP2, "MIDI program");
 	textOutShadow(4,   160, PAL_FORGRND, PAL_DSKTOP2, "Bender range (halftones)");
 
 	if (ins == NULL)
@@ -2693,14 +2607,14 @@ void drawInstEditorExt(void)
 
 void showInstEditorExt(void)
 {
-	if (editor.ui.extended)
+	if (ui.extended)
 		exitPatternEditorExtended();
 
 	hideTopScreen();
 	showTopScreen(false);
 
-	editor.ui.instEditorExtShown = true;
-	editor.ui.scopesShown = false;
+	ui.instEditorExtShown = true;
+	ui.scopesShown = false;
 	drawInstEditorExt();
 }
 
@@ -2718,14 +2632,14 @@ void hideInstEditorExt(void)
 	hidePushButton(PB_INST_EXT_MIDI_BEND_DOWN);
 	hidePushButton(PB_INST_EXT_MIDI_BEND_UP);
 
-	editor.ui.instEditorExtShown = false;
-	editor.ui.scopesShown = true;
+	ui.instEditorExtShown = false;
+	ui.scopesShown = true;
 	drawScopeFramework();
 }
 
 void toggleInstEditorExt(void)
 {
-	if (editor.ui.instEditorExtShown)
+	if (ui.instEditorExtShown)
 		hideInstEditorExt();
 	else
 		showInstEditorExt();
@@ -2773,7 +2687,7 @@ static bool testInstrSwitcherNormal(void) // Welcome to the Jungle
 				editor.srcInstr = newEntry;
 				updateInstrumentSwitcher();
 
-				if (editor.ui.advEditShown)
+				if (ui.advEditShown)
 					updateAdvEdit();
 			}
 
@@ -2798,8 +2712,8 @@ static bool testInstrSwitcherNormal(void) // Welcome to the Jungle
 				updateInstrumentSwitcher();
 				updateSampleEditorSample();
 
-				     if (editor.ui.sampleEditorShown) updateSampleEditor();
-				else if (editor.ui.instEditorShown)   updateInstEditor();
+				     if (ui.sampleEditorShown) updateSampleEditor();
+				else if (ui.instEditorShown)   updateInstEditor();
 			}
 
 			return true;
@@ -2850,7 +2764,7 @@ static bool testInstrSwitcherExtended(void) // Welcome to the Jungle 2 - The Hap
 				editor.srcInstr = newEntry;
 				updateInstrumentSwitcher();
 
-				if (editor.ui.advEditShown)
+				if (ui.advEditShown)
 					updateAdvEdit();
 			}
 
@@ -2892,7 +2806,7 @@ static bool testInstrSwitcherExtended(void) // Welcome to the Jungle 2 - The Hap
 				editor.srcInstr = newEntry;
 				updateInstrumentSwitcher();
 
-				if (editor.ui.advEditShown)
+				if (ui.advEditShown)
 					updateAdvEdit();
 			}
 
@@ -2923,10 +2837,10 @@ static bool testInstrSwitcherExtended(void) // Welcome to the Jungle 2 - The Hap
 
 bool testInstrSwitcherMouseDown(void)
 {
-	if (!editor.ui.instrSwitcherShown)
+	if (!ui.instrSwitcherShown)
 		return false;
 
-	if (editor.ui.extended)
+	if (ui.extended)
 		return testInstrSwitcherExtended();
 	else
 		return testInstrSwitcherNormal();
@@ -3085,7 +2999,10 @@ void saveInstr(UNICHAR *filenameU, int16_t nr)
 
 static int16_t getPATNote(int32_t freq)
 {
-	return (int16_t)round(((log(freq / (440.0 * 1000.0)) / M_LN2) * 12.0) + 48.0 + 9.0);
+	double dNote = (log2(freq * (1.0 / 440000.0)) * 12.0) + 57.0;
+	int32_t note = (int32_t)(dNote + 0.5);
+
+	return (int16_t)note;
 }
 
 static int32_t SDLCALL loadInstrThread(void *ptr)
@@ -3185,11 +3102,11 @@ static int32_t SDLCALL loadInstrThread(void *ptr)
 			ins->vibDepth = ih.vibDepth;
 			ins->vibRate = ih.vibRate;
 			ins->fadeOut = ih.fadeOut;
-			ins->midiOn = (ih.midiOn > 0) ? true : false;
+			ins->midiOn = (ih.midiOn == 1) ? true : false;
 			ins->midiChannel = ih.midiChannel;
 			ins->midiProgram = ih.midiProgram;
 			ins->midiBend = ih.midiBend;
-			ins->mute = (ih.mute > 0) ? true : false;
+			ins->mute = (ih.mute == 1) ? true : false;
 			ins->antSamp = ih.antSamp; // used in loadInstrSample()
 
 			// sanitize stuff for broken/unsupported instruments
@@ -3280,8 +3197,9 @@ static int32_t SDLCALL loadInstrThread(void *ptr)
 
 			if (s->len > 0)
 			{
-				s->pek = (int8_t *)malloc(s->len + LOOP_FIX_LEN);
-				if (s->pek == NULL)
+				s->pek = NULL;
+				s->origPek = (int8_t *)malloc(s->len + LOOP_FIX_LEN);
+				if (s->origPek == NULL)
 				{
 					freeInstr(editor.curInstr);
 					resumeAudio();
@@ -3289,6 +3207,8 @@ static int32_t SDLCALL loadInstrThread(void *ptr)
 					goto loadDone;
 				}
 
+				s->pek = s->origPek + SMP_DAT_OFFSET;
+
 				if (fread(s->pek, s->len, 1, f) != 1)
 				{
 					freeInstr(editor.curInstr);
@@ -3304,13 +3224,16 @@ static int32_t SDLCALL loadInstrThread(void *ptr)
 				{
 					s->typ &= ~32;
 
-					s->len /= 2;
-					s->repL /= 2;
-					s->repS /= 2;
+					s->len >>= 1;
+					s->repL >>= 1;
+					s->repS >>= 1;
 
-					newPtr = (int8_t *)realloc(s->pek, s->len + LOOP_FIX_LEN);
+					newPtr = (int8_t *)realloc(s->origPek, s->len + LOOP_FIX_LEN);
 					if (newPtr != NULL)
-						s->pek = newPtr;
+					{
+						s->origPek = newPtr;
+						s->pek = s->origPek + SMP_DAT_OFFSET;
+					}
 
 					stereoWarning = true;
 				}
@@ -3331,7 +3254,10 @@ static int32_t SDLCALL loadInstrThread(void *ptr)
 		{
 			// PAT - Gravis Ultrasound GF1 patch
 
-			if (ih_PAT.layers > 1 || ih_PAT.antSamp > 16 || ih_PAT.antSamp == 0)
+			if (ih_PAT.antSamp == 0)
+				ih_PAT.antSamp = 1; // to some patch makers, 0 means 1
+
+			if (ih_PAT.layers > 1 || ih_PAT.antSamp > MAX_SMP_PER_INST)
 			{
 				okBoxThreadSafe(0, "System message", "Incompatible instrument!");
 				goto loadDone;
@@ -3362,8 +3288,9 @@ static int32_t SDLCALL loadInstrThread(void *ptr)
 					goto loadDone;
 				}
 
-				s->pek = (int8_t *)malloc(ih_PATWave.waveSize + LOOP_FIX_LEN);
-				if (s->pek == NULL)
+				s->pek = NULL;
+				s->origPek = (int8_t *)malloc(ih_PATWave.waveSize + LOOP_FIX_LEN);
+				if (s->origPek == NULL)
 				{
 					freeInstr(editor.curInstr);
 					resumeAudio();
@@ -3371,15 +3298,17 @@ static int32_t SDLCALL loadInstrThread(void *ptr)
 					goto loadDone;
 				}
 
+				s->pek = s->origPek + SMP_DAT_OFFSET;
+
 				if (i == 0)
 				{
 					ins->vibSweep = ih_PATWave.vibSweep;
 
-					ins->vibRate = ih_PATWave.vibRate / 2;
+					ins->vibRate = (ih_PATWave.vibRate + 2) >> 2;
 					if (ins->vibRate > 0x3F)
 						ins->vibRate = 0x3F;
 
-					ins->vibDepth = ih_PATWave.vibDepth / 2;
+					ins->vibDepth = (ih_PATWave.vibDepth + 1) >> 1;
 					if (ins->vibDepth > 0x0F)
 						ins->vibDepth = 0x0F;
 				}
@@ -3397,38 +3326,41 @@ static int32_t SDLCALL loadInstrThread(void *ptr)
 						s->typ |= 1; // forward loop
 				}
 
-				s->pan = ih_PATWave.pan << 4;
+				s->pan = ((ih_PATWave.pan << 4) & 0xF0) | (ih_PATWave.pan & 0xF);
+
+				if (s->typ & 16)
+				{
+					ih_PATWave.waveSize &= 0xFFFFFFFE;
+					ih_PATWave.repS &= 0xFFFFFFFE;
+					ih_PATWave.repE &= 0xFFFFFFFE;
+				}
+
 				s->len = ih_PATWave.waveSize;
+				if (s->len > MAX_SAMPLE_LEN)
+					s->len = MAX_SAMPLE_LEN;
 
 				s->repS = ih_PATWave.repS;
 				if (s->repS > s->len)
 					s->repS = 0;
 
 				s->repL = ih_PATWave.repE - ih_PATWave.repS;
-
-				if (s->typ & 16)
-				{
-					s->len  &= 0xFFFFFFFE;
-					s->repS &= 0xFFFFFFFE;
-					s->repL &= 0xFFFFFFFE;
-				}
-
 				if (s->repL < 0)
 					s->repL = 0;
 
 				if (s->repS+s->repL > s->len)
 					s->repL = s->len - s->repS;
 
-				dFreq = round((1.0 + ih_PATWave.fineTune / 512.0) * ih_PATWave.sampleRate);
-				tuneSample(s, (int32_t)dFreq);
+				dFreq = (1.0 + (ih_PATWave.fineTune / 512.0)) * ih_PATWave.sampleRate;
+				int32_t freq = (int32_t)(dFreq + 0.5);
+				tuneSample(s, freq);
 
-				s->relTon -= (int8_t)(getPATNote(ih_PATWave.rootFrq) - (12*3));
-				s->relTon = CLAMP(s->relTon, -48, 71);
+				a = s->relTon - (getPATNote(ih_PATWave.rootFrq) - (12 * 3));
+				s->relTon = (uint8_t)CLAMP(a, -48, 71);
 
 				a = getPATNote(ih_PATWave.lowFrq);
-				a = CLAMP(a, 0, 95);
-
 				b = getPATNote(ih_PATWave.highFreq);
+
+				a = CLAMP(a, 0, 95);
 				b = CLAMP(b, 0, 95);
 
 				for (j = a; j <= b; j++)
