@@ -1,27 +1,35 @@
 #include <stdint.h>
 #include <stdbool.h>
-#include "ft2_header.h"
 #include "ft2_mix.h"
 #include "ft2_mix_macros.h"
+#include "ft2_intrp_table.h"
 
 /*
-   --------------------- fixed-point audio channel mixer ---------------------
-
-   This file has separate routines for EVERY possible sampling variation:
-   Interpolation, volume ramping, 8-bit, 16-bit, no loop, loop, bidi loop.
-   24 mixing routines in total.
-
-   Every voice has a function pointer set to the according mixing routine on
-   sample trigger (from replayer, but set in audio thread), using a function
-   pointer look-up table.
-   All voices are always cleared (thread safe) when changing any of the above
-   states from the GUI, so no problem there with deprecated cached function
-   pointers.
-
-   Mixing macros can be found in ft2_mix_macros.h.
-
-   Yes, this is a HUGE mess, and I hope you don't need to modify it.
-   If it's not broken, don't try to fix it!
+** --------------------- 32-bit fixed-point audio channel mixer ---------------------
+**              (Note: Mixing macros can be found in ft2_mix_macros.h)
+**
+** 8bitbubsy: This is mostly ported from the i386-asm 32-bit mixer that was introduced
+** in FT2.08 (MS-DOS). It has been changed and improved quite a bit, though...
+** Instead of 2-tap linear interpolation, it has 4-tap cubic spline interpolation.
+** For x86_64: Fixed-point precision is 32.32 instead of 16.16
+**
+** This file has separate routines for EVERY possible sampling variation:
+** Interpolation on/off, volume ramping on/off, 8-bit, 16-bit, no loop, loop, pingpong.
+** (24 mixing routines in total)
+**
+** Every voice has a function pointer set to the according mixing routine on sample
+** trigger (from replayer, but set in audio thread), using a function pointer look-up
+** table. All voices & pointers are always thread-safely cleared when changing any
+** of the above attributes from the GUI, to prevent possible thread-related issues.
+**
+** There's one problem with the 4-tap cubic spline resampling interpolation...
+** On looped samples where loopStart>0, the splines are not correct when reading
+** from the loopStart (or +1?) sample point. The difference in audio is very minor,
+** so it's not a big problem. It just has to stay like this the way the mixer works.
+** In cases where loopStart=0, the sample before index 0 (yes, we allocate enough
+** data and pre-increment main pointer to support negative look-up), is already
+** pre-fixed so that the splines will be correct.
+** ----------------------------------------------------------------------------------
 */
 
 /* ----------------------------------------------------------------------- */
@@ -30,24 +38,25 @@
 
 static void mix8bNoLoop(voice_t *v, uint32_t numSamples)
 {
-	const int8_t *CDA_LinearAdr;
-	bool mixInMono;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
+	const int8_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
+#endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_NO_LOOP
+		VOL0_MIXING_NO_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS
-	GET_DELTA
 	SET_BASE8
 
 	CDA_BytesLeft = numSamples;
@@ -97,24 +106,25 @@ static void mix8bNoLoop(voice_t *v, uint32_t numSamples)
 
 static void mix8bLoop(voice_t *v, uint32_t numSamples)
 {
-	const int8_t *CDA_LinearAdr;
-	bool mixInMono;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
+	const int8_t *CDA_LinearAdr, *smpPtr;;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
+#endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_LOOP
+		VOL0_MIXING_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS
-	GET_DELTA
 	SET_BASE8
 
 	CDA_BytesLeft = numSamples;
@@ -164,19 +174,21 @@ static void mix8bLoop(voice_t *v, uint32_t numSamples)
 
 static void mix8bBidiLoop(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int8_t *CDA_LinearAdr, *CDA_LinAdrRev;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol, CDA_IPValL, CDA_IPValH;
-	register uint32_t pos;
-	uint32_t delta, i, samplesToMix;
+	const int8_t *CDA_LinearAdr, *CDA_LinAdrRev, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos, delta;
+#else
+	uint32_t pos, delta;
+#endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_BIDI_LOOP
+		VOL0_MIXING_BIDI_LOOP
 		return;
 	}
 
@@ -231,27 +243,25 @@ static void mix8bBidiLoop(voice_t *v, uint32_t numSamples)
 
 static void mix8bNoLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	const int8_t *CDA_LinearAdr;
-	bool mixInMono;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int8_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
 #endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_NO_LOOP
+		VOL0_MIXING_NO_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS
-	GET_DELTA
 	SET_BASE8
 
 	CDA_BytesLeft = numSamples;
@@ -301,27 +311,25 @@ static void mix8bNoLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix8bLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	const int8_t *CDA_LinearAdr;
-	bool mixInMono;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int8_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
 #endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_LOOP
+		VOL0_MIXING_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS
-	GET_DELTA
 	SET_BASE8
 
 	CDA_BytesLeft = numSamples;
@@ -371,22 +379,21 @@ static void mix8bLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix8bBidiLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int8_t *CDA_LinearAdr, *CDA_LinAdrRev;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol, CDA_IPValL, CDA_IPValH;
-	register uint32_t pos;
-	uint32_t delta, i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int8_t *CDA_LinearAdr, *CDA_LinAdrRev, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos, delta;
+#else
+	uint32_t pos, delta;
 #endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_BIDI_LOOP
+		VOL0_MIXING_BIDI_LOOP
 		return;
 	}
 
@@ -442,22 +449,23 @@ static void mix8bBidiLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix8bRampNoLoop(voice_t *v, uint32_t numSamples)
 {
-	const int8_t *CDA_LinearAdr;
-	bool mixInMono;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
+	const int8_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
+#endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_NO_LOOP
+		VOL0_MIXING_NO_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS_RAMP
-	GET_DELTA
 	SET_BASE8
 
 	CDA_BytesLeft = numSamples;
@@ -516,22 +524,23 @@ static void mix8bRampNoLoop(voice_t *v, uint32_t numSamples)
 
 static void mix8bRampLoop(voice_t *v, uint32_t numSamples)
 {
-	const int8_t *CDA_LinearAdr;
-	bool mixInMono;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
+	const int8_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
+#endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_LOOP
+		VOL0_MIXING_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS_RAMP
-	GET_DELTA
 	SET_BASE8
 
 	CDA_BytesLeft = numSamples;
@@ -590,17 +599,19 @@ static void mix8bRampLoop(voice_t *v, uint32_t numSamples)
 
 static void mix8bRampBidiLoop(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int8_t *CDA_LinearAdr, *CDA_LinAdrRev;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol, CDA_IPValL, CDA_IPValH;
-	register uint32_t pos;
-	uint32_t delta, i, samplesToMix;
+	const int8_t *CDA_LinearAdr, *CDA_LinAdrRev, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos, delta;
+#else
+	uint32_t pos, delta;
+#endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_BIDI_LOOP
+		VOL0_MIXING_BIDI_LOOP
 		return;
 	}
 
@@ -665,25 +676,23 @@ static void mix8bRampBidiLoop(voice_t *v, uint32_t numSamples)
 
 static void mix8bRampNoLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	const int8_t *CDA_LinearAdr;
-	bool mixInMono;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int8_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
 #endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_NO_LOOP
+		VOL0_MIXING_NO_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS_RAMP
-	GET_DELTA
 	SET_BASE8
 
 	CDA_BytesLeft = numSamples;
@@ -742,25 +751,23 @@ static void mix8bRampNoLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix8bRampLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	const int8_t *CDA_LinearAdr;
-	bool mixInMono;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int8_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
 #endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_LOOP
+		VOL0_MIXING_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS_RAMP
-	GET_DELTA
 	SET_BASE8
 
 	CDA_BytesLeft = numSamples;
@@ -819,20 +826,19 @@ static void mix8bRampLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix8bRampBidiLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int8_t *CDA_LinearAdr, *CDA_LinAdrRev;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int8_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol, CDA_IPValL, CDA_IPValH;
-	register uint32_t pos;
-	uint32_t delta, i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int8_t *CDA_LinearAdr, *CDA_LinAdrRev, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos, delta;
+#else
+	uint32_t pos, delta;
 #endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_BIDI_LOOP
+		VOL0_MIXING_BIDI_LOOP
 		return;
 	}
 
@@ -902,24 +908,25 @@ static void mix8bRampBidiLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix16bNoLoop(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
+	const int16_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
+#endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_NO_LOOP
+		VOL0_MIXING_NO_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS
-	GET_DELTA
 	SET_BASE16
 
 	CDA_BytesLeft = numSamples;
@@ -969,24 +976,25 @@ static void mix16bNoLoop(voice_t *v, uint32_t numSamples)
 
 static void mix16bLoop(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
+	const int16_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
+#endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_LOOP
+		VOL0_MIXING_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS
-	GET_DELTA
 	SET_BASE16
 
 	CDA_BytesLeft = numSamples;
@@ -1036,19 +1044,21 @@ static void mix16bLoop(voice_t *v, uint32_t numSamples)
 
 static void mix16bBidiLoop(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr, *CDA_LinAdrRev;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol, CDA_IPValL, CDA_IPValH;
-	register uint32_t pos;
-	uint32_t delta, i, samplesToMix;
+	const int16_t *CDA_LinearAdr, *CDA_LinAdrRev, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos, delta;
+#else
+	uint32_t pos, delta;
+#endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_BIDI_LOOP
+		VOL0_MIXING_BIDI_LOOP
 		return;
 	}
 
@@ -1104,27 +1114,25 @@ static void mix16bBidiLoop(voice_t *v, uint32_t numSamples)
 
 static void mix16bNoLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int16_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
 #endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_NO_LOOP
+		VOL0_MIXING_NO_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS
-	GET_DELTA
 	SET_BASE16
 
 	CDA_BytesLeft = numSamples;
@@ -1174,27 +1182,25 @@ static void mix16bNoLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix16bLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int16_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
 #endif
 
 	GET_VOL
 
 	if ((CDA_LVol| CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_LOOP
+		VOL0_MIXING_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS
-	GET_DELTA
 	SET_BASE16
 
 	CDA_BytesLeft = numSamples;
@@ -1244,22 +1250,21 @@ static void mix16bLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix16bBidiLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr, *CDA_LinAdrRev;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol, CDA_IPValL, CDA_IPValH;
-	register uint32_t pos;
-	uint32_t delta, i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int16_t *CDA_LinearAdr, *CDA_LinAdrRev, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos, delta;
+#else
+	uint32_t pos, delta;
 #endif
 
 	GET_VOL
 
 	if ((CDA_LVol | CDA_RVol) == 0)
 	{
-		VOL0_OPTIMIZATION_BIDI_LOOP
+		VOL0_MIXING_BIDI_LOOP
 		return;
 	}
 
@@ -1315,22 +1320,23 @@ static void mix16bBidiLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix16bRampNoLoop(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
+	const int16_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
+#endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_NO_LOOP
+		VOL0_MIXING_NO_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS_RAMP
-	GET_DELTA
 	SET_BASE16
 
 	CDA_BytesLeft = numSamples;
@@ -1389,22 +1395,23 @@ static void mix16bRampNoLoop(voice_t *v, uint32_t numSamples)
 
 static void mix16bRampLoop(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
+	const int16_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
+#endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_LOOP
+		VOL0_MIXING_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS_RAMP
-	GET_DELTA
 	SET_BASE16
 
 	CDA_BytesLeft = numSamples;
@@ -1463,17 +1470,19 @@ static void mix16bRampLoop(voice_t *v, uint32_t numSamples)
 
 static void mix16bRampBidiLoop(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr, *CDA_LinAdrRev;
-	int32_t realPos, sample, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol, CDA_IPValL, CDA_IPValH;
-	register uint32_t pos;
-	uint32_t delta, i, samplesToMix;
+	const int16_t *CDA_LinearAdr, *CDA_LinAdrRev, *smpPtr;
+	int32_t realPos, sample, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos, delta;
+#else
+	uint32_t pos, delta;
+#endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_BIDI_LOOP
+		VOL0_MIXING_BIDI_LOOP
 		return;
 	}
 
@@ -1538,25 +1547,23 @@ static void mix16bRampBidiLoop(voice_t *v, uint32_t numSamples)
 
 static void mix16bRampNoLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int16_t *smpPtr;
-	register int32_t  CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int16_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
 #endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_NO_LOOP
+		VOL0_MIXING_NO_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS_RAMP
-	GET_DELTA
 	SET_BASE16
 
 	CDA_BytesLeft = numSamples;
@@ -1615,25 +1622,23 @@ static void mix16bRampNoLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix16bRampLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol;
-	register uint32_t pos, delta;
-	uint32_t i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int16_t *CDA_LinearAdr, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos;
+#else
+	uint32_t pos;
 #endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_LOOP
+		VOL0_MIXING_LOOP
 		return;
 	}
 
 	GET_MIXER_VARS_RAMP
-	GET_DELTA
 	SET_BASE16
 
 	CDA_BytesLeft = numSamples;
@@ -1692,20 +1697,19 @@ static void mix16bRampLoopIntrp(voice_t *v, uint32_t numSamples)
 
 static void mix16bRampBidiLoopIntrp(voice_t *v, uint32_t numSamples)
 {
-	bool mixInMono;
-	const int16_t *CDA_LinearAdr, *CDA_LinAdrRev;
-	int32_t realPos, sample, sample2, *audioMixL, *audioMixR, CDA_BytesLeft, CDA_LVolIP, CDA_RVolIP;
-	register const int16_t *smpPtr;
-	register int32_t CDA_LVol, CDA_RVol, CDA_IPValL, CDA_IPValH;
-	register uint32_t pos;
-	uint32_t delta, i, samplesToMix;
-#ifndef LERPMIX
-	int32_t sample3;
+	const int16_t *CDA_LinearAdr, *CDA_LinAdrRev, *smpPtr;
+	int32_t realPos, sample, sample2, sample3, sample4, *audioMixL, *audioMixR;
+	int32_t CDA_LVolIP, CDA_RVolIP, CDA_LVol, CDA_RVol;
+	uint32_t i, samplesToMix, CDA_BytesLeft;
+#if defined _WIN64 || defined __amd64__
+	uint64_t pos, delta;
+#else
+	uint32_t pos, delta;
 #endif
 
 	if ((v->SLVol1 | v->SRVol1 | v->SLVol2 | v->SRVol2) == 0)
 	{
-		VOL0_OPTIMIZATION_BIDI_LOOP
+		VOL0_MIXING_BIDI_LOOP
 		return;
 	}
 
