@@ -24,6 +24,8 @@
 #include "ft2_midi.h"
 #include "ft2_events.h"
 #include "ft2_video.h"
+#include "ft2_tables.h"
+#include "ft2_structs.h"
 
 /* This is a *HUGE* mess!
 ** I hope you never have to modify it, and you probably shouldn't either.
@@ -127,8 +129,8 @@ songS3MHeaderTyp;
 #endif
 
 static volatile uint8_t loadedFormat;
-static volatile bool stereoSamplesWarn, linearFreqTable, musicIsLoading, moduleLoaded, moduleFailedToLoad;
-static uint8_t oldPlayMode, pattBuff[12288];
+static volatile bool linearFreqTable, musicIsLoading, moduleLoaded, moduleFailedToLoad;
+static uint8_t oldPlayMode, pattBuff[12288], packedPattData[65536];
 static const uint8_t stmEff[16] = { 0, 0, 11, 0, 10, 2, 1, 3, 4, 7, 0, 5 ,6, 0, 0, 0 };
 static SDL_Thread *thread;
 
@@ -142,7 +144,7 @@ static void setupLoadedModule(void);
 static void freeTmpModule(void);
 static bool loadInstrHeader(FILE *f, uint16_t i);
 static bool loadInstrSample(FILE *f, uint16_t i);
-void unpackPatt(uint8_t *dst, uint16_t inn, uint16_t len, uint8_t antChn);
+void unpackPatt(uint8_t *dst, uint8_t *src, uint16_t len, int32_t antChn);
 static bool tmpPatternEmpty(uint16_t nr);
 static bool loadPatterns(FILE *f, uint16_t antPtn);
 
@@ -150,7 +152,6 @@ void checkSampleRepeat(sampleTyp *s);
 
 // ft2_replayer.c
 extern const char modSig[32][5];
-extern const uint16_t amigaPeriod[12*8];
 
 static bool allocateTmpInstr(int16_t nr)
 {
@@ -184,7 +185,7 @@ static uint8_t getModType(uint8_t *numChannels, const char *id)
 
 	if (IS_ID("M.K.", id) || IS_ID("M!K!", id) || IS_ID("NSMS", id) || IS_ID("LARD", id) || IS_ID("PATT", id))
 	{
-		modFormat = FORMAT_MK; // ProTracker or compatible	
+		modFormat = FORMAT_MK; // ProTracker or compatible
 	}
 	else if (id[1] == 'C' && id[2] == 'H' && id[3] == 'N')
 	{
@@ -200,11 +201,6 @@ static uint8_t getModType(uint8_t *numChannels, const char *id)
 	{
 		modFormat = FORMAT_FLT; // StarTrekker (4-channel modules only)
 	}
-	else if (IS_ID("FLT8", id))
-	{
-		modFormat = FORMAT_FLT; // StarTrekker (4-channel modules only)
-		*numChannels = 8;
-	}
 	else if (IS_ID("N.T.", id))
 	{
 		modFormat = FORMAT_NT; // NoiseTracker
@@ -364,23 +360,23 @@ static bool loadMusicMOD(FILE *f, uint32_t fileLength, bool fromExternalThread)
 		*/
 		modFormat = FORMAT_STK;
 
-		if (h_MOD15.repS == 0)
-			h_MOD15.repS = 120;
+		if (songTmp.repS == 0)
+			songTmp.repS = 120;
 
-		// jjk55.mod by Jesper Kyd has a bogus STK tempo value that should be ignored
+		// jjk55.mod by Jesper Kyd has a bogus STK tempo value that should be ignored (hackish!)
 		if (!strcmp("jjk55", h_MOD31.name))
-			h_MOD15.repS = 120;
+			songTmp.repS = 120;
 
 		// The "restart pos" field in STK is the inital tempo (must be converted to BPM first)
-		if (h_MOD15.repS != 120) // 120 is a special case and means 50Hz (125BPM)
+		if (songTmp.repS != 120) // 120 is a special case and means 50Hz (125BPM)
 		{
-			if (h_MOD15.repS > 220)
-				h_MOD15.repS = 220;
+			if (songTmp.repS > 220)
+				songTmp.repS = 220;
 
 			// convert UST tempo to BPM
 			uint16_t ciaPeriod = (240 - songTmp.repS) * 122;
 			double dHz = 709379.0 / ciaPeriod;
-			int32_t BPM = (int32_t)((dHz * (125.0 / 50.0)) + 0.5);
+			int32_t BPM = (int32_t)((dHz * 2.5) + 0.5);
 
 			songTmp.speed = (uint16_t)BPM;
 		}
@@ -450,7 +446,7 @@ static bool loadMusicMOD(FILE *f, uint32_t fileLength, bool fromExternalThread)
 				ton->effTyp = bytes[2] & 0x0F;
 				ton->eff = bytes[3];
 
-				if (mightBeSTK)
+				if (modFormat == FORMAT_STK)
 				{
 					if (ton->effTyp == 0xC || ton->effTyp == 0xD || ton->effTyp == 0xE)
 					{
@@ -519,7 +515,7 @@ static bool loadMusicMOD(FILE *f, uint32_t fileLength, bool fromExternalThread)
 	}
 
 	// pattern command conversion for non-PT formats
-	if (modFormat == FORMAT_STK || modFormat == FORMAT_FT2 || modFormat == FORMAT_NT || modFormat == FORMAT_HMNT || modFormat == FORMAT_FLT)
+	if (modFormat == FORMAT_STK || modFormat == FORMAT_NT || modFormat == FORMAT_HMNT || modFormat == FORMAT_FLT)
 	{
 		for (a = 0; a < b; a++)
 		{
@@ -628,18 +624,11 @@ static bool loadMusicMOD(FILE *f, uint32_t fileLength, bool fromExternalThread)
 
 		s->len = 2 * SWAP16(h_MOD31.instr[a].len);
 
-		s->pek = (int8_t *)malloc(s->len + LOOP_FIX_LEN);
-		if (s->pek == NULL)
-		{
-			showMsg(0, "System message", "Not enough memory!");
-			goto modLoadError;
-		}
-
 		if (modFormat != FORMAT_HMNT) // most of "His Master's Noisetracker" songs have junk sample names, so let's not load them
 			memcpy(s->name, songTmp.instrName[1+a], 22);
 
 		if (modFormat == FORMAT_HMNT) // finetune in "His Master's NoiseTracker" is different
-			h_MOD31.instr[a].fine = (uint8_t)((-h_MOD31.instr[a].fine & 0x1F) / 2); // one more bit of precision, + inverted
+			h_MOD31.instr[a].fine = (uint8_t)((-h_MOD31.instr[a].fine & 0x1F) >> 1); // one more bit of precision, + inverted
 
 		if (modFormat != FORMAT_STK)
 			s->fine = 8 * ((2 * ((h_MOD31.instr[a].fine & 0xF) ^ 8)) - 16);
@@ -657,14 +646,14 @@ static bool loadMusicMOD(FILE *f, uint32_t fileLength, bool fromExternalThread)
 			s->repL = 2;
 
 		// in The Ultimate SoundTracker, sample loop start is in bytes, not words
-		if (mightBeSTK)
-			s->repS /= 2;
+		if (modFormat == FORMAT_STK)
+			s->repS >>= 1;
 
 		// fix for poorly converted STK (< v2.5) -> PT/NT modules (FIXME: Worth keeping or not?)
-		if (!mightBeSTK && s->repL > 2 && s->repS+s->repL > s->len)
+		if (modFormat != FORMAT_STK && s->repL > 2 && s->repS+s->repL > s->len)
 		{
-			if ((s->repS/2) + s->repL <= s->len)
-				s->repS /= 2;
+			if ((s->repS>>1)+s->repL <= s->len)
+				s->repS >>= 1;
 		}
 
 		// fix overflown loop
@@ -687,13 +676,23 @@ static bool loadMusicMOD(FILE *f, uint32_t fileLength, bool fromExternalThread)
 		/* For Ultimate SoundTracker modules, only the loop area of a looped sample is played.
 		** Skip loading of eventual data present before loop start.
 		*/
-		if (modFormat == FORMAT_STK && (s->repS > 0 && s->repL < s->len))
+		if (modFormat == FORMAT_STK && s->repS > 0 && s->repL < s->len)
 		{
 			s->len -= s->repS;
 			fseek(f, s->repS, SEEK_CUR);
 			s->repS = 0;
 		}
 
+		s->pek = NULL;
+		s->origPek = (int8_t *)malloc(s->len + LOOP_FIX_LEN);
+		if (s->origPek == NULL)
+		{
+			showMsg(0, "System message", "Not enough memory!");
+			goto modLoadError;
+		}
+
+		s->pek = s->origPek + SMP_DAT_OFFSET;
+
 		int32_t bytesRead = (int32_t)fread(s->pek, 1, s->len, f);
 		if (bytesRead < s->len)
 		{
@@ -701,6 +700,13 @@ static bool loadMusicMOD(FILE *f, uint32_t fileLength, bool fromExternalThread)
 			memset(&s->pek[bytesRead], 0, bytesToClear);
 		}
 
+		// clear repL and repS on non-looping samples...
+		if (s->typ == 0)
+		{
+			s->repL = 0;
+			s->repS = 0;
+		}
+
 		fixSample(s);
 	}
 
@@ -725,18 +731,15 @@ static uint8_t stmTempoToBPM(uint8_t tempo) // ported from original ST2.3 replay
 	uint16_t hz = 50;
 
 	hz -= ((slowdowns[tempo >> 4] * (tempo & 15)) >> 4); // can and will underflow
-
-	bpm = (uint32_t)round(hz * 2.5);
+	bpm = (hz << 1) + (hz >> 1); // BPM = hz * 2.5
 	return (uint8_t)CLAMP(bpm, 32, 255); // result can be slightly off, but close enough...
 }
 
 static bool loadMusicSTM(FILE *f, uint32_t fileLength, bool fromExternalThread)
 {
-	bool check3xx;
-	uint8_t typ, tmp8, tempo;
-	int16_t i, j, k, ai, ap, tmp;
+	uint8_t typ, tempo;
+	int16_t i, j, k, ap, tmp;
 	uint16_t a;
-	int32_t len;
 	tonTyp *ton;
 	sampleTyp *s;
 	songSTMHeaderTyp h_STM;
@@ -877,18 +880,6 @@ static bool loadMusicSTM(FILE *f, uint32_t fileLength, bool fromExternalThread)
 					ton->eff = 0;
 				}
 
-				/* Remove any EDx with no note.
-				** SDx with no note in ST3 = does nothing
-				** EDx with no note in FT2 = still retriggers */
-				if (ton->effTyp == 0xE && (ton->eff & 0xF0) == 0xD0)
-				{
-					if (ton->ton == 0 || ton->ton == 97)
-					{
-						ton->eff = 0;
-						ton->effTyp = 0;
-					}
-				}
-
 				if (ton->effTyp > 35)
 				{
 					ton->effTyp = 0;
@@ -909,7 +900,6 @@ static bool loadMusicSTM(FILE *f, uint32_t fileLength, bool fromExternalThread)
 		}
 	}
 
-	ai = 31;
 	for (i = 0; i < 31; i++)
 	{
 		// trim off spaces at end of name
@@ -931,13 +921,16 @@ static bool loadMusicSTM(FILE *f, uint32_t fileLength, bool fromExternalThread)
 
 			s = &instrTmp[1+i]->samp[0];
 
-			s->pek = (int8_t *)malloc(h_STM.instr[i].len + LOOP_FIX_LEN);
-			if (s->pek == NULL)
+			s->pek = NULL;
+			s->origPek = (int8_t *)malloc(h_STM.instr[i].len + LOOP_FIX_LEN);
+			if (s->origPek == NULL)
 			{
 				showMsg(0, "System message", "Not enough memory!");
 				goto stmLoadError;
 			}
 
+			s->pek = s->origPek + SMP_DAT_OFFSET;
+
 			s->len = h_STM.instr[i].len;
 			tuneSample(s, h_STM.instr[i].rate);
 			s->vol = h_STM.instr[i].vol;
@@ -950,13 +943,13 @@ static bool loadMusicSTM(FILE *f, uint32_t fileLength, bool fromExternalThread)
 				if (s->repS+s->repL > s->len)
 					s->repL = s->len - s->repS;
 
-				s->typ = 1;
+				s->typ = 1; // enable loop
 			}
 			else
 			{
 				s->repS = 0;
 				s->repL = 0;
-				s->typ  = 0;
+				s->typ = 0;
 			}
 
 			if (s->vol > 64)
@@ -972,75 +965,6 @@ static bool loadMusicSTM(FILE *f, uint32_t fileLength, bool fromExternalThread)
 		}
 	}
 
-	// non-FT2: fix overflown 9xx and illegal 3xx
-
-	for (i = 0; i < ap; i++)
-	{
-		if (pattTmp[i] == NULL)
-			continue;
-
-		for (k = 0; k < songTmp.antChn; k++)
-		{
-			check3xx = false;
-			for (j = 0; j < 64; j++)
-			{
-				ton = &pattTmp[i][(j * MAX_VOICES) + k];
-
-				if (ton->ton > 0 && ton->ton < 97 && ton->effTyp != 0x3)
-					check3xx = true;
-
-				if (ton->ton > 0 && ton->ton < 97 && ton->effTyp == 0x3)
-					check3xx = false;
-
-				if (check3xx && ton->effTyp == 0x3)
-				{
-					if (ton->ton == 0 || ton->ton == 97)
-					{
-						ton->effTyp = 0;
-						ton->eff = 0;
-					}
-				}
-
-				if (ton->effTyp == 0x9 && ton->eff > 0)
-				{
-					if (ton->instr != 0 && ton->instr <= ai)
-					{
-						s = &instrTmp[ton->instr]->samp[0];
-						len = s->len;
-
-						tmp8 = 0;
-						if (len > 0)
-						{
-							tmp8 = ton->eff;
-							if (tmp8 >= len/256)
-							{
-								if (len/256 < 1)
-									tmp8 = 0;
-								else
-									tmp8 = (uint8_t)((len/256) - 1);
-							}
-						}
-
-						if (tmp8 > 0)
-						{
-							ton->eff = tmp8;
-						}
-						else
-						{
-							ton->effTyp = 0;
-							ton->eff = 0;
-						}
-					}
-					else
-					{
-						ton->effTyp = 0;
-						ton->eff = 0;
-					}
-				}
-			}
-		}
-	}
-
 	fclose(f);
 
 	moduleLoaded = true;
@@ -1086,14 +1010,13 @@ static int8_t countS3MChannels(uint16_t antPtn)
 static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 {
 	int8_t *tmpSmp;
-	bool check3xx, illegalUxx;
 	uint8_t ha[2048];
-	uint8_t s3mLastDEff[32], s3mLastEEff[32], s3mLastFEff[32];
-	uint8_t s3mLastSEff[32], s3mLastJEff[32], s3mLastGInstr[32], typ;
-	int16_t i, j, k, ai, ap, ver, ii, kk, tmp;
+	uint8_t alastnfo[32], alastefx[32], alastvibnfo[32], s3mLastGInstr[32];
+	uint8_t typ;
+	int16_t ai, ap, ver, ii, kk, tmp;
 	uint16_t ptnOfs[256];
-	int32_t len;
-	tonTyp ton, *pattTon;
+	int32_t i, j, k, len;
+	tonTyp ton;
 	sampleTyp *s;
 	songS3MHeaderTyp h_S3M;
 	songS3MinstrHeaderTyp h_S3MInstr;
@@ -1101,8 +1024,6 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 
 	showMsg = fromExternalThread ? okBoxThreadSafe : okBox;
 
-	stereoSamplesWarn = false;
-
 	rewind(f);
 
 	// start loading S3M
@@ -1129,13 +1050,7 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 		goto s3mLoadError;
 	}
 
-	// count real song table entries
-	songTmp.len = 256;
-	while (songTmp.len > 0 && songTmp.songTab[songTmp.len-1] == 255)
-		songTmp.len--;
-
-	if (songTmp.len == 256)
-		songTmp.len = 255;
+	songTmp.len = h_S3M.songTabLen;
 
 	// remove pattern separators (254)
 	k = 0;
@@ -1149,13 +1064,22 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 	}
 
 	if (k <= songTmp.len)
-		songTmp.len -= k;
+		songTmp.len -= (uint16_t)k;
 	else
-		songTmp.len = 0;
+		songTmp.len = 1;
+
+	for (i = 1; i < songTmp.len; i++)
+	{
+		if (songTmp.songTab[i] == 255)
+		{
+			songTmp.len = (uint16_t)i;
+			break;
+		}
+	}
 	
 	// clear unused song table entries
 	if (songTmp.len < 255)
-		memset(&songTmp.songTab[songTmp.len], 0, 256 - songTmp.len);
+		memset(&songTmp.songTab[songTmp.len], 0, 255-songTmp.len);
 
 	songTmp.speed = h_S3M.defTempo;
 	if (songTmp.speed < 32)
@@ -1182,8 +1106,8 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 	memcpy(songTmp.name, h_S3M.name, 20);
 	songTmp.name[20] = '\0';
 
-	ap  = h_S3M.antPatt;
-	ai  = h_S3M.antInstr;
+	ap = h_S3M.antPatt;
+	ai = h_S3M.antInstr;
 	ver = h_S3M.ver;
 
 	k = 31;
@@ -1204,19 +1128,15 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 
 	// *** PATTERNS ***
 
-	illegalUxx = false;
-
 	k = 0;
 	for (i = 0; i < ap; i++)
 	{
 		if (ptnOfs[i]  == 0)
 			continue; // empty pattern
 
-		memset(s3mLastDEff, 0, sizeof (s3mLastDEff));
-		memset(s3mLastEEff, 0, sizeof (s3mLastEEff));
-		memset(s3mLastFEff, 0, sizeof (s3mLastFEff));
-		memset(s3mLastSEff, 0, sizeof (s3mLastSEff));
-		memset(s3mLastJEff, 0, sizeof (s3mLastJEff));
+		memset(alastnfo, 0, sizeof (alastnfo));
+		memset(alastefx, 0, sizeof (alastefx));
+		memset(alastvibnfo, 0, sizeof (alastvibnfo));
 		memset(s3mLastGInstr, 0, sizeof (s3mLastGInstr));
 
 		fseek(f, ptnOfs[i] << 4, SEEK_SET);
@@ -1281,7 +1201,7 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 						}
 					}
 
-					// volume
+					// volume column
 					if (typ & 64)
 					{
 						ton.vol = pattBuff[k++];
@@ -1298,28 +1218,42 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 						ton.effTyp = pattBuff[k++];
 						ton.eff = pattBuff[k++];
 
-						if (ton.eff == 0)
+						if (ton.eff > 0)
 						{
-							if (ton.effTyp == 4)
+							alastnfo[ii] = ton.eff;
+							if (ton.effTyp == 8 || ton.effTyp == 21)
+								alastvibnfo[ii] = ton.eff; // H/U
+						}
+
+						// in ST3, a lot of effects directly share the same memory!
+						if (ton.eff == 0 && ton.effTyp != 7) // G
+						{
+							uint8_t efx = ton.effTyp;
+							if (efx == 8 || efx == 21) // H/U
+								ton.eff = alastvibnfo[ii];
+							else if ((efx >= 4 && efx <= 12) || (efx >= 17 && efx <= 19)) // D/E/F/I/J/K/L/Q/R/S
+								ton.eff = alastnfo[ii];
+
+							/* If effect data is zero and effect type was the same as last one, clear out
+							** data if it's not J or S (those have no memory in the equivalent XM effects).
+							** Also goes for extra fine pitch slides and fine volume slides,
+							** since they get converted to other effects.
+							*/
+							if (efx == alastefx[ii] && ton.effTyp != 10 && ton.effTyp != 19) // J/S
 							{
-								if ((s3mLastDEff[ii] & 0xF0) == 0xF0 || (s3mLastDEff[ii] & 0x0F) == 0x0F)
-									ton.eff = s3mLastDEff[ii];
+								uint8_t nfo = ton.eff;
+								bool extraFinePitchSlides = (efx == 5 || efx == 6) && ((nfo & 0xF0) == 0xE0);
+								bool fineVolSlides = (efx == 4 || efx == 11) &&
+								     ((nfo > 0xF0) || (((nfo & 0xF) == 0xF) && ((nfo & 0xF0) > 0)));
+
+								if (!extraFinePitchSlides && !fineVolSlides)
+									ton.eff = 0;
 							}
-							else if (ton.effTyp == 5) ton.eff = s3mLastEEff[ii];
-							else if (ton.effTyp == 6) ton.eff = s3mLastFEff[ii];
-							else if (ton.effTyp == 10) ton.eff = s3mLastJEff[ii];
-							else if (ton.effTyp == 19) ton.eff = s3mLastSEff[ii];
-						}
-						
-						if (ton.eff != 0)
-						{
-							     if (ton.effTyp == 4) s3mLastDEff[ii] = ton.eff;
-							else if (ton.effTyp == 5) s3mLastEEff[ii] = ton.eff;
-							else if (ton.effTyp == 6) s3mLastFEff[ii] = ton.eff;
-							else if (ton.effTyp == 10) s3mLastJEff[ii] = ton.eff;
-							else if (ton.effTyp == 19) s3mLastSEff[ii] = ton.eff;
 						}
 
+						if (ton.effTyp > 0)
+							alastefx[ii] = ton.effTyp;
+
 						switch (ton.effTyp)
 						{
 							case 1: // A
@@ -1337,14 +1271,12 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 							case 3: ton.effTyp = 0xD; break; // C
 							case 4: // D
 							{
-								     if ((ton.eff & 0xF0) == 0) ton.effTyp = 0xA;
-								else if ((ton.eff & 0x0F) == 0) ton.effTyp = 0xA;
-								else if ((ton.eff & 0xF0) == 0xF0)
+								if (ton.eff > 0xF0) // fine slide up
 								{
 									ton.effTyp = 0xE;
-									ton.eff = 0xB0 | (ton.eff & 15);
+									ton.eff = 0xB0 | (ton.eff & 0xF);
 								}
-								else if ((ton.eff & 0x0F) == 0x0F)
+								else if ((ton.eff & 0x0F) == 0x0F && (ton.eff & 0xF0) > 0) // fine slide down
 								{
 									ton.effTyp = 0xE;
 									ton.eff = 0xA0 | (ton.eff >> 4);
@@ -1352,10 +1284,8 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 								else
 								{
 									ton.effTyp = 0xA;
-									if (ton.eff & 0x0F)
+									if (ton.eff & 0x0F) // on D/K, last nybble has first priority in ST3
 										ton.eff &= 0x0F;
-									else
-										ton.eff &= 0xF0;
 								}
 							}
 							break;
@@ -1365,6 +1295,7 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 							{
 								if ((ton.eff & 0xF0) >= 0xE0)
 								{
+									// convert to fine slide
 									if ((ton.eff & 0xF0) == 0xE0)
 										tmp = 0x21;
 									else
@@ -1378,9 +1309,15 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 										ton.eff |= 0x10;
 
 									ton.effTyp = (uint8_t)tmp;
+
+									if (ton.effTyp == 0x21 && ton.eff == 0)
+									{
+										ton.effTyp = 0;
+									}
 								}
 								else
 								{
+									// convert to normal 1xx/2xx slide
 									ton.effTyp = 7 - ton.effTyp;
 								}
 							}
@@ -1388,18 +1325,46 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 
 							case 7: // G
 							{
+								ton.effTyp = 0x03;
+
 								// fix illegal slides (to new instruments)
 								if (ton.instr != 0 && ton.instr != s3mLastGInstr[ii])
 									ton.instr = s3mLastGInstr[ii];
+							}
+							break;
 
-								ton.effTyp = 0x03;
+							case 11: // K
+							{
+								if (ton.eff > 0xF0) // fine slide up
+								{
+									ton.effTyp = 0xE;
+									ton.eff = 0xB0 | (ton.eff & 0xF);
+
+									// if volume column is unoccupied, set to vibrato
+									if (ton.vol == 0)
+										ton.vol = 0xB0;
+								}
+								else if ((ton.eff & 0x0F) == 0x0F && (ton.eff & 0xF0) > 0) // fine slide down
+								{
+									ton.effTyp = 0xE;
+									ton.eff = 0xA0 | (ton.eff >> 4);
+
+									// if volume column is unoccupied, set to vibrato
+									if (ton.vol == 0)
+										ton.vol = 0xB0;
+								}
+								else
+								{
+									ton.effTyp = 0x6;
+									if (ton.eff & 0x0F) // on D/K, last nybble has first priority in ST3
+										ton.eff &= 0x0F;
+								}
 							}
 							break;
 
 							case 8: ton.effTyp = 0x04; break; // H
 							case 9: ton.effTyp = 0x1D; break; // I
 							case 10: ton.effTyp = 0x00; break; // J
-							case 11: ton.effTyp = 0x06; break; // K
 							case 12: ton.effTyp = 0x05; break; // L
 							case 15: ton.effTyp = 0x09; break; // O
 							case 17: ton.effTyp = 0x1B; break; // Q
@@ -1415,10 +1380,37 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 								else if (tmp == 0x2) ton.eff |= 0x50;
 								else if (tmp == 0x3) ton.eff |= 0x40;
 								else if (tmp == 0x4) ton.eff |= 0x70;
-								// we ignore S8x (set 4-bit pan) becuase it's not compatible with FT2 panning
+								// ignore S8x becuase it's not compatible with FT2 panning
 								else if (tmp == 0xB) ton.eff |= 0x60;
-								else if (tmp == 0xC) ton.eff |= 0xC0;
-								else if (tmp == 0xD) ton.eff |= 0xD0;
+								else if (tmp == 0xC) // Note Cut
+								{
+									ton.eff |= 0xC0;
+									if (ton.eff == 0xC0)
+									{
+										// EC0 does nothing in ST3 but cuts voice in FT2, remove effect
+										ton.effTyp = 0;
+										ton.eff = 0;
+									}
+								}
+								else if (tmp == 0xD) // Note Delay
+								{
+									ton.eff |= 0xD0;
+									if (ton.ton == 0 || ton.ton == 97)
+									{
+										// EDx without a note does nothing in ST3 but retrigs in FT2, remove effect
+										ton.effTyp = 0;
+										ton.eff = 0;
+									}
+									else if (ton.eff == 0xD0)
+									{
+										// ED0 prevents note/smp/vol from updating in ST3, remove everything
+										ton.ton = 0;
+										ton.instr = 0;
+										ton.vol = 0;
+										ton.effTyp = 0;
+										ton.eff = 0;
+									}
+								}
 								else if (tmp == 0xE) ton.eff |= 0xE0;
 								else if (tmp == 0xF) ton.eff |= 0xF0;
 								else
@@ -1440,40 +1432,18 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 							}
 							break;
 
-							case 21: // U (fine vibrato, doesn't exist in FT2, convert to normal vibrato)
+							case 22: // V
 							{
-								if ((ton.eff & 0x0F) != 0)
+								ton.effTyp = 0x10;
+								if (ton.eff > 0x40)
 								{
-									ton.eff = (ton.eff & 0xF0) | (((ton.eff & 15) + 1) / 4); // divide depth by 4
-									if ((ton.eff & 0x0F) == 0) // depth too low, remove effect
-									{
-										illegalUxx = true;
-										ton.effTyp = 0;
-										ton.eff = 0;
-									}
-									else
-									{
-										illegalUxx = false;
-										ton.effTyp = 0x04;
-									}
-								}
-								else
-								{
-									if (!illegalUxx)
-									{
-										ton.effTyp = 0x04;
-									}
-									else
-									{
-										ton.effTyp = 0;
-										ton.eff = 0;
-									}
+									// Vxx > 0x40 does nothing in ST3
+									ton.effTyp = 0;
+									ton.eff = 0;
 								}
 							}
 							break;
 
-							case 22: ton.effTyp = 0x10; break; // V
-
 							default:
 							{
 								ton.effTyp = 0;
@@ -1486,40 +1456,6 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 					if (ton.instr != 0 && ton.effTyp != 0x3)
 						s3mLastGInstr[ii] = ton.instr;
 
-					// EDx with no note does nothing in ST3 but retrigs in FT2, remove effect
-					if (ton.effTyp == 0xE && (ton.eff & 0xF0) == 0xD0)
-					{
-						if (ton.ton == 0 || ton.ton == 97)
-						{
-							ton.effTyp = 0;
-							ton.eff = 0;
-						}
-					}
-
-					// EDx with a zero will prevent note/instr/vol from updating in ST3, remove everything
-					if (ton.effTyp == 0xE && ton.eff == 0xD0)
-					{
-						ton.ton = 0;
-						ton.instr = 0;
-						ton.vol = 0;
-						ton.effTyp = 0;
-						ton.eff = 0;
-					}
-
-					// ECx with a zero does nothing in ST3 but cuts voice in FT2, remove effect
-					if (ton.effTyp == 0xE && ton.eff == 0xC0)
-					{
-						ton.effTyp = 0;
-						ton.eff = 0;
-					}
-
-					// Vxx with a value higher than 64 (0x40) does nothing in ST3, remove effect
-					if (ton.effTyp == 0x10 && ton.eff > 0x40)
-					{
-						ton.effTyp = 0;
-						ton.eff = 0;
-					}
-
 					if (ton.effTyp > 35)
 					{
 						ton.effTyp = 0;
@@ -1530,7 +1466,7 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 				}
 			}
 
-			if (tmpPatternEmpty(i))
+			if (tmpPatternEmpty((uint16_t)i))
 			{
 				if (pattTmp[i] != NULL)
 				{
@@ -1580,7 +1516,7 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 			}
 			else if (h_S3MInstr.memSeg > 0 && h_S3MInstr.len > 0)
 			{
-				if (!allocateTmpInstr(1 + i))
+				if (!allocateTmpInstr((int16_t)(1 + i)))
 				{
 					showMsg(0, "System message", "Not enough memory!");
 					goto s3mLoadError;
@@ -1590,15 +1526,16 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 				s = &instrTmp[1+i]->samp[0];
 
 				len = h_S3MInstr.len;
+				
+				bool hasLoop = h_S3MInstr.flags & 1;
+				bool stereoSample = (h_S3MInstr.flags >> 1) & 1;
+				bool is16Bit = (h_S3MInstr.flags >> 2) & 1;
+			
+				if (is16Bit) // 16-bit
+					len <<= 1;
 
-				if ((h_S3MInstr.flags & 2) != 0) // stereo
-				{
-					stereoSamplesWarn = false;
-					len *= 2;
-				}
-
-				if ((h_S3MInstr.flags & 4) != 0) // 16-bit
-					len *= 2;
+				if (stereoSample) // stereo
+					len <<= 1;
 
 				tmpSmp = (int8_t *)malloc(len + LOOP_FIX_LEN);
 				if (tmpSmp == NULL)
@@ -1607,15 +1544,17 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 					goto s3mLoadError;
 				}
 
+				int8_t *newPtr = tmpSmp + SMP_DAT_OFFSET;
+
 				memcpy(s->name, h_S3MInstr.name, 21);
 
-				if (h_S3MInstr.c2Spd > 65535)
+				if (h_S3MInstr.c2Spd > 65535) // ST3 (and OpenMPT) does this
 					h_S3MInstr.c2Spd = 65535;
 
 				tuneSample(s, h_S3MInstr.c2Spd);
 
-				s->len  = h_S3MInstr.len;
-				s->vol  = h_S3MInstr.vol;
+				s->len = h_S3MInstr.len;
+				s->vol = h_S3MInstr.vol;
 				s->repS = h_S3MInstr.repS;
 				s->repL = h_S3MInstr.repE - h_S3MInstr.repS;
 
@@ -1627,12 +1566,13 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 				{
 					s->repS = 0;
 					s->repL = 0;
+					hasLoop = false;
 				}
 
-				s->typ = (h_S3MInstr.flags & 1) + ((h_S3MInstr.flags & 4) << 2);
-
 				if (s->repL == 0)
-					s->typ &= 16; // turn off loop, keep 16-bit flag only
+					hasLoop = false;
+
+				s->typ = hasLoop + (is16Bit << 4);
 
 				if (s->vol > 64)
 					s->vol = 64;
@@ -1645,119 +1585,51 @@ static bool loadMusicS3M(FILE *f, uint32_t dataLength, bool fromExternalThread)
 				if ((h_S3MInstr.memSeg<<4)+len > (int32_t)dataLength)
 					len = dataLength - (h_S3MInstr.memSeg << 4);
 
-				if (fread(tmpSmp, len, 1, f) != 1)
+				if (ver == 1)
 				{
-					free(tmpSmp);
-					showMsg(0, "System message", "General I/O error during loading! Is the file in use?");
-					goto s3mLoadError;
+					fseek(f, len, SEEK_CUR); // sample not supported
 				}
-
-				if (ver != 1)
+				else
 				{
-					if ((h_S3MInstr.flags & 4) != 0)
+					if (fread(newPtr, len, 1, f) != 1)
 					{
-						conv16BitSample(tmpSmp, len, h_S3MInstr.flags & 2);
+						free(tmpSmp);
+						showMsg(0, "System message", "General I/O error during loading! Is the file in use?");
+						goto s3mLoadError;
+					}
 
-						s->pek = (int8_t *)malloc((h_S3MInstr.len * 2) + LOOP_FIX_LEN);
-						if (s->pek == NULL)
-						{
-							free(tmpSmp);
-							showMsg(0, "System message", "Not enough memory!");
-							goto s3mLoadError;
-						}
+					if (is16Bit)
+					{
+						conv16BitSample(newPtr, len, stereoSample);
 
-						memcpy(s->pek, tmpSmp, h_S3MInstr.len * 2);
+						s->origPek = tmpSmp;
+						s->pek = s->origPek + SMP_DAT_OFFSET;
 
-						s->len *= 2;
-						s->repS *= 2;
-						s->repL *= 2;
+						s->len <<= 1;
+						s->repS <<= 1;
+						s->repL <<= 1;
 					}
 					else
 					{
-						conv8BitSample(tmpSmp, len, h_S3MInstr.flags & 2);
+						conv8BitSample(newPtr, len, stereoSample);
 
-						s->pek = (int8_t *)malloc(h_S3MInstr.len + LOOP_FIX_LEN);
-						if (s->pek == NULL)
+						s->origPek = tmpSmp;
+						s->pek = s->origPek + SMP_DAT_OFFSET;
+					}
+
+					// if stereo sample: reduce memory footprint after sample was downmixed to mono
+					if (stereoSample)
+					{
+						newPtr = (int8_t *)realloc(s->origPek, s->len + LOOP_FIX_LEN);
+						if (newPtr != NULL)
 						{
-							free(tmpSmp);
-							showMsg(0, "System message", "Not enough memory!");
-							goto s3mLoadError;
+							s->origPek = newPtr;
+							s->pek = s->origPek + SMP_DAT_OFFSET;
 						}
-
-						memcpy(s->pek, tmpSmp, h_S3MInstr.len);
 					}
 
 					fixSample(s);
 				}
-
-				free(tmpSmp);
-			}
-		}
-	}
-
-	if (stereoSamplesWarn)
-		showMsg(0, "System message", "Stereo samples were found and will be converted to mono.");
-
-	// non-FT2: fix overflown 9xx and illegal 3xx slides
-
-	for (i = 0; i < ap; i++)
-	{
-		if (pattTmp[i] == NULL)
-			continue;
-
-		for (k = 0; k < songTmp.antChn; k++)
-		{
-			check3xx = false;
-			for (j = 0; j < 64; j++)
-			{
-				pattTon = &pattTmp[i][(j * MAX_VOICES) + k];
-
-				// fix illegal 3xx slides
-
-				if (pattTon->ton > 0 && pattTon->ton < 97)
-					check3xx = pattTon->effTyp != 0x3;
-
-				if (check3xx && pattTon->effTyp == 0x3)
-				{
-					if (pattTon->ton == 0 || pattTon->ton == 97)
-					{
-						pattTon->effTyp = 0;
-						pattTon->eff = 0;
-					}
-				}
-
-				/* In ST3 in GUS mode, an overflowed sample offset behaves like this:
-				** - Non-looped sample: Cut voice
-				** - Looped sample: Wrap around loop point
-				**
-				** What we do here is to change the sample offset value to point to
-				** the wrapped sample loop position. This may be off by up to 256 bytes
-				** though...
-				*/
-
-				if (pattTon->effTyp == 0x9 && pattTon->eff > 0 && pattTon->instr > 0 && pattTon->instr <= ai && ai <= 128)
-				{
-					s = &instrTmp[pattTon->instr]->samp[0];
-					if (s->len > 0 && (s->typ & 1)) // only handle non-empty looping samples
-					{
-						uint32_t loopEnd = s->repS + s->repL;
-						uint32_t offset = pattTon->eff * 256;
-
-						if (offset >= loopEnd)
-						{
-							if (s->repL >= 2)
-								offset = s->repS + ((offset - loopEnd) % s->repL);
-							else
-								offset = s->repS;
-
-							offset = (offset + (1 << 7)) >> 8; // convert to rounded sample offset value
-							if (offset > 255)
-								offset = 255;
-
-							pattTon->eff = (uint8_t)offset;
-						}
-					}
-				}
 			}
 		}
 	}
@@ -1791,7 +1663,6 @@ bool doLoadMusic(bool fromExternalThread)
 
 	showMsg = fromExternalThread ? okBoxThreadSafe : okBox;
 
-	stereoSamplesWarn = false;
 	linearFreqTable = false;
 
 	if (editor.tmpFilenameU == NULL)
@@ -1877,14 +1748,13 @@ bool doLoadMusic(bool fromExternalThread)
 	songTmp.len = h.len;
 	songTmp.repS = h.repS;
 	songTmp.antChn = (uint8_t)h.antChn;
-	songTmp.speed = h.defSpeed ? h.defSpeed : 125;
-	songTmp.tempo = h.defTempo ? h.defTempo : 6;
+	songTmp.speed = h.defSpeed;
+	songTmp.tempo = h.defTempo;
 	songTmp.ver = h.ver;
 	linearFreqTable = h.flags & 1;
 
-	songTmp.speed = CLAMP(songTmp.speed, 32, 255);
-	if (songTmp.tempo > 31)
-		songTmp.tempo = 31;
+	songTmp.speed = CLAMP(songTmp.speed, 1, 999);
+	songTmp.tempo = CLAMP(songTmp.tempo, 1, 99);
 
 	songTmp.initialTempo = songTmp.tempo;
 
@@ -1950,9 +1820,6 @@ bool doLoadMusic(bool fromExternalThread)
 		}
 	}
 
-	if (stereoSamplesWarn)
-		showMsg(0, "System message", "Stereo samples were found and will be converted to mono.");
-
 	fclose(f);
 
 	moduleLoaded = true;
@@ -2141,11 +2008,11 @@ static bool loadInstrHeader(FILE *f, uint16_t i)
 			ins->vibDepth = ih.vibDepth;
 			ins->vibRate = ih.vibRate;
 			ins->fadeOut = ih.fadeOut;
-			ins->midiOn = (ih.midiOn > 0) ? true : false;
+			ins->midiOn = (ih.midiOn == 1) ? true : false;
 			ins->midiChannel = ih.midiChannel;
 			ins->midiProgram = ih.midiProgram;
 			ins->midiBend = ih.midiBend;
-			ins->mute = (ih.mute > 0) ? true : false;
+			ins->mute = (ih.mute == 1) ? true : false;
 			ins->antSamp = ih.antSamp; // used in loadInstrSample()
 
 			// sanitize stuff for broken/unsupported instruments
@@ -2178,7 +2045,6 @@ static bool loadInstrHeader(FILE *f, uint16_t i)
 				if ((uint16_t)ins->envPP[j][0] > 32767) ins->envPP[j][0] = 32767;
 				if ((uint16_t)ins->envVP[j][1] > 64) ins->envVP[j][1] = 64;
 				if ((uint16_t)ins->envPP[j][1] > 63) ins->envPP[j][1] = 63;
-				
 			}
 		}
 
@@ -2276,10 +2142,13 @@ static bool loadInstrSample(FILE *f, uint16_t i)
 				l = MAX_SAMPLE_LEN;
 			}
 
-			s->pek = (int8_t *)malloc(l + LOOP_FIX_LEN);
-			if (s->pek == NULL)
+			s->pek = NULL;
+			s->origPek = (int8_t *)malloc(l + LOOP_FIX_LEN);
+			if (s->origPek == NULL)
 				return false;
 
+			s->pek = s->origPek + SMP_DAT_OFFSET;
+
 			int32_t bytesRead = (int32_t)fread(s->pek, 1, l, f);
 			if (bytesRead < l)
 			{
@@ -2296,15 +2165,16 @@ static bool loadInstrSample(FILE *f, uint16_t i)
 			{
 				s->typ &= ~32; // remove stereo flag
 
-				s->len /= 2;
-				s->repL /= 2;
-				s->repS /= 2;
+				s->len >>= 1;
+				s->repL >>= 1;
+				s->repS >>= 1;
 
-				newPtr = (int8_t *)realloc(s->pek, s->len + LOOP_FIX_LEN);
+				newPtr = (int8_t *)realloc(s->origPek, s->len + LOOP_FIX_LEN);
 				if (newPtr != NULL)
-					s->pek = newPtr;
-
-				stereoSamplesWarn = true;
+				{
+					s->origPek = newPtr;
+					s->pek = s->origPek + SMP_DAT_OFFSET;
+				}
 			}
 		}
 
@@ -2323,15 +2193,14 @@ static bool loadInstrSample(FILE *f, uint16_t i)
 	return true;
 }
 
-void unpackPatt(uint8_t *dst, uint16_t inn, uint16_t len, uint8_t antChn)
+void unpackPatt(uint8_t *dst, uint8_t *src, uint16_t len, int32_t antChn)
 {
-	uint8_t note, data, *src;
+	uint8_t note, data;
 	int32_t srcEnd, srcIdx;
 
 	if (dst == NULL)
 		return;
 
-	src = dst + inn;
 	srcEnd = len * TRACK_WIDTH;
 	srcIdx = 0;
 
@@ -2399,7 +2268,7 @@ static bool tmpPatternEmpty(uint16_t nr)
 	return true;
 }
 
-void clearUnusedChannels(tonTyp *p, int16_t pattLen, uint8_t antChn)
+void clearUnusedChannels(tonTyp *p, int16_t pattLen, int32_t antChn)
 {
 	if (p == NULL || antChn >= MAX_VOICES)
 		return;
@@ -2411,8 +2280,8 @@ void clearUnusedChannels(tonTyp *p, int16_t pattLen, uint8_t antChn)
 static bool loadPatterns(FILE *f, uint16_t antPtn)
 {
 	bool pattLenWarn;
-	uint8_t tmpLen, *pattPtr;
-	uint16_t i, a;
+	uint8_t tmpLen;
+	uint16_t i;
 	patternHeaderTyp ph;
 
 	pattLenWarn = false;
@@ -2455,6 +2324,11 @@ static bool loadPatterns(FILE *f, uint16_t antPtn)
 			goto pattCorrupt;
 
 		pattLensTmp[i] = ph.pattLen;
+		if (pattLensTmp[i] > MAX_PATT_LEN)
+		{
+			pattLensTmp[i] = MAX_PATT_LEN;
+			pattLenWarn = true;
+		}
 
 		if (ph.dataLen > 0)
 		{
@@ -2465,15 +2339,11 @@ static bool loadPatterns(FILE *f, uint16_t antPtn)
 				return false;
 			}
 
-			a = ph.pattLen * TRACK_WIDTH;
-
-			pattPtr = (uint8_t *)pattTmp[i];
-			memset(pattPtr, 0, a);
-
-			if (fread(&pattPtr[a - ph.dataLen], 1, ph.dataLen, f) != ph.dataLen)
+			if (fread(packedPattData, 1, ph.dataLen, f) != ph.dataLen)
 				goto pattCorrupt;
 
-			unpackPatt(pattPtr, a - ph.dataLen, ph.pattLen, songTmp.antChn);
+			unpackPatt((uint8_t *)pattTmp[i], packedPattData, pattLensTmp[i], songTmp.antChn);
+
 			clearUnusedChannels(pattTmp[i], pattLensTmp[i], songTmp.antChn);
 		}
 
@@ -2487,16 +2357,10 @@ static bool loadPatterns(FILE *f, uint16_t antPtn)
 
 			pattLensTmp[i] = 64;
 		}
-
-		if (pattLensTmp[i] > 256)
-		{
-			pattLensTmp[i] = 64;
-			pattLenWarn = true;
-		}
 	}
 
 	if (pattLenWarn)
-		okBoxThreadSafe(0, "System message", "The module contains pattern lengths above 256! They will be set to 64.");
+		okBoxThreadSafe(0, "System message", "This module contains pattern(s) with a length above 256! They will be truncated.");
 
 	return true;
 
@@ -2519,9 +2383,6 @@ static void setupLoadedModule(void)
 	playMode = PLAYMODE_IDLE;
 	songPlaying = false;
 
-	editor.currVolEnvPoint = 0;
-	editor.currPanEnvPoint = 0;
-
 #ifdef HAS_MIDI
 	midi.currMIDIVibDepth = 0;
 	midi.currMIDIPitch = 0;
@@ -2552,50 +2413,59 @@ static void setupLoadedModule(void)
 	// support non-even channel numbers
 	if (song.antChn & 1)
 	{
-		if (++song.antChn > MAX_VOICES)
+		song.antChn++;
+		if (song.antChn > MAX_VOICES)
 			song.antChn = MAX_VOICES;
 	}
 
-	if (song.repS > song.len)
+	if (song.len == 0)
+		song.len = 1;
+
+	if (song.repS >= song.len)
 		song.repS = 0;
 
 	song.globVol = 64;
-	song.timer = 1;
 
 	setScrollBarEnd(SB_POS_ED, (song.len - 1) + 5);
 	setScrollBarPos(SB_POS_ED, 0, false);
 
 	resetChannels();
-	refreshScopes();
-	setPos(0, 0, false);
+	setPos(0, 0, true);
 	setSpeed(song.speed);
 
 	editor.tmpPattern = editor.editPattern; // set kludge variable
 	editor.speed = song.speed;
 	editor.tempo = song.tempo;
-	editor.timer = 1;
+	editor.timer = song.timer;
 	editor.globalVol = song.globVol;
 
-	setFrqTab((loadedFormat == FORMAT_XM) ? linearFreqTable : false);
+	if (loadedFormat == FORMAT_XM)
+		setFrqTab(linearFreqTable);
+	else
+		setFrqTab(false);
+
 	unlockMixerCallback();
 
+	editor.currVolEnvPoint = 0;
+	editor.currPanEnvPoint = 0;
+
+	refreshScopes();
 	exitTextEditing();
 	updateTextBoxPointers();
 	resetChannelOffset();
 	updateChanNums();
 	resetWavRenderer();
 	clearPattMark();
-	song.musicTime = 0;
 	resetTrimSizes();
+	resetPlaybackTime();
 
 	diskOpSetFilename(DISKOP_ITEM_MODULE, editor.tmpFilenameU);
 
 	// redraw top part of screen
-	if (editor.ui.extended)
+	if (ui.extended)
 	{
-		// first exit extended mode, then re-enter
-		togglePatternEditorExtended();
-		togglePatternEditorExtended();
+		togglePatternEditorExtended(); // exit
+		togglePatternEditorExtended(); // re-enter (force redrawing)
 	}
 	else
 	{
@@ -2607,8 +2477,8 @@ static void setupLoadedModule(void)
 	updateSampleEditorSample();
 	showBottomScreen(); // redraw bottom screen (also redraws pattern editor)
 
-	if (editor.ui.instEditorShown)
-		drawPiano(); // redraw piano now (since if playing = wait for next tick update)
+	if (ui.instEditorShown)
+		drawPiano(NULL); // redraw piano now (since if playing = wait for next tick update)
 
 	removeSongModifiedFlag();
 
@@ -2680,7 +2550,7 @@ void loadDroppedFile(char *fullPathUTF8, bool songModifiedCheck)
 	int32_t fullPathLen, filesize;
 	UNICHAR *fullPathU;
 
-	if (editor.ui.sysReqShown || fullPathUTF8 == NULL)
+	if (ui.sysReqShown || fullPathUTF8 == NULL)
 		return;
 
 	fullPathLen = (int32_t)strlen(fullPathUTF8);
