@@ -11,6 +11,8 @@
 #include "ft2_mouse.h"
 #include "ft2_sample_ed.h"
 #include "ft2_module_loader.h"
+#include "ft2_tables.h"
+#include "ft2_structs.h"
 
 /* These savers are directly ported, so they should act identical to FT2
 ** except for some very minor changes.
@@ -18,17 +20,20 @@
 
 static SDL_Thread *thread;
 
-static uint16_t packPatt(uint8_t *pattPtr, uint16_t numRows);
+static uint8_t packedPattData[65536];
+static uint16_t packPatt(uint8_t *writePtr, uint8_t *pattPtr, uint16_t numRows);
 
-// ft2_replayer.c
-extern const char modSig[32][5];
-extern const uint16_t amigaPeriod[12*8];
+static const char modSig[32][5] =
+{
+	"1CHN", "2CHN", "3CHN", "4CHN", "5CHN", "6CHN", "7CHN", "8CHN",
+	"9CHN", "10CH", "11CH", "12CH", "13CH", "14CH", "15CH", "16CH",
+	"17CH", "18CH", "19CH", "20CH", "21CH", "22CH", "23CH", "24CH",
+	"25CH", "26CH", "27CH", "28CH", "29CH", "30CH", "31CH", "32CH"
+};
 
 bool saveXM(UNICHAR *filenameU)
 {
-	uint8_t *pattPtr;
 	int16_t ap, ai, i, j, k, a;
-	uint16_t b, c;
 	size_t result;
 	songHeaderTyp h;
 	patternHeaderTyp ph;
@@ -54,7 +59,7 @@ bool saveXM(UNICHAR *filenameU)
 	h.headerSize = 20 + 256;
 	h.len = song.len;
 	h.repS = song.repS;
-	h.antChn = song.antChn;
+	h.antChn = (uint16_t)song.antChn;
 	h.defTempo = song.tempo;
 	h.defSpeed = song.speed;
 
@@ -76,7 +81,7 @@ bool saveXM(UNICHAR *filenameU)
 		ai--;
 	h.antInstrs = ai;
 
-	h.flags = linearFrqTab;
+	h.flags = audio.linearFreqTable;
 	memcpy(h.songTab, song.songTab, sizeof (song.songTab));
 
 	if (fwrite(&h, sizeof (h), 1, f) != 1)
@@ -115,18 +120,10 @@ bool saveXM(UNICHAR *filenameU)
 		}
 		else
 		{
-			c = packPatt((uint8_t *)patt[i], pattLens[i]);
-			b = pattLens[i] * TRACK_WIDTH;
-			ph.dataLen = c;
+			ph.dataLen = packPatt(packedPattData, (uint8_t *)patt[i], pattLens[i]);
 
 			result = fwrite(&ph, ph.patternHeaderSize, 1, f);
-			result += fwrite(patt[i], ph.dataLen, 1, f);
-
-			pattPtr = (uint8_t *)patt[i];
-
-			memcpy(&pattPtr[b-c], patt[i], c);
-			unpackPatt(pattPtr, b - c, pattLens[i], song.antChn);
-			clearUnusedChannels(patt[i], pattLens[i], song.antChn);
+			result += fwrite(packedPattData, ph.dataLen, 1, f);
 
 			if (result != 2) // write was not OK
 			{
@@ -270,7 +267,7 @@ static bool saveMOD(UNICHAR *filenameU)
 	incompatEfx = false;
 	noteUnderflow = false;
 
-	if (linearFrqTab)
+	if (audio.linearFreqTable)
 		okBoxThreadSafe(0, "System message", "Linear frequency table used!");
 
 	// sanity checking
@@ -391,15 +388,15 @@ static bool saveMOD(UNICHAR *filenameU)
 		{
 			smp = &instr[i]->samp[0];
 
-			l1 = smp->len / 2;
-			l2 = smp->repS / 2;
-			l3 = smp->repL / 2;
+			l1 = smp->len >> 1;
+			l2 = smp->repS >> 1;
+			l3 = smp->repL >> 1;
 
 			if (smp->typ & 16)
 			{
-				l1 /= 2;
-				l2 /= 2;
-				l3 /= 2;
+				l1 >>= 1;
+				l2 >>= 1;
+				l3 >>= 1;
 			}
 
 			if (l1 > 32767)
@@ -535,7 +532,7 @@ static bool saveMOD(UNICHAR *filenameU)
 
 		restoreSample(smp);
 
-		l1 = smp->len / 2;
+		l1 = smp->len >> 1;
 		if (smp->typ & 16) // 16-bit sample (convert to 8-bit)
 		{
 			if (l1 > 65534)
@@ -572,7 +569,8 @@ static bool saveMOD(UNICHAR *filenameU)
 
 			if (l1 > 32767)
 				l1 = 32767;
-			l1 *= 2;
+
+			l1 <<= 1;
 
 			if (fwrite(smp->pek, 1, l1, f) != (size_t)l1)
 			{
@@ -632,9 +630,9 @@ void saveMusic(UNICHAR *filenameU)
 	SDL_DetachThread(thread);
 }
 
-static uint16_t packPatt(uint8_t *pattPtr, uint16_t numRows)
+static uint16_t packPatt(uint8_t *writePtr, uint8_t *pattPtr, uint16_t numRows)
 {
-	uint8_t bytes[5], packBits, *writePtr, *firstBytePtr;
+	uint8_t bytes[5], packBits, *firstBytePtr;
 	uint16_t totalPackLen;
 
 	totalPackLen = 0;
@@ -642,7 +640,6 @@ static uint16_t packPatt(uint8_t *pattPtr, uint16_t numRows)
 	if (pattPtr == NULL)
 		return 0;
 
-	writePtr = pattPtr;
 	for (uint16_t row = 0; row < numRows; row++)
 	{
 		for (uint16_t chn = 0; chn < song.antChn; chn++)
@@ -684,7 +681,7 @@ static uint16_t packPatt(uint8_t *pattPtr, uint16_t numRows)
 			totalPackLen += (uint16_t)(writePtr - firstBytePtr); // bytes writen
 		}
 
-		// skip unused channels
+		// skip unused channels (unpacked patterns always have 32 channels)
 		pattPtr += sizeof (tonTyp) * (MAX_VOICES - song.antChn);
 	}
 
