@@ -16,27 +16,29 @@
 #include "ft2_mouse.h"
 #include "ft2_config.h"
 #include "ft2_diskop.h"
-#include "ft2_gfxdata.h"
 #include "ft2_audioselector.h"
 #include "ft2_midi.h"
-#include "ft2_gfxdata.h"
+#include "ft2_bmp.h"
+#include "ft2_structs.h"
 
 #define NUM_CURSORS 6
 
+mouse_t mouse; // globalized
+
 static bool mouseBusyGfxBackwards;
 static int16_t mouseShape;
 static int32_t mouseModeGfxOffs, mouseBusyGfxFrame;
 static SDL_Cursor *cursors[NUM_CURSORS];
 
-static bool setSystemCursor(SDL_Cursor *cursor)
+static bool setSystemCursor(SDL_Cursor *cur)
 {
-	if (cursor == NULL)
+	if (cur == NULL)
 	{
 		SDL_SetCursor(SDL_GetDefaultCursor());
 		return false;
 	}
 
-	SDL_SetCursor(cursor);
+	SDL_SetCursor(cur);
 	return true;
 }
 
@@ -57,19 +59,21 @@ bool createMouseCursors(void) // creates scaled SDL surfaces for current mouse p
 {
 	freeMouseCursors();
 
-	const uint8_t *cursorsSrc = mouseCursors;
+	const uint8_t *cursorsSrc = bmp.mouseCursors;
 	switch (config.mouseType)
 	{
-		case MOUSE_IDLE_SHAPE_NICE:   cursorsSrc += 0 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
-		case MOUSE_IDLE_SHAPE_UGLY:   cursorsSrc += 1 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
-		case MOUSE_IDLE_SHAPE_AWFUL:  cursorsSrc += 2 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
+		case MOUSE_IDLE_SHAPE_NICE: cursorsSrc += 0 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
+		case MOUSE_IDLE_SHAPE_UGLY: cursorsSrc += 1 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
+		case MOUSE_IDLE_SHAPE_AWFUL: cursorsSrc += 2 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
 		case MOUSE_IDLE_SHAPE_USABLE: cursorsSrc += 3 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
 		default: break;
 	}
 
 	for (uint32_t i = 0; i < NUM_CURSORS; i++)
 	{
-		SDL_Surface *surface = SDL_CreateRGBSurface(0, MOUSE_CURSOR_W*video.yScale, MOUSE_CURSOR_H*video.yScale, 32, 0, 0, 0, 0);
+		int32_t scaleFactor = video.yScale;
+
+		SDL_Surface *surface = SDL_CreateRGBSurface(0, MOUSE_CURSOR_W*scaleFactor, MOUSE_CURSOR_H*scaleFactor, 32, 0, 0, 0, 0);
 		if (surface == NULL)
 		{
 			freeMouseCursors();
@@ -87,11 +91,11 @@ bool createMouseCursors(void) // creates scaled SDL surfaces for current mouse p
 
 		const uint8_t *srcPixels8;
 		if (i == 3) // text edit cursor
-			srcPixels8 = &mouseCursors[12 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)];
+			srcPixels8 = &bmp.mouseCursors[12 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)];
 		else if (i == 4) // mouse busy (wall clock)
-			srcPixels8 = &mouseCursorBusyClock[2 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)]; // pick a good still-frame
+			srcPixels8 = &bmp.mouseCursorBusyClock[2 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)]; // pick a good still-frame
 		else if (i == 5) // mouse busy (hourglass)
-			srcPixels8 = &mouseCursorBusyGlass[2 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)]; // pick a good still-frame
+			srcPixels8 = &bmp.mouseCursorBusyGlass[2 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)]; // pick a good still-frame
 		else // normal idle cursor + disk op. "delete/rename" cursors
 			srcPixels8 = &cursorsSrc[i * (4 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H))];
 
@@ -105,10 +109,10 @@ bool createMouseCursors(void) // creates scaled SDL surfaces for current mouse p
 		// blit upscaled cursor to surface
 		for (uint32_t y = 0; y < MOUSE_CURSOR_H; y++)
 		{
-			uint32_t *outX = &dstPixels32[(y * video.yScale) * surface->w];
-			for (uint32_t yScale = 0; yScale < video.yScale; yScale++)
+			uint32_t *outX = &dstPixels32[(y * scaleFactor) * surface->w];
+			for (int32_t yScale = 0; yScale < scaleFactor; yScale++)
 			{
-				for (uint32_t x = 0; x < MOUSE_CURSOR_W; x++)
+				for (int32_t x = 0; x < MOUSE_CURSOR_W; x++)
 				{
 					uint8_t srcPix = srcPixels8[(y * MOUSE_CURSOR_W) + x];
 					if (srcPix != PAL_TRANSPR)
@@ -119,7 +123,7 @@ bool createMouseCursors(void) // creates scaled SDL surfaces for current mouse p
 						else if (srcPix == PAL_BCKGRND)
 							pixel = border;
 
-						for (uint32_t xScale = 0; xScale < video.yScale; xScale++)
+						for (int32_t xScale = 0; xScale < scaleFactor; xScale++)
 							outX[xScale] = pixel;
 					}
 
@@ -164,13 +168,13 @@ void setMousePosToCenter(void)
 {
 	if (video.fullscreen)
 	{
-		mouse.setPosX = video.displayW / 2;
-		mouse.setPosY = video.displayH / 2;
+		mouse.setPosX = video.displayW >> 1;
+		mouse.setPosY = video.displayH >> 1;
 	}
 	else
 	{
-		mouse.setPosX = video.renderW / 2;
-		mouse.setPosY = video.renderH / 2;
+		mouse.setPosX = video.renderW >> 1;
+		mouse.setPosY = video.renderH >> 1;
 	}
 
 	mouse.setPosFlag = true;
@@ -206,7 +210,7 @@ void animateBusyMouse(void)
 			}
 
 			changeSpriteData(SPRITE_MOUSE_POINTER,
-				&mouseCursorBusyClock[(mouseBusyGfxFrame % MOUSE_CLOCK_ANI_FRAMES) * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)]);
+				&bmp.mouseCursorBusyClock[(mouseBusyGfxFrame % MOUSE_CLOCK_ANI_FRAMES) * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)]);
 		}
 	}
 	else
@@ -222,7 +226,7 @@ void animateBusyMouse(void)
 			mouseBusyGfxFrame = (mouseBusyGfxFrame + 1) % MOUSE_GLASS_ANI_FRAMES;
 
 			changeSpriteData(SPRITE_MOUSE_POINTER,
-				&mouseCursorBusyGlass[mouseBusyGfxFrame * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)]);
+				&bmp.mouseCursorBusyGlass[mouseBusyGfxFrame * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)]);
 		}
 	}
 }
@@ -234,20 +238,20 @@ void setMouseShape(int16_t shape)
 	if (editor.busy)
 	{
 		if (config.mouseAnimType == MOUSE_BUSY_SHAPE_CLOCK)
-			gfxPtr = &mouseCursorBusyClock[(mouseBusyGfxFrame % MOUSE_GLASS_ANI_FRAMES) * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)];
+			gfxPtr = &bmp.mouseCursorBusyClock[(mouseBusyGfxFrame % MOUSE_GLASS_ANI_FRAMES) * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)];
 		else
-			gfxPtr = &mouseCursorBusyGlass[(mouseBusyGfxFrame % MOUSE_CLOCK_ANI_FRAMES) * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)];
+			gfxPtr = &bmp.mouseCursorBusyGlass[(mouseBusyGfxFrame % MOUSE_CLOCK_ANI_FRAMES) * (MOUSE_CURSOR_W * MOUSE_CURSOR_H)];
 	}
 	else
 	{
-		gfxPtr = &mouseCursors[mouseModeGfxOffs];
+		gfxPtr = &bmp.mouseCursors[mouseModeGfxOffs];
 		switch (shape)
 		{
-			case MOUSE_IDLE_SHAPE_NICE:   gfxPtr += 0  * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
-			case MOUSE_IDLE_SHAPE_UGLY:   gfxPtr += 1  * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
-			case MOUSE_IDLE_SHAPE_AWFUL:  gfxPtr += 2  * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
-			case MOUSE_IDLE_SHAPE_USABLE: gfxPtr += 3  * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
-			case MOUSE_IDLE_TEXT_EDIT:    gfxPtr += 12 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
+			case MOUSE_IDLE_SHAPE_NICE: gfxPtr += 0 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
+			case MOUSE_IDLE_SHAPE_UGLY: gfxPtr += 1 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
+			case MOUSE_IDLE_SHAPE_AWFUL: gfxPtr += 2 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
+			case MOUSE_IDLE_SHAPE_USABLE: gfxPtr += 3 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
+			case MOUSE_IDLE_TEXT_EDIT: gfxPtr += 12 * (MOUSE_CURSOR_W * MOUSE_CURSOR_H); break;
 			default: return;
 		}
 	}
@@ -285,7 +289,7 @@ static void clearTextEditMouse(void)
 
 static void changeCursorIfOverTextBoxes(void)
 {
-	int16_t i, mx, my;
+	int32_t i, mx, my;
 	textBox_t *t;
 
 	mouse.mouseOverTextBox = false;
@@ -297,7 +301,7 @@ static void changeCursorIfOverTextBoxes(void)
 
 	for (i = 0; i < NUM_TEXTBOXES; i++)
 	{
-		if (editor.ui.sysReqShown && i > 0)
+		if (ui.sysReqShown && i > 0)
 			continue;
 
 		t = &textBoxes[i];
@@ -344,38 +348,32 @@ void setMouseBusy(bool busy) // can be called from other threads
 {
 	if (busy)
 	{
-		editor.ui.setMouseIdle = false;
-		editor.ui.setMouseBusy = true;
+		ui.setMouseIdle = false;
+		ui.setMouseBusy = true;
 	}
 	else
 	{
-		editor.ui.setMouseBusy = false;
-		editor.ui.setMouseIdle = true;
+		ui.setMouseBusy = false;
+		ui.setMouseIdle = true;
 	}
 }
 
 void mouseAnimOn(void)
 {
-	editor.ui.setMouseBusy = false;
-	editor.ui.setMouseIdle = false;
+	ui.setMouseBusy = false;
+	ui.setMouseIdle = false;
 
 	editor.busy = true;
 	setMouseShape(config.mouseAnimType);
-
-	//if (config.specialFlags2 & HARDWARE_MOUSE && cBusy != NULL)
-	//	SDL_SetCursor(cBusy);
 }
 
 void mouseAnimOff(void)
 {
-	editor.ui.setMouseBusy = false;
-	editor.ui.setMouseIdle = false;
+	ui.setMouseBusy = false;
+	ui.setMouseIdle = false;
 
 	editor.busy = false;
 	setMouseShape(config.mouseType);
-
-	//if (config.specialFlags2 & HARDWARE_MOUSE && cArrow != NULL)
-	//	SDL_SetCursor(cArrow);
 }
 
 static void mouseWheelDecRow(void)
@@ -408,10 +406,10 @@ static void mouseWheelIncRow(void)
 
 void mouseWheelHandler(bool directionUp)
 {
-	if (editor.ui.sysReqShown || editor.editTextFlag)
+	if (ui.sysReqShown || editor.editTextFlag)
 		return;
 
-	if (editor.ui.extended)
+	if (ui.extended)
 	{
 		if (mouse.y <= 52)
 		{
@@ -430,7 +428,7 @@ void mouseWheelHandler(bool directionUp)
 	{
 		// top screens
 
-		if (editor.ui.helpScreenShown)
+		if (ui.helpScreenShown)
 		{
 			// help screen
 
@@ -445,7 +443,7 @@ void mouseWheelHandler(bool directionUp)
 				helpScrollDown();
 			}
 		}
-		else if (editor.ui.diskOpShown)
+		else if (ui.diskOpShown)
 		{
 			// disk op - 3x speed
 			if (mouse.x <= 355)
@@ -464,7 +462,7 @@ void mouseWheelHandler(bool directionUp)
 				}
 			}
 		}
-		else if (editor.ui.configScreenShown)
+		else if (ui.configScreenShown)
 		{
 			if (editor.currConfigScreen == CONFIG_SCREEN_IO_DEVICES)
 			{
@@ -487,15 +485,15 @@ void mouseWheelHandler(bool directionUp)
 #endif
 		}
 
-		if (!editor.ui.aboutScreenShown  && !editor.ui.helpScreenShown &&
-			!editor.ui.configScreenShown && !editor.ui.nibblesShown)
+		if (!ui.aboutScreenShown  && !ui.helpScreenShown &&
+			!ui.configScreenShown && !ui.nibblesShown)
 		{
 			if (mouse.x >= 421 && mouse.y <= 173)
 			{
 				     if (mouse.y <= 93) directionUp ? decCurIns() : incCurIns();
 				else if (mouse.y >= 94) directionUp ? decCurSmp() : incCurSmp();
 			}
-			else if (!editor.ui.diskOpShown && mouse.x <= 111 && mouse.y <= 76)
+			else if (!ui.diskOpShown && mouse.x <= 111 && mouse.y <= 76)
 			{
 				directionUp ? decSongPos() : incSongPos();
 			}
@@ -505,12 +503,12 @@ void mouseWheelHandler(bool directionUp)
 	{
 		// bottom screens
 
-		if (editor.ui.sampleEditorShown)
+		if (ui.sampleEditorShown)
 		{
 			if (mouse.y >= 174 && mouse.y <= 328)
 				directionUp ? mouseZoomSampleDataIn() : mouseZoomSampleDataOut();
 		}
-		else if (editor.ui.patternEditorShown)
+		else if (ui.patternEditorShown)
 		{
 			directionUp ? mouseWheelDecRow() : mouseWheelIncRow();
 		}
@@ -519,7 +517,7 @@ void mouseWheelHandler(bool directionUp)
 
 static bool testSamplerDataMouseDown(void)
 {
-	if (editor.ui.sampleEditorShown && mouse.y >= 174 && mouse.y <= 327 && editor.ui.sampleDataOrLoopDrag == -1)
+	if (ui.sampleEditorShown && mouse.y >= 174 && mouse.y <= 327 && ui.sampleDataOrLoopDrag == -1)
 	{
 		handleSampleDataMouseDown(false);
 		return true;
@@ -532,10 +530,10 @@ static bool testPatternDataMouseDown(void)
 {
 	uint16_t y1, y2;
 
-	if (editor.ui.patternEditorShown)
+	if (ui.patternEditorShown)
 	{
-		y1 = editor.ui.extended ? 56 : 176;
-		y2 = editor.ui.pattChanScrollShown ? 382 : 396;
+		y1 = ui.extended ? 56 : 176;
+		y2 = ui.pattChanScrollShown ? 382 : 396;
 
 		if (mouse.y >= y1 && mouse.y <= y2 && mouse.x >= 29 && mouse.x <= 602)
 		{
@@ -549,26 +547,21 @@ static bool testPatternDataMouseDown(void)
 
 void mouseButtonUpHandler(uint8_t mouseButton)
 {
-#ifndef __APPLE__
-	if (!video.fullscreen) // release mouse button trap
-		SDL_SetWindowGrab(video.window, SDL_FALSE);
-#endif
-
 	if (mouseButton == SDL_BUTTON_LEFT)
 	{
 		mouse.leftButtonPressed = false;
 		mouse.leftButtonReleased = true;
 
-		if (editor.ui.leftLoopPinMoving)
+		if (ui.leftLoopPinMoving)
 		{
 			setLeftLoopPinState(false);
-			editor.ui.leftLoopPinMoving = false;
+			ui.leftLoopPinMoving = false;
 		}
 
-		if (editor.ui.rightLoopPinMoving)
+		if (ui.rightLoopPinMoving)
 		{
 			setRightLoopPinState(false);
-			editor.ui.rightLoopPinMoving = false;
+			ui.rightLoopPinMoving = false;
 		}
 	}
 	else if (mouseButton == SDL_BUTTON_RIGHT)
@@ -576,7 +569,6 @@ void mouseButtonUpHandler(uint8_t mouseButton)
 		mouse.rightButtonPressed = false;
 		mouse.rightButtonReleased = true;
 
-		
 		if (editor.editSampleFlag)
 		{
 			// right mouse button released after hand-editing sample data
@@ -585,13 +577,13 @@ void mouseButtonUpHandler(uint8_t mouseButton)
 
 			resumeAudio();
 
-			if (editor.ui.sampleEditorShown)
+			if (ui.sampleEditorShown)
 				writeSample(true);
 
 			setSongModifiedFlag();
+
 			editor.editSampleFlag = false;
 		}
-
 	}
 
 	mouse.firstTimePressingButton = false;
@@ -602,13 +594,13 @@ void mouseButtonUpHandler(uint8_t mouseButton)
 	if ( mouse.leftButtonPressed && !mouse.rightButtonPressed) return;
 	if (!mouse.leftButtonPressed &&  mouse.rightButtonPressed) return;
 
-	if (editor.ui.sampleEditorShown)
+	if (ui.sampleEditorShown)
 		testSmpEdMouseUp();
 
 	mouse.lastX = 0;
 	mouse.lastY = 0;
 
-	editor.ui.sampleDataOrLoopDrag = -1;
+	ui.sampleDataOrLoopDrag = -1;
 
 	// check if we released a GUI object
 	testDiskOpMouseRelease();
@@ -618,9 +610,9 @@ void mouseButtonUpHandler(uint8_t mouseButton)
 	testRadioButtonMouseRelease();
 
 	// revert "delete/rename" mouse modes (disk op.)
-	if (mouse.lastUsedObjectID != PB_DISKOP_DELETE && mouse.lastUsedObjectID != PB_DISKOP_RENAME)
+	if (mouse.mode != MOUSE_MODE_NORMAL)
 	{
-		if (mouse.mode != MOUSE_MODE_NORMAL)
+		if (mouse.lastUsedObjectID != PB_DISKOP_DELETE && mouse.lastUsedObjectID != PB_DISKOP_RENAME)
 			setMouseMode(MOUSE_MODE_NORMAL);
 	}
 
@@ -630,22 +622,17 @@ void mouseButtonUpHandler(uint8_t mouseButton)
 
 void mouseButtonDownHandler(uint8_t mouseButton)
 {
-#ifndef __APPLE__
-	if (!video.fullscreen) // trap mouse pointer while holding down left and/or right button
-		SDL_SetWindowGrab(video.window, SDL_TRUE);
-#endif
-
 	// if already holding left button and clicking right, don't do mouse down handling
 	if (mouseButton == SDL_BUTTON_RIGHT && mouse.leftButtonPressed)
 	{
-		if (editor.ui.sampleDataOrLoopDrag == -1)
+		if (ui.sampleDataOrLoopDrag == -1)
 		{
 			mouse.rightButtonPressed = true;
 			mouse.rightButtonReleased = false;
 		}
 
 		// kludge - we must do scope solo/unmute all here
-		if (!editor.ui.sysReqShown)
+		if (!ui.sysReqShown)
 			testScopesMouseDown();
 
 		return;
@@ -654,14 +641,14 @@ void mouseButtonDownHandler(uint8_t mouseButton)
 	// if already holding right button and clicking left, don't do mouse down handling
 	if (mouseButton == SDL_BUTTON_LEFT && mouse.rightButtonPressed)
 	{
-		if (editor.ui.sampleDataOrLoopDrag == -1)
+		if (ui.sampleDataOrLoopDrag == -1)
 		{
 			mouse.leftButtonPressed = true;
 			mouse.leftButtonReleased = false;
 		}
 
 		// kludge - we must do scope solo/unmute all here
-		if (!editor.ui.sysReqShown)
+		if (!ui.sysReqShown)
 			testScopesMouseDown();
 
 		return;
@@ -698,34 +685,35 @@ void mouseButtonDownHandler(uint8_t mouseButton)
 	if (mouse.lastUsedObjectType != OBJECT_PUSHBUTTON && mouse.lastUsedObjectID != OBJECT_ID_NONE)
 		return;
 
-	// kludge #3
+	// kludge #3 :(
 	if (!mouse.rightButtonPressed)
 		mouse.lastUsedObjectID = OBJECT_ID_NONE;
 
 	// check if we pressed a GUI object
 
 	/* test objects like this - clickable things *never* overlap, so no need to test all
-	** other objects if we clicked on one already */
+	** other objects if we clicked on one already
+	*/
 
 	testInstrSwitcherMouseDown(); // kludge: allow right click to both change ins. and edit text
 
-	if (testTextBoxMouseDown())     return;
-	if (testPushButtonMouseDown())  return;
-	if (testCheckBoxMouseDown())    return;
-	if (testScrollBarMouseDown())   return;
+	if (testTextBoxMouseDown()) return;
+	if (testPushButtonMouseDown()) return;
+	if (testCheckBoxMouseDown()) return;
+	if (testScrollBarMouseDown()) return;
 	if (testRadioButtonMouseDown()) return;
 
-	// from this point, we don't need to test more widgets if a system request box is shown
-	if (editor.ui.sysReqShown)
+	// at this point, we don't need to test more widgets if a system request is shown
+	if (ui.sysReqShown)
 		return;
 
 	if (testInstrVolEnvMouseDown(false)) return;
 	if (testInstrPanEnvMouseDown(false)) return;
-	if (testDiskOpMouseDown(false))      return;
-	if (testPianoKeysMouseDown(false))   return;
-	if (testSamplerDataMouseDown())      return;
-	if (testPatternDataMouseDown())      return;
-	if (testScopesMouseDown())           return;
+	if (testDiskOpMouseDown(false)) return;
+	if (testPianoKeysMouseDown(false)) return;
+	if (testSamplerDataMouseDown()) return;
+	if (testPatternDataMouseDown()) return;
+	if (testScopesMouseDown()) return;
 	if (testAudioDeviceListsMouseDown()) return;
 
 #ifdef HAS_MIDI
@@ -733,6 +721,18 @@ void mouseButtonDownHandler(uint8_t mouseButton)
 #endif
 }
 
+static void sendMouseButtonUpEvent(uint8_t button)
+{
+	SDL_Event event;
+
+	memset(&event, 0, sizeof (event));
+
+	event.type = SDL_MOUSEBUTTONUP;
+	event.button.button = button;
+
+	SDL_PushEvent(&event);
+}
+
 void handleLastGUIObjectDown(void)
 {
 	if (mouse.lastUsedObjectType == OBJECT_NONE)
@@ -744,11 +744,11 @@ void handleLastGUIObjectDown(void)
 		{
 			switch (mouse.lastUsedObjectType)
 			{
-				case OBJECT_PUSHBUTTON:  handlePushButtonsWhileMouseDown();  break;
+				case OBJECT_PUSHBUTTON: handlePushButtonsWhileMouseDown(); break;
 				case OBJECT_RADIOBUTTON: handleRadioButtonsWhileMouseDown(); break;
-				case OBJECT_CHECKBOX:    handleCheckBoxesWhileMouseDown();   break;
-				case OBJECT_SCROLLBAR:   handleScrollBarsWhileMouseDown();   break;
-				case OBJECT_TEXTBOX:     handleTextBoxWhileMouseDown();      break;
+				case OBJECT_CHECKBOX: handleCheckBoxesWhileMouseDown(); break;
+				case OBJECT_SCROLLBAR: handleScrollBarsWhileMouseDown(); break;
+				case OBJECT_TEXTBOX: handleTextBoxWhileMouseDown(); break;
 				default: break;
 			}
 		}
@@ -757,107 +757,124 @@ void handleLastGUIObjectDown(void)
 			// test non-standard GUI elements
 			switch (mouse.lastUsedObjectType)
 			{
-				case OBJECT_INSTRSWITCH: testInstrSwitcherMouseDown();       break;
-				case OBJECT_PATTERNMARK: handlePatternDataMouseDown(true);   break;
-				case OBJECT_DISKOPLIST:  testDiskOpMouseDown(true);          break;
-				case OBJECT_SMPDATA:     handleSampleDataMouseDown(true);    break;
-				case OBJECT_PIANO:       testPianoKeysMouseDown(true);       break;
-				case OBJECT_INSVOLENV:   testInstrVolEnvMouseDown(true);     break;
-				case OBJECT_INSPANENV:   testInstrPanEnvMouseDown(true);     break;
+				case OBJECT_INSTRSWITCH: testInstrSwitcherMouseDown(); break;
+				case OBJECT_PATTERNMARK: handlePatternDataMouseDown(true); break;
+				case OBJECT_DISKOPLIST: testDiskOpMouseDown(true); break;
+				case OBJECT_SMPDATA: handleSampleDataMouseDown(true); break;
+				case OBJECT_PIANO: testPianoKeysMouseDown(true); break;
+				case OBJECT_INSVOLENV: testInstrVolEnvMouseDown(true); break;
+				case OBJECT_INSPANENV: testInstrPanEnvMouseDown(true); break;
 				default: break;
 			}
 		}
+
+		/* Hack to send "mouse button up" events if we released the mouse button(s)
+		** outside of the window...
+		*/
+		if (mouse.x < 0 || mouse.x >= SCREEN_W || mouse.y < 0 || mouse.y >= SCREEN_H)
+		{
+			if (mouse.leftButtonPressed && !(mouse.buttonState & SDL_BUTTON_LMASK))
+				sendMouseButtonUpEvent(SDL_BUTTON_LEFT);
+
+			if (mouse.rightButtonPressed && !(mouse.buttonState & SDL_BUTTON_RMASK))
+				sendMouseButtonUpEvent(SDL_BUTTON_RIGHT);
+		}
 	}
 }
 
 void updateMouseScaling(void)
 {
-	if (video.renderW > 0.0) video.dMouseXMul = (double)SCREEN_W / video.renderW;
-	if (video.renderH > 0.0) video.dMouseYMul = (double)SCREEN_H / video.renderH;
+	if (video.renderW > 0.0) video.fMouseXMul = (float)SCREEN_W / video.renderW;
+	if (video.renderH > 0.0) video.fMouseYMul = (float)SCREEN_H / video.renderH;
 }
 
 void readMouseXY(void)
 {
-	int16_t x, y;
-	int32_t mx, my;
+	int32_t mx, my, windowX, windowY;
 
 	if (mouse.setPosFlag)
 	{
-		mouse.setPosFlag = false;
-
-		if (SDL_GetWindowFlags(video.window) & SDL_WINDOW_SHOWN)
+		if (!video.windowHidden)
 			SDL_WarpMouseInWindow(video.window, mouse.setPosX, mouse.setPosY);
 
+		mouse.setPosFlag = false;
 		return;
 	}
 
-	SDL_PumpEvents(); // gathers all pending input from devices into the event queue (less mouse lag)
-	SDL_GetMouseState(&mx, &my);
+	if (video.useDesktopMouseCoords)
+		mouse.buttonState = SDL_GetGlobalMouseState(&mx, &my);
+	else
+		mouse.buttonState = SDL_GetMouseState(&mx, &my);
 
-	/* in centered fullscreen mode, trap the mouse inside the framed image
-	** and subtract the coords to match the OS mouse position (fixes touch from touchscreens) */
-	if (video.fullscreen && !(config.windowFlags & FILTERING))
+	if (video.fullscreen)
 	{
-		if (mx < video.renderX)
-		{
-			mx = video.renderX;
-			SDL_WarpMouseInWindow(video.window, mx, my);
-		}
-		else if (mx >= video.renderX+video.renderW)
-		{
-			mx = (video.renderX + video.renderW) - 1;
-			SDL_WarpMouseInWindow(video.window, mx, my);
-		}
+		/* If fullscreen without filtering mode, translate coords and warp mouse
+		** inside the render space.
+		** Fullscreen + filtering mode takes up 100% of the screen area used, so no
+		** need to translate coords in that mode.
+		*/
 
-		if (my < video.renderY)
+		if (!(config.windowFlags & FILTERING))
 		{
-			my = video.renderY;
-			SDL_WarpMouseInWindow(video.window, mx, my);
-		}
-		else if (my >= video.renderY+video.renderH)
-		{
-			my = (video.renderY + video.renderH) - 1;
-			SDL_WarpMouseInWindow(video.window, mx, my);
-		}
+			if (!(config.specialFlags2 & HARDWARE_MOUSE))
+			{
+				bool warpMouse = false;
 
-		mx -= video.renderX;
-		my -= video.renderY;
+				if (mx < video.renderX)
+				{
+					mx = video.renderX;
+					warpMouse = true;
+				}
+				else if (mx >= video.renderX+video.renderW)
+				{
+					mx = (video.renderX + video.renderW) - 1;
+					warpMouse = true;
+				}
+
+				if (my < video.renderY)
+				{
+					my = video.renderY;
+					warpMouse = true;
+				}
+				else if (my >= video.renderY+video.renderH)
+				{
+					my = (video.renderY + video.renderH) - 1;
+					warpMouse = true;
+				}
+
+				if (warpMouse)
+					SDL_WarpMouseInWindow(video.window, mx, my);
+			}
+
+			// convert fullscreen coords to window (centered image) coords
+			mx -= video.renderX;
+			my -= video.renderY;
+		}
+	}
+	else if (video.useDesktopMouseCoords)
+	{
+		// convert desktop coords to window coords
+
+		// (a call to this function is really fast in windowed mode)
+		SDL_GetWindowPosition(video.window, &windowX, &windowY);
+
+		mx -= windowX;
+		my -= windowY;
 	}
 
-	if (mx < 0) mx = 0;
-	if (my < 0) mx = 0;
-
 	// multiply coords by video upscaling factors (don't round)
-	mx = (int32_t)(mx * video.dMouseXMul);
-	my = (int32_t)(my * video.dMouseYMul);
-
-	if (mx >= SCREEN_W) mx = SCREEN_W - 1;
-	if (my >= SCREEN_H) my = SCREEN_H - 1;
+	mouse.x = (int32_t)(mx * video.fMouseXMul);
+	mouse.y = (int32_t)(my * video.fMouseYMul);
 
 	if (config.specialFlags2 & HARDWARE_MOUSE)
 	{
-		// hardware mouse (OS)
-		mouse.x = (int16_t)mx;
-		mouse.y = (int16_t)my;
+		// hardware mouse mode (OS)
 		hideSprite(SPRITE_MOUSE_POINTER);
 	}
 	else
 	{
-		// software mouse (FT2 mouse)
-		x = (int16_t)mx;
-		y = (int16_t)my;
-
-		mouse.x = x;
-		mouse.y = y;
-
-		// for text editing cursor (do this after clamp)
-		x += mouse.xBias;
-		y += mouse.yBias;
-
-		if (x < 0) x = 0;
-		if (y < 0) y = 0;
-
-		setSpritePos(SPRITE_MOUSE_POINTER, x, y);
+		// software mouse mode (FT2 mouse)
+		setSpritePos(SPRITE_MOUSE_POINTER, mouse.x + mouse.xBias, mouse.y + mouse.yBias);
 	}
 
 	changeCursorIfOverTextBoxes();
