@@ -8,7 +8,9 @@
 #include "ft2_video.h"
 #include "ft2_gui.h"
 #include "ft2_pattern_ed.h"
-#include "ft2_gfxdata.h"
+#include "ft2_bmp.h"
+#include "ft2_tables.h"
+#include "ft2_structs.h"
 
 #define NI_MAXLEVEL 30
 
@@ -23,8 +25,8 @@ static const char *NI_HelpText[] =
 	"The \"Wrap\" option controls whether it's possible to walk through",
 	"the screen edges or not. Turn it on and use your brain to get",
 	"the maximum out of this feature.",
-	"The \"Surround\" option turns nibbles into a completely different",
-	"game. Don't change this option during play! (You'll see why)",
+	"The \"Surround\" option turns Nibbles into a completely different",
+	"game. Don't change this option during play! (you'll see why)",
 	"We wish you many hours of fun playing this game."
 };
 #define NIBBLES_HELP_LINES (sizeof (NI_HelpText) / sizeof (char *))
@@ -43,7 +45,7 @@ typedef struct
 static const char nibblesCheatCode1[] = "skip", nibblesCheatCode2[] = "triton";
 static char nibblesCheatBuffer[16];
 
-const char convHexTable2[10] = { 7, 8, 9, 10, 11, 12, 13, 16, 17, 18 };
+static const char convHexTable2[10] = { 7, 8, 9, 10, 11, 12, 13, 16, 17, 18 };
 static const uint8_t NI_Speeds[4] = { 12, 8, 6, 4 };
 static bool NI_EternalLives;
 static uint8_t NI_CheatIndex, NI_CurSpeed, NI_CurTick60Hz, NI_CurSpeed60Hz, NI_Screen[51][23], NI_Level;
@@ -53,6 +55,53 @@ static int32_t NI_P1Score, NI_P2Score;
 static nibbleCrd NI_P1[256], NI_P2[256];
 static nibbleBufferTyp nibblesBuffer[2];
 
+/* Non-FT2 feature: Check if either the Desktop or Buttons palette color
+** is so close to black that the player would have troubles seeing the walls
+** when playing Nibbles.
+*/
+// ------------------------------------------------------------------------
+// ------------------------------------------------------------------------
+static uint8_t rgb24ToLuminosity(uint32_t rgb24)
+{
+	const uint8_t R = RGB32_R(rgb24);
+	const uint8_t G = RGB32_G(rgb24);
+	const uint8_t B = RGB32_B(rgb24);
+
+	// get highest channel value
+	uint8_t hi = 0;
+	if (hi < R) hi = R;
+	if (hi < G) hi = G;
+	if (hi < B) hi = B;
+
+	// get lowest channel value
+	uint8_t lo = 255;
+	if (lo > R) lo = R;
+	if (lo > G) lo = G;
+	if (lo > B) lo = B;
+
+	return (hi + lo) >> 1; // 0..255
+}
+
+static bool wallColorsAreCloseToBlack(void)
+{
+#define LUMINOSITY_THRESHOLD 4
+
+	uint8_t wallColor1L = rgb24ToLuminosity(video.palette[PAL_DESKTOP]);
+	uint8_t wallColor2L = rgb24ToLuminosity(video.palette[PAL_BUTTONS]);
+
+	/* Since the rest of the wall colors are based on lower and higher
+	** contrast values from these two primary colors, we don't really
+	** need to check them all.
+	*/
+
+	if (wallColor1L <= LUMINOSITY_THRESHOLD || wallColor2L <= LUMINOSITY_THRESHOLD)
+		return true;
+
+	return false;
+}
+// ------------------------------------------------------------------------
+// ------------------------------------------------------------------------
+
 static void redrawNibblesScreen(void)
 {
 	uint8_t x, y, c;
@@ -147,7 +196,7 @@ static void nibblesGetLevel(int16_t nr)
 	for (x = 0; x < 51; x++)
 	{
 		for (y = 0; y < 23; y++)
-			NI_Screen[x][y] = nibblesStages[((readY + y) * 530) + (readX + x)];
+			NI_Screen[x][y] = bmp.nibblesStages[((readY + y) * 530) + (readX + x)];
 	}
 }
 
@@ -192,8 +241,8 @@ static void nibblesCreateLevel(int16_t nr)
 	x = (51 + 2) * (nr % 10);
 	y = (23 + 2) * (nr / 10);
 
-	NI_P1Dir = nibblesStages[(y * 530) + (x + 1)];
-	NI_P2Dir = nibblesStages[(y * 530) + (x + 0)];
+	NI_P1Dir = bmp.nibblesStages[(y * 530) + (x + 1)];
+	NI_P2Dir = bmp.nibblesStages[(y * 530) + (x + 0)];
 
 	NI_P1Len = 5;
 	NI_P2Len = 5;
@@ -221,7 +270,7 @@ static void nibbleWriteLevelSprite(int16_t xOut, int16_t yOut, int16_t nr)
 	readX = (51 + 2) * (nr % 10);
 	readY = (23 + 2) * (nr / 10);
 
-	src = (uint8_t *)&nibblesStages[(readY * 530) + readX];
+	src = (uint8_t *)&bmp.nibblesStages[(readY * 530) + readX];
 	dst = &video.frameBuffer[(yOut * SCREEN_W) + xOut];
 
 	for (uint16_t y = 0; y < 23+2; y++)
@@ -265,7 +314,7 @@ void nibblesHighScore(void)
 {
 	if (editor.NI_Play)
 	{
-		okBox(0, "System message", "No highscoretable is available during play.");
+		okBox(0, "Nibbles message", "The highscore table is not available during play.");
 		return;
 	}
 
@@ -366,21 +415,15 @@ static bool nibblesInvalid(int16_t x, int16_t y, int16_t d)
 
 static void drawScoresLives(void)
 {
-	char str[8];
-
 	// player 1
-	hexOutBg(89, 27, PAL_FORGRND, PAL_DESKTOP, NI_P1Score, 8);
-
 	assert(NI_P1Lives < 100);
-	sprintf(str, "%02d", NI_P1Lives);
-	textOutFixed(131, 39, PAL_FORGRND, PAL_DESKTOP, str);
+	hexOutBg(89, 27, PAL_FORGRND, PAL_DESKTOP, NI_P1Score, 8);
+	textOutFixed(131, 39, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[NI_P1Lives]);
 
 	// player 2
-	hexOutBg(89, 75, PAL_FORGRND, PAL_DESKTOP, NI_P2Score, 8);
-
 	assert(NI_P2Lives < 100);
-	sprintf(str, "%02d", NI_P2Lives);
-	textOutFixed(131, 87, PAL_FORGRND, PAL_DESKTOP, str);
+	hexOutBg(89, 75, PAL_FORGRND, PAL_DESKTOP, NI_P2Score, 8);
+	textOutFixed(131, 87, PAL_FORGRND, PAL_DESKTOP, dec2StrTab[NI_P2Lives]);
 }
 
 static void nibblesDecLives(int16_t l1, int16_t l2)
@@ -414,6 +457,10 @@ static void nibblesDecLives(int16_t l1, int16_t l2)
 		editor.NI_Play = false;
 		okBox(0, "Nibbles message", "GAME OVER");
 
+		// prevent highscore table from showing overflowing level graphics
+		if (NI_Level >= NI_MAXLEVEL)
+			NI_Level = NI_MAXLEVEL - 1;
+
 		if (NI_P1Score > config.NI_HighScore[9].score)
 		{
 			strcpy(name, "Unknown");
@@ -515,7 +562,7 @@ void moveNibblePlayers(void)
 {
 	int16_t i, j;
 
-	if (editor.ui.sysReqShown || --NI_CurTick60Hz != 0)
+	if (ui.sysReqShown || --NI_CurTick60Hz != 0)
 		return;
 
 	if (nibblesBufferFull(0))
@@ -685,11 +732,11 @@ NoMove:
 
 void showNibblesScreen(void)
 {
-	if (editor.ui.extended)
+	if (ui.extended)
 		exitPatternEditorExtended();
 
 	hideTopScreen();
-	editor.ui.nibblesShown = true;
+	ui.nibblesShown = true;
 
 	drawFramework(0,     0, 632,   3, FRAMEWORK_TYPE1);
 	drawFramework(0,     3, 148,  49, FRAMEWORK_TYPE1);
@@ -719,7 +766,7 @@ void showNibblesScreen(void)
 
 	drawScoresLives();
 
-	blitFast(569, 7, nibblesLogo, 59, 91);
+	blitFast(569, 7, bmp.nibblesLogo, 59, 91);
 
 	showPushButton(PB_NIBBLES_PLAY);
 	showPushButton(PB_NIBBLES_HELP);
@@ -766,7 +813,7 @@ void hideNibblesScreen(void)
 	hideCheckBox(CB_NIBBLES_GRID);
 	hideCheckBox(CB_NIBBLES_WRAP);
 
-	editor.ui.nibblesShown = false;
+	ui.nibblesShown = false;
 }
 
 void exitNibblesScreen(void)
@@ -781,16 +828,19 @@ void nibblesPlay(void)
 {
 	if (editor.NI_Play)
 	{
-		if (okBox(2, "Nibbles request", "Restart the current game of nibbles?") != 1)
+		if (okBox(2, "Nibbles request", "Restart current game of Nibbles?") != 1)
 			return;
 	}
 
 	if (config.NI_Surround && config.NI_AntPlayers == 0)
 	{
-		okBox(0, "Nibbles message", "\"Surround\" is not appropriate in one-player mode.");
+		okBox(0, "Nibbles message", "Surround mode is not appropriate in one-player mode.");
 		return;
 	}
 
+	if (wallColorsAreCloseToBlack())
+		okBox(0, "Nibbles warning", "The Desktop/Button colors are set to values that make the walls hard to see!");
+
 	assert(config.NI_Speed < 4);
 	NI_CurSpeed = NI_Speeds[config.NI_Speed];
 
@@ -812,7 +862,7 @@ void nibblesHelp(void)
 {
 	if (editor.NI_Play)
 	{
-		okBox(0, "System message", "No help available during play.");
+		okBox(0, "System message", "Help is not available during play.");
 		return;
 	}
 
@@ -827,7 +877,7 @@ void nibblesExit(void)
 {
 	if (editor.NI_Play)
 	{
-		if (okBox(2, "System request", "Quit current game of nibbles?") == 1)
+		if (okBox(2, "System request", "Quit current game of Nibbles?") == 1)
 		{
 			editor.NI_Play = false;
 			exitNibblesScreen();
@@ -910,7 +960,7 @@ void nibblesKeyAdministrator(SDL_Scancode scancode)
 {
 	if (scancode == SDL_SCANCODE_ESCAPE)
 	{
-		if (okBox(2, "System request", "Quit current game of nibbles?") == 1)
+		if (okBox(2, "System request", "Quit current game of Nibbles?") == 1)
 		{
 			editor.NI_Play = false;
 			exitNibblesScreen();
