@@ -4,6 +4,8 @@
 #include "ft2_gui.h"
 #include "ft2_config.h"
 #include "ft2_video.h"
+#include "ft2_palette.h"
+#include "ft2_tables.h"
 
 uint8_t cfg_ColorNr = 0; // globalized
 
@@ -15,7 +17,7 @@ static const uint8_t scaleOrder[3] = { 8, 4, 9 };
 static uint8_t palContrast[12][2] = // palette desktop/button contrasts
 {
 	{59, 55}, {59, 53}, {56, 59}, {68, 55}, {57, 59}, {48, 55},
-	{66, 62}, {68, 57}, {46, 57}, {57, 55}, {62, 57}, {52, 57}
+	{66, 62}, {68, 57}, {58, 42}, {57, 55}, {62, 57}, {52, 57}
 };
 
 void setCustomPalColor(uint32_t color)
@@ -25,7 +27,7 @@ void setCustomPalColor(uint32_t color)
 
 void setPal16(pal16 *p, bool redrawScreen)
 {
-#define LOOP_PIN_COL_SUB 118
+#define LOOP_PIN_COL_SUB 75
 #define TEXT_MARK_COLOR 0x0078D7
 #define BOX_SELECT_COLOR 0x7F7F7F
 
@@ -69,6 +71,11 @@ static void showColorErrorMsg(void)
 	okBox(0, "System message", "Default colors cannot be modified.");
 }
 
+static void showMouseColorErrorMsg(void)
+{
+	okBox(0, "System message", "Mouse color can only be changed when \"Software mouse\" is enabled.");
+}
+
 static double palPow(double dX, double dY)
 {
 	if (dY == 1.0)
@@ -124,7 +131,7 @@ static void updatePaletteEditor(void)
 
 static void paletteDragMoved(void)
 {
-	uint8_t nr, p, contrast;
+	uint8_t nr, p;
 	int16_t i, k;
 
 	if (config.cfg_StdPalNr != PAL_USER_DEFINED)
@@ -137,7 +144,7 @@ static void paletteDragMoved(void)
 	if ((config.specialFlags2 & HARDWARE_MOUSE) && cfg_ColorNr == 3)
 	{
 		updatePaletteEditor(); // resets colors/contrast vars
-		okBox(0, "System message", "Mouse color can only be changed when \"Software mouse\" is enabled.");
+		showMouseColorErrorMsg();
 		return;
 	}
 
@@ -150,17 +157,25 @@ static void paletteDragMoved(void)
 
 	if (cfg_ColorNr == 4 || cfg_ColorNr == 5)
 	{
+		double dRed = cfg_Red;
+		double dGreen = cfg_Green;
+		double dBlue = cfg_Blue;
+
+		int32_t contrast = cfg_Contrast;
+		if (contrast < 1)
+			contrast = 1;
+
+		double dContrast = contrast * (1.0 / 40.0);
+
 		for (i = 0; i < 3; i++)
 		{
 			k = scaleOrder[i] + (cfg_ColorNr - 4) * 2;
 
-			contrast = cfg_Contrast;
-			if (contrast < 1) // no idea why FT2 limits contrast to 1
-				contrast = 1;
+			double dMul = palPow((i + 1) * (1.0 / 2.0), dContrast);
 
-			palTable[p][k].r = palMax((int32_t)round(cfg_Red * palPow((i + 1) / 2.0, contrast / 40.0)));
-			palTable[p][k].g = palMax((int32_t)round(cfg_Green * palPow((i + 1) / 2.0, contrast / 40.0)));
-			palTable[p][k].b = palMax((int32_t)round(cfg_Blue * palPow((i + 1) / 2.0, contrast / 40.0)));
+			palTable[p][k].r = palMax((int32_t)((dRed * dMul) + 0.5));
+			palTable[p][k].g = palMax((int32_t)((dGreen * dMul) + 0.5));
+			palTable[p][k].b = palMax((int32_t)((dBlue * dMul) + 0.5));
 		}
 
 		palContrast[p][cfg_ColorNr-4] = cfg_Contrast;
@@ -220,6 +235,8 @@ void configPalRDown(void)
 {
 	if (config.cfg_StdPalNr != PAL_USER_DEFINED)
 		showColorErrorMsg();
+	else if ((config.specialFlags2 & HARDWARE_MOUSE) && cfg_ColorNr == 3)
+		showMouseColorErrorMsg();
 	else
 		scrollBarScrollLeft(SB_PAL_R, 1);
 }
@@ -228,6 +245,8 @@ void configPalRUp(void)
 {
 	if (config.cfg_StdPalNr != PAL_USER_DEFINED)
 		showColorErrorMsg();
+	else if ((config.specialFlags2 & HARDWARE_MOUSE) && cfg_ColorNr == 3)
+		showMouseColorErrorMsg();
 	else
 		scrollBarScrollRight(SB_PAL_R, 1);
 }
@@ -236,6 +255,8 @@ void configPalGDown(void)
 {
 	if (config.cfg_StdPalNr != PAL_USER_DEFINED)
 		showColorErrorMsg();
+	else if ((config.specialFlags2 & HARDWARE_MOUSE) && cfg_ColorNr == 3)
+		showMouseColorErrorMsg();
 	else
 		scrollBarScrollLeft(SB_PAL_G, 1);
 }
@@ -244,6 +265,8 @@ void configPalGUp(void)
 {
 	if (config.cfg_StdPalNr != PAL_USER_DEFINED)
 		showColorErrorMsg();
+	else if ((config.specialFlags2 & HARDWARE_MOUSE) && cfg_ColorNr == 3)
+		showMouseColorErrorMsg();
 	else
 		scrollBarScrollRight(SB_PAL_G, 1);
 }
@@ -252,6 +275,8 @@ void configPalBDown(void)
 {
 	if (config.cfg_StdPalNr != PAL_USER_DEFINED)
 		showColorErrorMsg();
+	else if ((config.specialFlags2 & HARDWARE_MOUSE) && cfg_ColorNr == 3)
+		showMouseColorErrorMsg();
 	else
 		scrollBarScrollLeft(SB_PAL_B, 1);
 }
@@ -260,6 +285,8 @@ void configPalBUp(void)
 {
 	if (config.cfg_StdPalNr != PAL_USER_DEFINED)
 		showColorErrorMsg();
+	else if ((config.specialFlags2 & HARDWARE_MOUSE) && cfg_ColorNr == 3)
+		showMouseColorErrorMsg();
 	else
 		scrollBarScrollRight(SB_PAL_B, 1);
 }
@@ -268,6 +295,8 @@ void configPalContDown(void)
 {
 	if (config.cfg_StdPalNr != PAL_USER_DEFINED)
 		showColorErrorMsg();
+	else if ((config.specialFlags2 & HARDWARE_MOUSE) && cfg_ColorNr == 3)
+		showMouseColorErrorMsg();
 	else
 		scrollBarScrollLeft(SB_PAL_CONTRAST, 1);
 }
@@ -276,6 +305,8 @@ void configPalContUp(void)
 {
 	if (config.cfg_StdPalNr != PAL_USER_DEFINED)
 		showColorErrorMsg();
+	else if ((config.specialFlags2 & HARDWARE_MOUSE) && cfg_ColorNr == 3)
+		showMouseColorErrorMsg();
 	else
 		scrollBarScrollRight(SB_PAL_CONTRAST, 1);
 }
@@ -388,12 +419,12 @@ void rbConfigPalBlues(void)
 	checkRadioButton(RB_CONFIG_PAL_BLUES);
 }
 
-void rbConfigPalSpacePigs(void)
+void rbConfigPalDarkMode(void)
 {
-	config.cfg_StdPalNr = PAL_SPACE_PIGS;
+	config.cfg_StdPalNr = PAL_DARK_MODE;
 	updatePaletteEditor();
 	setPal16(palTable[config.cfg_StdPalNr], true);
-	checkRadioButton(RB_CONFIG_PAL_SPACE_PIGS);
+	checkRadioButton(RB_CONFIG_PAL_DARK_MODE);
 }
 
 void rbConfigPalGold(void)
@@ -443,79 +474,3 @@ void rbConfigPalUserDefined(void)
 	setPal16(palTable[config.cfg_StdPalNr], true);
 	checkRadioButton(RB_CONFIG_PAL_USER_DEFINED);
 }
-
-pal16 palTable[12][16] = // FT2 palettes (18-bit VGA RGB, 16 color palette)
-{
-	{
-		{0, 0, 0},{30, 38, 63},{0, 0, 17},{63, 63, 63},
-		{27, 36, 40},{63, 63, 63},{40, 40, 40},{0, 0, 0},
-		{10, 13, 14},{49, 63, 63},{15, 15, 15},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{21, 40, 63},{0, 0, 17},{63, 63, 63},
-		{6, 39, 35},{63, 63, 63},{40, 40, 40},{0, 0, 0},
-		{2, 14, 13},{11, 63, 63},{16, 16, 16},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{39, 52, 63},{8, 8, 13},{57, 57, 63},
-		{10, 21, 33},{63, 63, 63},{37, 37, 45},{0, 0, 0},
-		{4, 8, 13},{18, 37, 58},{13, 13, 16},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{47, 47, 47},{9, 9, 9},{63, 63, 63},
-		{37, 29, 7},{63, 63, 63},{40, 40, 40},{0, 0, 0},
-		{11, 9, 2},{63, 58, 14},{15, 15, 15},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{46, 45, 46},{13, 9, 9},{63, 63, 63},
-		{22, 19, 22},{63, 63, 63},{36, 32, 34},{0, 0, 0},
-		{8, 7, 8},{39, 34, 39},{13, 12, 12},{63, 58, 62},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{19, 49, 54},{0, 11, 7},{52, 63, 61},
-		{9, 31, 21},{63, 63, 63},{40, 40, 40},{0, 0, 0},
-		{4, 13, 9},{15, 50, 34},{15, 15, 15},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{27, 37, 53},{0, 0, 20},{63, 63, 63},
-		{7, 12, 21},{63, 63, 63},{38, 39, 39},{0, 0, 0},
-		{2, 4, 7},{14, 23, 41},{13, 13, 13},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{63, 54, 62},{18, 3, 3},{63, 63, 63},
-		{36, 19, 25},{63, 63, 63},{40, 40, 40},{0, 0, 0},
-		{11, 6, 8},{63, 38, 50},{15, 15, 15},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{63, 0, 63},{0, 21, 0},{63, 44, 0},
-		{0, 63, 0},{63, 63, 63},{63, 0, 0},{0, 0, 0},
-		{0, 28, 0},{0, 63, 0},{23, 0, 0},{63, 0, 0},
-		{0, 63, 63},{0, 63, 63},{0, 63, 63},{0, 63, 63}
-	},
-	{
-		{0, 0, 0},{50, 46, 63},{15, 0, 16},{59, 58, 63},
-		{34, 21, 41},{63, 63, 63},{40, 40, 40},{0, 0, 0},
-		{13, 8, 15},{61, 37, 63},{15, 15, 15},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{63, 63, 32},{10, 10, 10},{63, 63, 63},
-		{18, 29, 32},{63, 63, 63},{39, 39, 39},{0, 0, 0},
-		{6, 10, 11},{34, 54, 60},{15, 15, 15},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	},
-	{
-		{0, 0, 0},{36, 47, 63},{9, 9, 16},{63, 63, 63},
-		{19, 24, 38},{63, 63, 63},{39, 39, 39},{0, 0, 0},
-		{8, 10, 15},{32, 41, 63},{15, 15, 15},{63, 63, 63},
-		{63, 63, 63},{63, 63, 63},{63, 63, 63},{63, 63, 63}
-	}
-};
