@@ -7,10 +7,12 @@
 #include <stdint.h>
 #include "ft2_header.h"
 #include "ft2_pattern_ed.h"
-#include "ft2_gfxdata.h"
 #include "ft2_config.h"
 #include "ft2_gui.h"
 #include "ft2_video.h"
+#include "ft2_tables.h"
+#include "ft2_bmp.h"
+#include "ft2_structs.h"
 
 static tonTyp emptyPattern[MAX_VOICES * MAX_PATT_LEN];
 
@@ -31,212 +33,6 @@ static const uint16_t sharpNote2Char_big[12] = { 36*16, 37*16, 36*16, 37*16, 36*
 static const uint16_t flatNote1Char_big[12] = { 12*16, 13*16, 13*16, 14*16, 14*16, 15*16, 16*16, 16*16, 10*16, 10*16, 11*16, 11*16 };
 static const uint16_t flatNote2Char_big[12] = { 36*16, 38*16, 36*16, 38*16, 36*16, 36*16, 38*16, 36*16, 38*16, 36*16, 38*16, 36*16 };
 
-static const uint8_t noteTab1[96] =
-{
-	0,1,2,3,4,5,6,7,8,9,10,11,
-	0,1,2,3,4,5,6,7,8,9,10,11,
-	0,1,2,3,4,5,6,7,8,9,10,11,
-	0,1,2,3,4,5,6,7,8,9,10,11,
-	0,1,2,3,4,5,6,7,8,9,10,11,
-	0,1,2,3,4,5,6,7,8,9,10,11,
-	0,1,2,3,4,5,6,7,8,9,10,11,
-	0,1,2,3,4,5,6,7,8,9,10,11
-};
-
-static const uint8_t noteTab2[96] =
-{
-	0,0,0,0,0,0,0,0,0,0,0,0,
-	1,1,1,1,1,1,1,1,1,1,1,1,
-	2,2,2,2,2,2,2,2,2,2,2,2,
-	3,3,3,3,3,3,3,3,3,3,3,3,
-	4,4,4,4,4,4,4,4,4,4,4,4,
-	5,5,5,5,5,5,5,5,5,5,5,5,
-	6,6,6,6,6,6,6,6,6,6,6,6,
-	7,7,7,7,7,7,7,7,7,7,7,7
-};
-
-static const uint8_t hex2Dec[256] =
-{
-	0,1,2,3,4,5,6,7,8,9,
-	16,17,18,19,20,21,22,23,24,25,
-	32,33,34,35,36,37,38,39,40,41,
-	48,49,50,51,52,53,54,55,56,57,
-	64,65,66,67,68,69,70,71,72,73,
-	80,81,82,83,84,85,86,87,88,89,
-	96,97,98,99,100,101,102,103,104,105,
-	112,113,114,115,116,117,118,119,120,121,
-	128,129,130,131,132,133,134,135,136,137,
-	144,145,146,147,148,149,150,151,152,153,
-
-	0,1,2,3,4,5,6,7,8,9,
-	16,17,18,19,20,21,22,23,24,25,
-	32,33,34,35,36,37,38,39,40,41,
-	48,49,50,51,52,53,54,55,56,57,
-	64,65,66,67,68,69,70,71,72,73,
-	80,81,82,83,84,85,86,87,88,89,
-	96,97,98,99,100,101,102,103,104,105,
-	112,113,114,115,116,117,118,119,120,121,
-	128,129,130,131,132,133,134,135,136,137,
-	144,145,146,147,148,149,150,151,152,153,
-
-	0,1,2,3,4,5,6,7,8,9,
-	16,17,18,19,20,21,22,23,24,25,
-	32,33,34,35,36,37,38,39,40,41,
-	48,49,50,51,52,53,54,55,56,57,
-	64,65,66,67,68,69,70,71,72,73,
-	80,81,82,83,84,85
-};
-
-static const pattCoord_t pattCoordTable[2][2][2] =
-{
-	// no pattern stretch
-	{
-		// no pattern channel scroll
-		{
-			{ 176, 292, 177, 283, 293, 13, 13 }, // normal pattern editor
-			{  56, 228,  57, 219, 229, 20, 21 }, // extended pattern editor
-		},
-
-		// pattern channel scroll
-		{
-			{ 176, 285, 177, 276, 286, 12, 12 }, // normal pattern editor
-			{  56, 221,  57, 212, 222, 19, 20 }, // extended pattern editor
-		}
-	},
-
-	// pattern stretch
-	{
-		// no pattern channel scroll
-		{
-			{ 177, 286, 178, 277, 288,  9, 10 }, // normal pattern editor
-			{  56, 232,  58, 223, 234, 15, 15 }, // extended pattern editor
-		},
-
-		// pattern channel scroll
-		{
-			{  176, 285, 177, 276, 286,  9,  9 }, // normal pattern editor
-			{   56, 220,  57, 211, 221, 14, 15 }, // extended pattern editor
-		},
-	}
-};
-
-static const pattCoord2_t pattCoord2Table[2][2][2] =
-{
-	// no pattern stretch
-	{
-		// no pattern channel scroll
-		{
-			{ 175, 291, 107, 107 }, //   normal pattern editor
-			{  55, 227, 163, 171 }, // extended pattern editor
-		},
-
-		// pattern channel scroll
-		{
-			{ 175, 284, 100, 100 }, //   normal pattern editor
-			{  55, 220, 156, 164 }, // extended pattern editor
-		}
-	},
-
-	// pattern stretch
-	{
-		// no pattern channel scroll
-		{
-			{ 175, 285, 101, 113 }, //   normal pattern editor
-			{  55, 231, 167, 167 }, // extended pattern editor
-		},
-
-		// pattern channel scroll
-		{
-			{ 175, 284, 100, 100 }, //   normal pattern editor
-			{  55, 219, 155, 165 }, // extended pattern editor
-		},
-	}
-};
-
-static const markCoord_t markCoordTable[2][2][2] =
-{
-	// no pattern stretch
-	{
-		// no pattern channel scroll
-		{
-			{ 177, 281, 293 }, //   normal pattern editor
-			{  57, 217, 229 }, // extended pattern editor
-		},
-
-		// pattern channel scroll
-		{
-			{ 177, 274, 286 }, //   normal pattern editor
-			{  57, 210, 222 }, // extended pattern editor
-		}
-	},
-
-	// pattern stretch
-	{
-		// no pattern channel scroll
-		{
-			{ 176, 275, 286 }, //   normal pattern editor
-			{  56, 221, 232 }, // extended pattern editor
-		},
-
-		// pattern channel scroll
-		{
-			{ 175, 274, 284 }, //   normal pattern editor
-			{  55, 209, 219 }, // extended pattern editor
-		},
-	}
-};
-
-static const uint8_t pattCursorXTab[2 * 4 * 8] =
-{
-	// no volume column shown
-	32, 88, 104, 0, 0, 120, 136, 152, //  4 columns visible
-	32, 80,  88, 0, 0,  96, 104, 112, //  6 columns visible
-	32, 56,  64, 0, 0,  72,  80,  88, //  8 columns visible
-	32, 52,  56, 0, 0,  60,  64,  68, // 12 columns visible
-
-	// volume column shown
-	32, 96, 104, 120, 128, 144, 152, 160, //  4 columns visible
-	32, 56,  64,  80,  88,  96, 104, 112, //  6 columns visible
-	32, 60,  64,  72,  76,  84,  88,  92, //  8 columns visible
-	32, 60,  64,  72,  76,  84,  88,  92, // 12 columns visible
-};
-
-static const uint8_t pattCursorWTab[2 * 4 * 8] =
-{
-	// no volume column shown
-	48, 16, 16, 0, 0, 16, 16, 16, //  4 columns visible
-	48,  8,  8, 0, 0,  8,  8,  8, //  6 columns visible
-	24,  8,  8, 0, 0,  8,  8,  8, //  8 columns visible
-	16,  4,  4, 0, 0,  4,  4,  4, // 12 columns visible
-
-	// volume column shown
-	48,  8,  8,  8,  8,  8,  8,  8, //  4 columns visible
-	24,  8,  8,  8,  8,  8,  8,  8, //  6 columns visible
-	24,  4,  4,  4,  4,  4,  4,  4, //  8 columns visible
-	24,  4,  4,  4,  4,  4,  4,  4  // 12 columns visible
-};
-
-// global tables
-
-const char chDecTab1[MAX_VOICES+1] = 
-{
-	'0', '0', '0', '0', '0', '0', '0', '0', '0', '0',
-	'1', '1', '1', '1', '1', '1', '1', '1', '1', '1',
-	'2', '2', '2', '2', '2', '2', '2', '2', '2', '2',
-	'3', '3', '3'
-};
-
-const char chDecTab2[MAX_VOICES+1] = 
-{
-	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
-	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
-	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
-	'0', '1', '2'
-};
-
-// ft2_pattern_ed.c
-extern const uint16_t chanWidths[6];
-
 static void pattCharOut(uint32_t xPos, uint32_t yPos, uint8_t chr, uint8_t fontType, uint32_t color);
 static void drawEmptyNoteSmall(uint32_t xPos, uint32_t yPos, uint32_t color);
 static void drawKeyOffSmall(uint32_t xPos, uint32_t yPos, uint32_t color);
@@ -251,8 +47,8 @@ static void drawNoteBig(uint32_t xPos, uint32_t yPos, int32_t ton, uint32_t colo
 void updatePattFontPtrs(void)
 {
 	//config.ptnFont is pre-clamped and safe
-	font4Ptr = &font4Data[config.ptnFont * (FONT4_WIDTH * FONT4_CHAR_H)];
-	font5Ptr = &font5Data[config.ptnFont * (FONT5_WIDTH * FONT5_CHAR_H)];
+	font4Ptr = &bmp.font4[config.ptnFont * (FONT4_WIDTH * FONT4_CHAR_H)];
+	font5Ptr = &bmp.font4[(4 + config.ptnFont) * (FONT4_WIDTH * FONT4_CHAR_H)];
 }
 
 void drawPatternBorders(void)
@@ -263,14 +59,14 @@ void drawPatternBorders(void)
 	const pattCoord2_t *pattCoord;
 
 	// get heights/pos/rows depending on configuration
-	pattCoord = &pattCoord2Table[config.ptnUnpressed][editor.ui.pattChanScrollShown][editor.ui.extended];
+	pattCoord = &pattCoord2Table[config.ptnUnpressed][ui.pattChanScrollShown][ui.extended];
 
 	// set pattern cursor Y position
 	editor.ptnCursorY = pattCoord->lowerRowsY - 9;
 
-	chans = editor.ui.numChannelsShown;
-	if (chans > editor.ui.maxVisibleChannels)
-		chans = editor.ui.maxVisibleChannels;
+	chans = ui.numChannelsShown;
+	if (chans > ui.maxVisibleChannels)
+		chans = ui.maxVisibleChannels;
 
 	// in some configurations, there will be two empty channels to the right, fix that
 	if (chans == 2)
@@ -283,14 +79,14 @@ void drawPatternBorders(void)
 	chanWidth = chanWidths[(chans / 2) - 1] + 2;
 
 	// fill scrollbar framework (if needed)
-	if (editor.ui.pattChanScrollShown)
+	if (ui.pattChanScrollShown)
 		drawFramework(0, 383, 632, 17, FRAMEWORK_TYPE1);
 
 	if (config.ptnFrmWrk)
 	{
 		// pattern editor w/ framework
 
-		if (editor.ui.extended)
+		if (ui.extended)
 		{
 			vLine(0,   54, 345, PAL_DSKTOP1);
 			vLine(631, 53, 346, PAL_DSKTOP2);
@@ -301,7 +97,7 @@ void drawPatternBorders(void)
 			hLine(0, 53, 631, PAL_DSKTOP1);
 			hLine(1, 54, 630, PAL_DESKTOP);
 
-			if (!editor.ui.pattChanScrollShown)
+			if (!ui.pattChanScrollShown)
 			{
 				hLine(1, 398, 630, PAL_DESKTOP);
 				hLine(0, 399, 632, PAL_DSKTOP2);
@@ -318,7 +114,7 @@ void drawPatternBorders(void)
 			hLine(0, 173, 631, PAL_DSKTOP1);
 			hLine(1, 174, 630, PAL_DESKTOP);
 
-			if (!editor.ui.pattChanScrollShown)
+			if (!ui.pattChanScrollShown)
 			{
 				hLine(1, 398, 630, PAL_DESKTOP);
 				hLine(0, 399, 632, PAL_DSKTOP2);
@@ -354,21 +150,21 @@ void drawPatternBorders(void)
 	{
 		// pattern editor without framework
 
-		if (editor.ui.extended)
+		if (ui.extended)
 		{
-			clearSize = editor.ui.pattChanScrollShown ? (SCREEN_W * sizeof (int32_t) * 330) : (SCREEN_W * sizeof (int32_t) * 347);
+			clearSize = ui.pattChanScrollShown ? (SCREEN_W * sizeof (int32_t) * 330) : (SCREEN_W * sizeof (int32_t) * 347);
 			memset(&video.frameBuffer[53 * SCREEN_W], 0, clearSize);
 		}
 		else
 		{
-			clearSize = editor.ui.pattChanScrollShown ? (SCREEN_W * sizeof(int32_t) * 210) : (SCREEN_W * sizeof(int32_t) * 227);
+			clearSize = ui.pattChanScrollShown ? (SCREEN_W * sizeof(int32_t) * 210) : (SCREEN_W * sizeof(int32_t) * 227);
 			memset(&video.frameBuffer[173 * SCREEN_W], 0, clearSize);
 		}
 
 		drawFramework(0, pattCoord->lowerRowsY - 10, SCREEN_W, 11, FRAMEWORK_TYPE1);
 	}
 
-	if (editor.ui.pattChanScrollShown)
+	if (ui.pattChanScrollShown)
 	{
 		showScrollBar(SB_CHAN_SCROLL);
 		showPushButton(PB_CHAN_SCROLL_LEFT);
@@ -388,13 +184,13 @@ static void writeCursor(void)
 {
 	uint32_t *dstPtr, xPos, width, tabOffset;
 
-	tabOffset = (config.ptnS3M * 32) + (columnModeTab[editor.ui.numChannelsShown - 1] * 8) + editor.cursor.object;
+	tabOffset = (config.ptnS3M * 32) + (columnModeTab[ui.numChannelsShown - 1] * 8) + cursor.object;
 
 	xPos = pattCursorXTab[tabOffset];
 	width = pattCursorWTab[tabOffset];
 
 	assert(editor.ptnCursorY > 0 && xPos > 0 && width > 0);
-	xPos += ((editor.cursor.ch - editor.ui.channelOffset) * editor.ui.patternChannelWidth);
+	xPos += ((cursor.ch - ui.channelOffset) * ui.patternChannelWidth);
 
 	dstPtr = &video.frameBuffer[(editor.ptnCursorY * SCREEN_W) + xPos];
 	for (uint32_t y = 0; y < 9; y++)
@@ -416,8 +212,8 @@ static void writePatternBlockMark(int32_t currRow, uint32_t rowHeight, const pat
 	if (pattMark.markY1 > pattMark.markY2)
 		return;
 
-	startCh = editor.ui.channelOffset;
-	endCh = editor.ui.channelOffset + (editor.ui.numChannelsShown - 1);
+	startCh = ui.channelOffset;
+	endCh = ui.channelOffset + (ui.numChannelsShown - 1);
 	startRow = currRow - pattCoord->numUpperRows;
 	endRow = currRow + pattCoord->numLowerRows;
 
@@ -425,18 +221,18 @@ static void writePatternBlockMark(int32_t currRow, uint32_t rowHeight, const pat
 	if (pattMark.markX1 > endCh || pattMark.markX2 < startCh || pattMark.markY1 > endRow || pattMark.markY2 < startRow)
 		return;
 
-	markCoord = &markCoordTable[config.ptnUnpressed][editor.ui.pattChanScrollShown][editor.ui.extended];
+	markCoord = &markCoordTable[config.ptnUnpressed][ui.pattChanScrollShown][ui.extended];
 	pattYStart = markCoord->upperRowsY;
 
 	// X1
 
-	x1 = 32 + ((pattMark.markX1 - editor.ui.channelOffset) * editor.ui.patternChannelWidth);
+	x1 = 32 + ((pattMark.markX1 - ui.channelOffset) * ui.patternChannelWidth);
 	if (x1 < 32)
 		x1 = 32;
 
 	// X2
 
-	x2 = (32 - 8) + (((pattMark.markX2 + 1) - editor.ui.channelOffset) * editor.ui.patternChannelWidth);
+	x2 = (32 - 8) + (((pattMark.markX2 + 1) - ui.channelOffset) * ui.patternChannelWidth);
 	if (x2 > 608)
 		x2 = 608;
 
@@ -477,7 +273,7 @@ static void writePatternBlockMark(int32_t currRow, uint32_t rowHeight, const pat
 	}
 
 	// kludge! (some mark situations could overwrite illegal areas)
-	if (config.ptnUnpressed && editor.ui.pattChanScrollShown)
+	if (config.ptnUnpressed && ui.pattChanScrollShown)
 	{
 		if (y1 == pattCoord->upperRowsY-1 || y1 == pattCoord->lowerRowsY-1)
 			y1++;
@@ -512,12 +308,12 @@ static void writePatternBlockMark(int32_t currRow, uint32_t rowHeight, const pat
 
 static void drawChannelNumbering(uint16_t yPos)
 {
-#define CH_NUM_XPOS 29
+#define CH_NUM_XPOS 30
 
 	uint16_t xPos = CH_NUM_XPOS;
-	int32_t ch = editor.ui.channelOffset + 1;
+	int32_t ch = ui.channelOffset + 1;
 
-	for (uint8_t i = 0; i < editor.ui.numChannelsShown; i++)
+	for (uint8_t i = 0; i < ui.numChannelsShown; i++)
 	{
 		if (ch < 10)
 		{
@@ -530,7 +326,7 @@ static void drawChannelNumbering(uint16_t yPos)
 		}
 
 		ch++;
-		xPos += editor.ui.patternChannelWidth;
+		xPos += ui.patternChannelWidth;
 	}
 }
 
@@ -590,7 +386,7 @@ static void showNoteNum(uint32_t xPos, uint32_t yPos, int16_t ton, uint32_t colo
 
 	assert(ton >= 0 && ton <= 97);
 
-	if (editor.ui.numChannelsShown <= 4)
+	if (ui.numChannelsShown <= 4)
 	{
 		if (ton <= 0 || ton > 97)
 			drawEmptyNoteBig(xPos, yPos, color);
@@ -614,13 +410,13 @@ static void showInstrNum(uint32_t xPos, uint32_t yPos, uint8_t ins, uint32_t col
 {
 	uint8_t chr1, chr2, charW, fontType;
 
-	if (editor.ui.numChannelsShown <= 4)
+	if (ui.numChannelsShown <= 4)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
 		xPos += 67;
 	}
-	else if (editor.ui.numChannelsShown <= 6)
+	else if (ui.numChannelsShown <= 6)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
@@ -655,7 +451,7 @@ static void showVolEfx(uint32_t xPos, uint32_t yPos, uint8_t vol, uint32_t color
 {
 	uint8_t char1, char2, fontType, charW;
 
-	if (editor.ui.numChannelsShown <= 4)
+	if (ui.numChannelsShown <= 4)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
@@ -667,7 +463,7 @@ static void showVolEfx(uint32_t xPos, uint32_t yPos, uint8_t vol, uint32_t color
 		else
 			char2 = vol & 0x0F;
 	}
-	else if (editor.ui.numChannelsShown <= 6)
+	else if (ui.numChannelsShown <= 6)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
@@ -700,13 +496,13 @@ static void showEfx(uint32_t xPos, uint32_t yPos, uint8_t effTyp, uint8_t eff, u
 {
 	uint8_t fontType, charW;
 
-	if (editor.ui.numChannelsShown <= 4)
+	if (ui.numChannelsShown <= 4)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
 		xPos += 115;
 	}
-	else if (editor.ui.numChannelsShown <= 6)
+	else if (ui.numChannelsShown <= 6)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
@@ -732,7 +528,7 @@ static void showNoteNumNoVolColumn(uint32_t xPos, uint32_t yPos, int16_t ton, ui
 
 	assert(ton >= 0 && ton <= 97);
 
-	if (editor.ui.numChannelsShown <= 6)
+	if (ui.numChannelsShown <= 6)
 	{
 		if (ton <= 0 || ton > 97)
 			drawEmptyNoteBig(xPos, yPos, color);
@@ -741,7 +537,7 @@ static void showNoteNumNoVolColumn(uint32_t xPos, uint32_t yPos, int16_t ton, ui
 		else
 			drawNoteBig(xPos, yPos, ton, color);
 	}
-	else if (editor.ui.numChannelsShown <= 8)
+	else if (ui.numChannelsShown <= 8)
 	{
 		if (ton <= 0 || ton > 97)
 			drawEmptyNoteMedium(xPos, yPos, color);
@@ -765,19 +561,19 @@ static void showInstrNumNoVolColumn(uint32_t xPos, uint32_t yPos, uint8_t ins, u
 {
 	uint8_t chr1, chr2, charW, fontType;
 
-	if (editor.ui.numChannelsShown <= 4)
+	if (ui.numChannelsShown <= 4)
 	{
 		fontType = FONT_TYPE5;
 		charW = FONT5_CHAR_W;
 		xPos += 59;
 	}
-	else if (editor.ui.numChannelsShown <= 6)
+	else if (ui.numChannelsShown <= 6)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
 		xPos += 51;
 	}
-	else if (editor.ui.numChannelsShown <= 8)
+	else if (ui.numChannelsShown <= 8)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
@@ -821,19 +617,19 @@ static void showEfxNoVolColumn(uint32_t xPos, uint32_t yPos, uint8_t effTyp, uin
 {
 	uint8_t charW, fontType;
 
-	if (editor.ui.numChannelsShown <= 4)
+	if (ui.numChannelsShown <= 4)
 	{
 		fontType = FONT_TYPE5;
 		charW = FONT5_CHAR_W;
 		xPos += 91;
 	}
-	else if (editor.ui.numChannelsShown <= 6)
+	else if (ui.numChannelsShown <= 6)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
 		xPos += 67;
 	}
-	else if (editor.ui.numChannelsShown <= 8)
+	else if (ui.numChannelsShown <= 8)
 	{
 		fontType = FONT_TYPE4;
 		charW = FONT4_CHAR_W;
@@ -872,19 +668,19 @@ void writePattern(int32_t currRow, int32_t pattern)
 
 	// setup variables
 
-	chans = editor.ui.numChannelsShown;
-	if (chans > editor.ui.maxVisibleChannels)
-		chans = editor.ui.maxVisibleChannels;
+	chans = ui.numChannelsShown;
+	if (chans > ui.maxVisibleChannels)
+		chans = ui.maxVisibleChannels;
 
 	assert(chans >= 2 && chans <= 12);
 
 	// get channel width
 	chanWidth = chanWidths[(chans / 2) - 1];
-	editor.ui.patternChannelWidth = (uint16_t)(chanWidth + 3);
+	ui.patternChannelWidth = (uint16_t)(chanWidth + 3);
 
 	// get heights/pos/rows depending on configuration
 	rowHeight = config.ptnUnpressed ? 11 : 8;
-	pattCoord = &pattCoordTable[config.ptnUnpressed][editor.ui.pattChanScrollShown][editor.ui.extended];
+	pattCoord = &pattCoordTable[config.ptnUnpressed][ui.pattChanScrollShown][ui.extended];
 	midRowTextY = pattCoord->midRowTextY;
 	lowerRowsTextY = pattCoord->lowerRowsTextY;
 	row = currRow - pattCoord->numUpperRows;
@@ -892,7 +688,7 @@ void writePattern(int32_t currRow, int32_t pattern)
 	textY = pattCoord->upperRowsTextY;
 
 	afterCurrRow = currRow + 1;
-	numChannels = editor.ui.numChannelsShown;
+	numChannels = ui.numChannelsShown;
 	pattPtr = patt[pattern];
 	numRows = pattLens[pattern];
 	noteTextColors[0] = video.palette[PAL_PATTEXT]; // not selected
@@ -900,7 +696,7 @@ void writePattern(int32_t currRow, int32_t pattern)
 
 	// increment pattern data pointer by horizontal scrollbar offset/channel
 	if (pattPtr != NULL)
-		pattPtr += editor.ui.channelOffset;
+		pattPtr += ui.channelOffset;
 
 	// set up function pointers for drawing
 	if (config.ptnS3M)
@@ -933,7 +729,7 @@ void writePattern(int32_t currRow, int32_t pattern)
 				note = &pattPtr[(uint32_t)row * MAX_VOICES];
 
 			xPos = 29;
-			xWidth = editor.ui.patternChannelWidth;
+			xWidth = ui.patternChannelWidth;
 
 			color = noteTextColors[selectedRowFlag];
 			for (int32_t j = 0; j < numChannels; j++)
@@ -1023,7 +819,7 @@ static void pattCharOut(uint32_t xPos, uint32_t yPos, uint8_t chr, uint8_t fontT
 
 	if (fontType == FONT_TYPE3)
 	{
-		srcPtr = &font3Data[chr * FONT3_CHAR_W];
+		srcPtr = &bmp.font3[chr * FONT3_CHAR_W];
 		for (y = 0; y < FONT3_CHAR_H; y++)
 		{
 			for (x = 0; x < FONT3_CHAR_W; x++)
@@ -1089,7 +885,7 @@ static void pattCharOut(uint32_t xPos, uint32_t yPos, uint8_t chr, uint8_t fontT
 	}
 	else
 	{
-		srcPtr = &font7Data[chr * FONT7_CHAR_W];
+		srcPtr = &bmp.font7[chr * FONT7_CHAR_W];
 		for (y = 0; y < FONT7_CHAR_H; y++)
 		{
 			for (x = 0; x < FONT7_CHAR_W; x++)
@@ -1119,7 +915,7 @@ static void drawEmptyNoteSmall(uint32_t xPos, uint32_t yPos, uint32_t color)
 	uint32_t tmp;
 #endif
 
-	srcPtr = &font7Data[18 * FONT7_CHAR_W];
+	srcPtr = &bmp.font7[18 * FONT7_CHAR_W];
 	dstPtr = &video.frameBuffer[(yPos * SCREEN_W) + xPos];
 
 	for (uint32_t y = 0; y < FONT7_CHAR_H; y++)
@@ -1150,7 +946,7 @@ static void drawKeyOffSmall(uint32_t xPos, uint32_t yPos, uint32_t color)
 	uint32_t tmp;
 #endif
 
-	srcPtr = &font7Data[21 * FONT7_CHAR_W];
+	srcPtr = &bmp.font7[21 * FONT7_CHAR_W];
 	dstPtr = &video.frameBuffer[(yPos * SCREEN_W) + (xPos + 2)];
 
 	for (uint32_t y = 0; y < FONT7_CHAR_H; y++)
@@ -1200,9 +996,9 @@ static void drawNoteSmall(uint32_t xPos, uint32_t yPos, int32_t ton, uint32_t co
 		char2 = flatNote2Char_small[note];
 	}
 
-	ch1Ptr = &font7Data[char1];
-	ch2Ptr = &font7Data[char2];
-	ch3Ptr = &font7Data[char3];
+	ch1Ptr = &bmp.font7[char1];
+	ch2Ptr = &bmp.font7[char2];
+	ch3Ptr = &bmp.font7[char3];
 	dstPtr = &video.frameBuffer[(yPos * SCREEN_W) + xPos];
 
 	for (uint32_t y = 0; y < FONT7_CHAR_H; y++)
@@ -1397,7 +1193,7 @@ static void drawKeyOffBig(uint32_t xPos, uint32_t yPos, uint32_t color)
 	uint32_t tmp;
 #endif
 
-	srcPtr = &font4Data[61 * FONT4_CHAR_W];
+	srcPtr = &bmp.font4[61 * FONT4_CHAR_W];
 	dstPtr = &video.frameBuffer[(yPos * SCREEN_W) + xPos];
 
 	for (uint32_t y = 0; y < FONT4_CHAR_H; y++)
