@@ -341,12 +341,8 @@ enum
 	PUSHBUTTON_PRESSED   = 1
 };
 
-#define RADIOBUTTON_W 11
-#define RADIOBUTTON_H 11
-#define RADIOBUTTON_STATES 3
-
 // amount of frames to wait
-#define BUTTON_DOWN_DELAY 8
+#define BUTTON_DOWN_DELAY 25
 
 // font #1/#2 special characters (used for buttons)
 #define ARROW_UP_STRING "\x05"
