@@ -15,6 +15,8 @@
 #include "ft2_diskop.h"
 #include "ft2_mouse.h"
 #include "ft2_wav_renderer.h"
+#include "ft2_bmp.h"
+#include "ft2_structs.h"
 
 radioButton_t radioButtons[NUM_RADIOBUTTONS] =
 {
@@ -29,12 +31,12 @@ radioButton_t radioButtons[NUM_RADIOBUTTONS] =
 
 	// ------ HELP SCREEN RADIOBUTTONS ------
 	//x, y,   w,  group,         funcOnUp
-	{ 5, 16,  68, RB_GROUP_HELP, rbHelpFeatures },
-	{ 5, 31,  59, RB_GROUP_HELP, rbHelpEffects },
-	{ 5, 46,  70, RB_GROUP_HELP, rbHelpKeyboard },
-	{ 5, 61, 108, RB_GROUP_HELP, rbHelpHowToUseFT2 },
-	{ 5, 76, 100, RB_GROUP_HELP, rbHelpFAQ },
-	{ 5, 91,  85, RB_GROUP_HELP, rbHelpKnownBugs },
+	{ 5, 18,  69, RB_GROUP_HELP, rbHelpFeatures },
+	{ 5, 34,  60, RB_GROUP_HELP, rbHelpEffects },
+	{ 5, 50,  71, RB_GROUP_HELP, rbHelpKeyboard },
+	{ 5, 66, 109, RB_GROUP_HELP, rbHelpHowToUseFT2 },
+	{ 5, 82, 101, RB_GROUP_HELP, rbHelpFAQ },
+	{ 5, 98,  86, RB_GROUP_HELP, rbHelpKnownBugs },
 
 	// ------ NIBBLES SCREEN RADIOBUTTONS ------
 	//x, y,   w,   group,                       funcOnUp
@@ -62,12 +64,11 @@ radioButton_t radioButtons[NUM_RADIOBUTTONS] =
 
 	// ------ CONFIG SCREEN LEFT RADIOBUTTONS ------
 	//x, y,  w,  group,                  funcOnUp
-	{ 4, 19, 87, RB_GROUP_CONFIG_SELECT, rbConfigIODevices },
-	{ 4, 35, 59, RB_GROUP_CONFIG_SELECT, rbConfigLayout },
-	{ 4, 51, 99, RB_GROUP_CONFIG_SELECT, rbConfigMiscellaneous },
-
+	{ 5, 18, 85, RB_GROUP_CONFIG_SELECT, rbConfigIODevices },
+	{ 5, 34, 57, RB_GROUP_CONFIG_SELECT, rbConfigLayout },
+	{ 5, 50, 97, RB_GROUP_CONFIG_SELECT, rbConfigMiscellaneous },
 #ifdef HAS_MIDI
-	{ 4, 67, 74, RB_GROUP_CONFIG_SELECT, rbConfigMidiInput },
+	{ 5, 66, 72, RB_GROUP_CONFIG_SELECT, rbConfigMidiInput },
 #endif
 
 	// ------ CONFIG AUDIO ------
@@ -85,15 +86,12 @@ radioButton_t radioButtons[NUM_RADIOBUTTONS] =
 
 	// audio output frequency
 	//x,   y,   w,  group,                      funcOnUp
-#ifdef __APPLE__
-	{ 509, 16, 121, RB_GROUP_CONFIG_AUDIO_FREQ, rbConfigAudio44kHz },
-	{ 509, 30,  66, RB_GROUP_CONFIG_AUDIO_FREQ, rbConfigAudio48kHz },
-#else
 	{ 509, 16,  66, RB_GROUP_CONFIG_AUDIO_FREQ, rbConfigAudio44kHz },
 	{ 509, 30, 121, RB_GROUP_CONFIG_AUDIO_FREQ, rbConfigAudio48kHz },
-#endif
+#if defined __amd64__ || defined _WIN64
 	{ 509, 44,  66, RB_GROUP_CONFIG_AUDIO_FREQ, rbConfigAudio96kHz },
-
+	{ 509, 58,  73, RB_GROUP_CONFIG_AUDIO_FREQ, rbConfigAudio192kHz },
+#endif
 	// audio input frequency
 	//x,   y,  w,   group,                            funcOnUp
 	{ 180, 156, 60, RB_GROUP_CONFIG_AUDIO_INPUT_FREQ, rbConfigAudioInput44kHz },
@@ -154,7 +152,7 @@ radioButton_t radioButtons[NUM_RADIOBUTTONS] =
 	{ 399, 103, 105, RB_GROUP_CONFIG_PAL_PRESET, rbConfigPalAuroraBorealis },
 	{ 512, 103,  45, RB_GROUP_CONFIG_PAL_PRESET, rbConfigPalRose },
 	{ 399, 117,  47, RB_GROUP_CONFIG_PAL_PRESET, rbConfigPalBlues },
-	{ 512, 117,  81, RB_GROUP_CONFIG_PAL_PRESET, rbConfigPalSpacePigs },
+	{ 512, 117,  77, RB_GROUP_CONFIG_PAL_PRESET, rbConfigPalDarkMode },
 	{ 399, 131,  40, RB_GROUP_CONFIG_PAL_PRESET, rbConfigPalGold },
 	{ 512, 131,  56, RB_GROUP_CONFIG_PAL_PRESET, rbConfigPalViolent },
 	{ 399, 145,  87, RB_GROUP_CONFIG_PAL_PRESET, rbConfigPalHeavyMetal },
@@ -217,12 +215,9 @@ radioButton_t radioButtons[NUM_RADIOBUTTONS] =
 	{ 205, 95, 83, RB_GROUP_WAV_RENDER_BITDEPTH, rbWavRenderBitDepth32 }
 };
 
-// defined at the bottom of this file
-extern const uint8_t radioButtonGraphics[3][RADIOBUTTON_W * RADIOBUTTON_H];
-
 void drawRadioButton(uint16_t radioButtonID)
 {
-	uint8_t state;
+	uint8_t *gfxPtr;
 	radioButton_t *radioButton;
 
 	assert(radioButtonID < NUM_RADIOBUTTONS);
@@ -232,9 +227,8 @@ void drawRadioButton(uint16_t radioButtonID)
 
 	assert(radioButton->x < SCREEN_W && radioButton->y < SCREEN_H);
 
-	state = radioButton->state;
-	if (state <= RADIOBUTTON_STATES-1)
-		blitFast(radioButton->x, radioButton->y, radioButtonGraphics[state], RADIOBUTTON_W, RADIOBUTTON_H);
+	gfxPtr = &bmp.radiobuttonGfx[radioButton->state*(RADIOBUTTON_W*RADIOBUTTON_H)];
+	blitFast(radioButton->x, radioButton->y, gfxPtr, RADIOBUTTON_W, RADIOBUTTON_H);
 }
 
 void showRadioButton(uint16_t radioButtonID)
@@ -328,7 +322,7 @@ bool testRadioButtonMouseDown(void)
 {
 	radioButton_t *radioButton;
 
-	if (editor.ui.sysReqShown)
+	if (ui.sysReqShown)
 		return false;
 
 	for (uint16_t i = 0; i < NUM_RADIOBUTTONS; i++)
@@ -371,93 +365,3 @@ void testRadioButtonMouseRelease(void)
 			radioButton->callbackFunc();
 	}
 }
-
-const uint8_t radioButtonGraphics[3][RADIOBUTTON_W * RADIOBUTTON_H] =
-{
-	// unticked
-	{
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP1,
-		PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP1,
-		PAL_DSKTOP1, PAL_DSKTOP2, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP1,
-		PAL_DSKTOP1, PAL_DSKTOP2, PAL_DESKTOP, PAL_DSKTOP2, PAL_DSKTOP1,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP1,
-		PAL_DSKTOP1, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DSKTOP2, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP1,
-		PAL_DSKTOP1, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2, PAL_DSKTOP1, PAL_DESKTOP,
-		PAL_DSKTOP2, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_DESKTOP, PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_BCKGRND,
-		PAL_DSKTOP2, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_BCKGRND, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_BCKGRND,
-		PAL_DSKTOP2, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP, PAL_BCKGRND,
-		PAL_DSKTOP2, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2, PAL_BCKGRND,
-		PAL_DSKTOP2, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DSKTOP2, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP
-	},
-
-	// ticked
-	{
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP, PAL_BCKGRND, PAL_DSKTOP2,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP, PAL_FORGRND, PAL_DESKTOP,
-		PAL_BCKGRND, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP, PAL_FORGRND, PAL_FORGRND,
-		PAL_FORGRND, PAL_DESKTOP, PAL_BCKGRND, PAL_DSKTOP2, PAL_DESKTOP,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP, PAL_FORGRND, PAL_FORGRND,
-		PAL_FORGRND, PAL_FORGRND, PAL_FORGRND, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_DESKTOP, PAL_DSKTOP1, PAL_DSKTOP2, PAL_DESKTOP,
-		PAL_FORGRND, PAL_FORGRND, PAL_FORGRND, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP1,
-		PAL_DSKTOP2, PAL_DESKTOP, PAL_FORGRND, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DSKTOP1, PAL_DSKTOP2, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP1, PAL_DSKTOP2,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP
-	},
-
-	// pressed
-	{
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP, PAL_BCKGRND, PAL_DSKTOP2,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_BCKGRND, PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_BCKGRND, PAL_DSKTOP2, PAL_DESKTOP,
-		PAL_DSKTOP2, PAL_BCKGRND, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP2, PAL_DESKTOP, PAL_DSKTOP1, PAL_DSKTOP2, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP1,
-		PAL_DSKTOP2, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DSKTOP1, PAL_DSKTOP2, PAL_DESKTOP, PAL_DSKTOP2,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DSKTOP1, PAL_DSKTOP2,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DSKTOP1, PAL_DSKTOP1, PAL_DESKTOP, PAL_DESKTOP, PAL_DESKTOP,
-		PAL_DESKTOP
-	}
-};
