@@ -57,7 +57,10 @@ enum // RADIOBUTTONS
 	// AUDIO FREQUENCY
 	RB_CONFIG_AUDIO_44KHZ,
 	RB_CONFIG_AUDIO_48KHZ,
+#if defined __amd64__ || defined _WIN64
 	RB_CONFIG_AUDIO_96KHZ,
+	RB_CONFIG_AUDIO_192KHZ,
+#endif
 
 	// AUDIO INPUT FREQUENCY
 	RB_CONFIG_AUDIO_INPUT_44KHZ,
@@ -110,7 +113,7 @@ enum // RADIOBUTTONS
 	RB_CONFIG_PAL_AURORA_BOREALIS,
 	RB_CONFIG_PAL_ROSE,
 	RB_CONFIG_PAL_BLUES,
-	RB_CONFIG_PAL_SPACE_PIGS,
+	RB_CONFIG_PAL_DARK_MODE,
 	RB_CONFIG_PAL_GOLD,
 	RB_CONFIG_PAL_VIOLENT,
 	RB_CONFIG_PAL_HEAVY_METAL,
