@@ -1,9 +1,9 @@
 /* This file contains the routines for the following sample editor functions:
- * - Resampler
- * - Echo
- * - Mix
- * - Volume
- */
+** - Resampler
+** - Echo
+** - Mix
+** - Volume
+**/
 
 // for finding memory leaks in debug mode with Visual Studio
 #if defined _DEBUG && defined _MSC_VER
@@ -23,28 +23,27 @@
 #include "ft2_inst_ed.h"
 #include "ft2_sample_ed.h"
 #include "ft2_keyboard.h"
+#include "ft2_tables.h"
+#include "ft2_structs.h"
+
+static volatile bool stopThread;
 
 static int8_t smpEd_RelReSmp, mix_Balance = 50;
-static bool stopThread, echo_AddMemory, exitFlag, outOfMemory;
+static bool echo_AddMemory, exitFlag, outOfMemory;
 static int16_t vol_StartVol = 100, vol_EndVol = 100, echo_nEcho = 1, echo_VolChange = 30;
 static int32_t echo_Distance = 0x100;
 static SDL_Thread *thread;
 
 static void pbExit(void)
 {
-	editor.ui.sysReqShown = false;
+	ui.sysReqShown = false;
 	exitFlag = true;
 }
 
 static void windowOpen(void)
 {
-	editor.ui.sysReqShown = true;
-	editor.ui.sysReqEnterPressed = false;
-
-#ifndef __APPLE__
-	if (!video.fullscreen) // release mouse button trap
-		SDL_SetWindowGrab(video.window, SDL_FALSE);
-#endif
+	ui.sysReqShown = true;
+	ui.sysReqEnterPressed = false;
 
 	unstuckLastUsedGUIElement();
 	SDL_EventState(SDL_DROPFILE, SDL_DISABLE);
@@ -97,7 +96,7 @@ static int32_t SDLCALL resampleThread(void *ptr)
 	s = &instr[editor.curInstr]->samp[editor.curSmp];
 
 	mask = (s->typ & 16) ? 0xFFFFFFFE : 0xFFFFFFFF;
-	dLenMul = exp2(smpEd_RelReSmp / 12.0);
+	dLenMul = exp2(smpEd_RelReSmp * (1.0 / 12.0));
 
 	dNewLen = s->len * dLenMul;
 	if (dNewLen > (double)MAX_SAMPLE_LEN)
@@ -110,14 +109,16 @@ static int32_t SDLCALL resampleThread(void *ptr)
 	{
 		outOfMemory = true;
 		setMouseBusy(false);
-		editor.ui.sysReqShown = false;
+		ui.sysReqShown = false;
 		return true;
 	}
 
+	int8_t *newPtr = p2 + SMP_DAT_OFFSET;
+
 	p1 = s->pek;
 
 	// don't use the potentially clamped newLen value here
-	delta64 = ((uint64_t)s->len << 32) / (uint64_t)(s->len * dLenMul); // 32.32 fixed point delta
+	delta64 = ((int64_t)s->len << 32) / (int64_t)(s->len * dLenMul); // 32.32 fixed point delta
 
 	posFrac64 = 0; // 32.32 fixed point position.fraction
 
@@ -129,9 +130,9 @@ static int32_t SDLCALL resampleThread(void *ptr)
 		if (s->typ & 16)
 		{
 			src16 = (int16_t *)p1;
-			dst16 = (int16_t *)p2;
+			dst16 = (int16_t *)newPtr;
 
-			resampleLen = newLen / 2;
+			resampleLen = newLen >> 1;
 			for (uint32_t i = 0; i < resampleLen; i++)
 			{
 				dst16[i] = src16[posFrac64 >> 32];
@@ -141,7 +142,7 @@ static int32_t SDLCALL resampleThread(void *ptr)
 		else
 		{
 			src8 = p1;
-			dst8 = p2;
+			dst8 = newPtr;
 
 			for (uint32_t i = 0; i < newLen; i++)
 			{
@@ -151,24 +152,30 @@ static int32_t SDLCALL resampleThread(void *ptr)
 		}
 	}
 
-	free(p1);
+	free(s->origPek);
 
 	s->relTon = CLAMP(s->relTon + smpEd_RelReSmp, -48, 71);
 
-	s->len = newLen;
-	s->pek = p2;
-	s->repS = (int32_t)(s->repS * dLenMul) & mask;
-	s->repL = (int32_t)(s->repL * dLenMul) & mask;
+	s->len = newLen & mask;
+
+	s->origPek = p2;
+	s->pek = s->origPek + SMP_DAT_OFFSET;
+
+	s->repS = (int32_t)(s->repS * dLenMul);
+	s->repL = (int32_t)(s->repL * dLenMul);
+
+	s->repS &= mask;
+	s->repL &= mask;
 
 	if (s->repS >= s->len)
-		s->repS = s->len - 1;
+		s->repS = s->len-1;
 
 	if (s->repS+s->repL > s->len)
 		s->repL = s->len - s->repS;
 
 	if (s->typ & 16)
 	{
-		s->len  &= 0xFFFFFFFE;
+		s->len &= 0xFFFFFFFE;
 		s->repS &= 0xFFFFFFFE;
 		s->repL &= 0xFFFFFFFE;
 	}
@@ -182,7 +189,7 @@ static int32_t SDLCALL resampleThread(void *ptr)
 	setSongModifiedFlag();
 	setMouseBusy(false);
 
-	editor.ui.sysReqShown = false;
+	ui.sysReqShown = false;
 	return true;
 }
 
@@ -229,7 +236,7 @@ static void drawResampleBox(void)
 	s = &instr[editor.curInstr]->samp[editor.curSmp];
 
 	mask = (s->typ & 16) ? 0xFFFFFFFE : 0xFFFFFFFF;
-	dLenMul = exp2(smpEd_RelReSmp / 12.0);
+	dLenMul = exp2(smpEd_RelReSmp * (1.0 / 12.0));
 
 	dNewLen = s->len * dLenMul;
 	if (dNewLen > (double)MAX_SAMPLE_LEN)
@@ -340,10 +347,10 @@ void pbSampleResample(void)
 	outOfMemory = false;
 
 	exitFlag = false;
-	while (editor.ui.sysReqShown)
+	while (ui.sysReqShown)
 	{
 		readInput();
-		if (editor.ui.sysReqEnterPressed)
+		if (ui.sysReqEnterPressed)
 			pbDoResampling();
 
 		setSyncedReplayerVars();
@@ -428,7 +435,7 @@ static void pbEchoFadeoutUp(void)
 
 static int32_t SDLCALL createEchoThread(void *ptr)
 {
-	int8_t *readPtr, *writePtr, *newPtr;
+	int8_t *readPtr, *writePtr, *writePtr8, *newPtr;
 	bool is16Bit;
 	int16_t *readPtr16, *writePtr16;
 	int32_t nEchoes, distance, readLen, writeLen, i, j;
@@ -445,24 +452,44 @@ static int32_t SDLCALL createEchoThread(void *ptr)
 	is16Bit = (s->typ & 16) ? true : false;
 	distance = echo_Distance * 16;
 
+	// scale value for faster math and suitable rounding for PCM waveforms (DIV -> arithmetic bitshift right)
+	volChange = (echo_VolChange * 256) / 100; // 0..100 -> 0..256
+
+	if (echo_nEcho < 1)
+	{
+		ui.sysReqShown = false;
+		return true;
+	}
+
 	// calculate real number of echoes
-	j = is16Bit ? 32768 : 128; i = 0;
+	j = 32768;
+	i = 0;
 	while (i < echo_nEcho && j > 0)
 	{
-		j = (j * echo_VolChange) / 100;
+		j = (j * volChange) >> 8;
 		i++;
 	}
 	nEchoes = i + 1;
 
+	if (nEchoes < 1)
+	{
+		ui.sysReqShown = false;
+		return true;
+	}
+
 	// set write length (either original length or full echo length)
 	writeLen = readLen;
 	if (echo_AddMemory)
 	{
-		tmp64 = writeLen + ((int64_t)distance * (nEchoes - 1));
+		tmp64 = (int64_t)distance * (nEchoes - 1);
+		if (is16Bit)
+			tmp64 <<= 1;
+
+		tmp64 += writeLen;
 		if (tmp64 > MAX_SAMPLE_LEN)
-			writeLen = MAX_SAMPLE_LEN;
-		else
-			writeLen += distance * (nEchoes - 1);
+			tmp64 = MAX_SAMPLE_LEN;
+
+		writeLen = (int32_t)tmp64;
 
 		if (is16Bit)
 			writeLen &= 0xFFFFFFFE;
@@ -473,7 +500,7 @@ static int32_t SDLCALL createEchoThread(void *ptr)
 	{
 		outOfMemory = true;
 		setMouseBusy(false);
-		editor.ui.sysReqShown = false;
+		ui.sysReqShown = false;
 		return false;
 	}
 
@@ -482,18 +509,15 @@ static int32_t SDLCALL createEchoThread(void *ptr)
 
 	writeIdx = 0;
 
-	// scale value for faster math and suitable rounding for PCM waveforms (DIV -> arithmetic bitshift right)
-	volChange = (int32_t)round((echo_VolChange * 256) / 100.0);
-	if (volChange > 256)
-		volChange = 256;
-
 	if (is16Bit)
 	{
 		readPtr16 = (int16_t *)readPtr;
-		writePtr16 = (int16_t *)writePtr;
+		writePtr16 = (int16_t *)&writePtr[SMP_DAT_OFFSET];
 
 		writeLen >>= 1;
-		while (!stopThread && writeIdx < writeLen)
+		readLen >>= 1;
+
+		while (writeIdx < writeLen)
 		{
 			smpOut = 0;
 			smpMul = 32768;
@@ -501,45 +525,57 @@ static int32_t SDLCALL createEchoThread(void *ptr)
 			echoRead = writeIdx;
 			echoCycle = nEchoes;
 
-			while (!stopThread && echoRead > 0 && echoCycle-- > 0)
+			while (!stopThread)
 			{
 				if (echoRead < readLen)
-					smpOut += (readPtr16[echoRead] * smpMul) >> (16-1);
+					smpOut += (readPtr16[echoRead] * smpMul) >> 15;
 
 				smpMul = (smpMul * volChange) >> 8;
+
 				echoRead -= distance;
+				echoCycle--;
+
+				if (echoRead <= 0 || echoCycle <= 0)
+					break;
 			}
 			CLAMP16(smpOut);
 
 			writePtr16[writeIdx++] = (int16_t)smpOut;
 		}
+
 		writeLen <<= 1;
 	}
 	else
 	{
-		while (!stopThread && writeIdx < writeLen)
+		writePtr8 = writePtr + SMP_DAT_OFFSET;
+		while (writeIdx < writeLen)
 		{
 			smpOut = 0;
-			smpMul = 65536;
+			smpMul = 32768;
 
 			echoRead = writeIdx;
 			echoCycle = nEchoes;
 
-			while (!stopThread && echoRead > 0 && echoCycle-- > 0)
+			while (!stopThread)
 			{
 				if (echoRead < readLen)
-					smpOut += (readPtr[echoRead] * smpMul) >> (16-8);
+					smpOut += (readPtr[echoRead] * smpMul) >> (15-8);
 
 				smpMul = (smpMul * volChange) >> 8;
+
 				echoRead -= distance;
+				echoCycle--;
+
+				if (echoRead <= 0 || echoCycle <= 0)
+					break;
 			}
 			CLAMP16(smpOut);
 
-			writePtr[writeIdx++] = (int8_t)(smpOut >> 8);
+			writePtr8[writeIdx++] = (int8_t)(smpOut >> 8);
 		}
 	}
 
-	free(readPtr);
+	free(s->origPek);
 
 	if (stopThread)
 	{
@@ -547,13 +583,25 @@ static int32_t SDLCALL createEchoThread(void *ptr)
 
 		newPtr = (int8_t *)realloc(writePtr, writeIdx + LOOP_FIX_LEN);
 		if (newPtr != NULL)
-			s->pek = newPtr;
+		{
+			s->origPek = newPtr;
+			s->pek = s->origPek + SMP_DAT_OFFSET;
+		}
+		else
+		{
+			if (writePtr != NULL)
+				free(writePtr);
+
+			s->origPek = s->pek = NULL;
+			writeLen = 0;
+		}
 
 		editor.updateCurSmp = true;
 	}
 	else
 	{
-		s->pek = writePtr;
+		s->origPek = writePtr;
+		s->pek = s->origPek + SMP_DAT_OFFSET;
 	}
 
 	if (is16Bit)
@@ -567,7 +615,7 @@ static int32_t SDLCALL createEchoThread(void *ptr)
 	setSongModifiedFlag();
 	setMouseBusy(false);
 
-	editor.ui.sysReqShown = false;
+	ui.sysReqShown = false;
 	return true;
 }
 
@@ -609,26 +657,23 @@ static void drawEchoBox(void)
 	hLine(x + 2,     y + h - 3, w - 4, PAL_BUTTON1);
 
 	textOutShadow(177, 226, PAL_FORGRND, PAL_BUTTON2, "Number of echoes");
-	textOutShadow(177, 239, PAL_FORGRND, PAL_BUTTON2, "Echo distance");
-	textOutShadow(177, 253, PAL_FORGRND, PAL_BUTTON2, "Fade out");
+	textOutShadow(177, 240, PAL_FORGRND, PAL_BUTTON2, "Echo distance");
+	textOutShadow(177, 254, PAL_FORGRND, PAL_BUTTON2, "Fade out");
 	textOutShadow(192, 270, PAL_FORGRND, PAL_BUTTON2, "Add memory to sample");
 
 	assert(echo_nEcho <= 1024);
 
-	charOut(315 + (0 * 7), 226, PAL_FORGRND, '0' + (echo_nEcho / 1000) % 10);
+	charOut(315 + (0 * 7), 226, PAL_FORGRND, '0' + (char)(echo_nEcho / 1000));
 	charOut(315 + (1 * 7), 226, PAL_FORGRND, '0' + (echo_nEcho / 100) % 10);
 	charOut(315 + (2 * 7), 226, PAL_FORGRND, '0' + (echo_nEcho / 10) % 10);
 	charOut(315 + (3 * 7), 226, PAL_FORGRND, '0' + (echo_nEcho % 10));
 
-	assert((echo_Distance * 16) <= 262144);
-
-	hexOut(308, 240, PAL_FORGRND, echo_Distance * 16, 5);
+	assert(echo_Distance <= 0x4000);
+	hexOut(308, 240, PAL_FORGRND, (uint32_t)echo_Distance << 4, 5);
 
 	assert(echo_VolChange <= 100);
+	textOutFixed(312, 254, PAL_FORGRND, PAL_BUTTONS, dec3StrTab[echo_VolChange]);
 
-	charOut(312 + (0 * 7), 254, PAL_FORGRND, '0' + (echo_VolChange / 100) % 10);
-	charOut(312 + (1 * 7), 254, PAL_FORGRND, '0' + (echo_VolChange / 10) % 10);
-	charOut(312 + (2 * 7), 254, PAL_FORGRND, '0' + (echo_VolChange % 10));
 	charOutShadow(313 + (3 * 7), 254, PAL_FORGRND, PAL_BUTTON2, '%');
 }
 
@@ -804,10 +849,10 @@ void pbSampleEcho(void)
 	outOfMemory = false;
 
 	exitFlag = false;
-	while (editor.ui.sysReqShown)
+	while (ui.sysReqShown)
 	{
 		readInput();
-		if (editor.ui.sysReqEnterPressed)
+		if (ui.sysReqEnterPressed)
 			pbCreateEcho();
 
 		setSyncedReplayerVars();
@@ -851,7 +896,7 @@ static int32_t SDLCALL mixThread(void *ptr)
 	if (destIns == mixIns && destSmp == mixSmp)
 	{
 		setMouseBusy(false);
-		editor.ui.sysReqShown = false;
+		ui.sysReqShown = false;
 		return true;
 	}
 
@@ -893,15 +938,18 @@ static int32_t SDLCALL mixThread(void *ptr)
 		}
 	}
 
-	mix8Size = (mixTyp & 16) ? (mixLen / 2) : mixLen;
-	dest8Size = (destTyp & 16) ? (destLen / 2) : destLen;
+	bool src16Bits = (mixTyp >> 4) & 1;
+	bool dst16Bits = (destTyp >> 4) & 1;
+
+	mix8Size = src16Bits ? (mixLen >> 1) : mixLen;
+	dest8Size = dst16Bits ? (destLen >> 1) : destLen;
 	max8Size = (dest8Size > mix8Size) ? dest8Size : mix8Size;
-	maxLen = (destTyp & 16) ? (max8Size * 2) : max8Size;
+	maxLen = dst16Bits ? (max8Size << 1) : max8Size;
 
 	if (maxLen <= 0)
 	{
 		setMouseBusy(false);
-		editor.ui.sysReqShown = false;
+		ui.sysReqShown = false;
 		return true;
 	}
 
@@ -910,7 +958,7 @@ static int32_t SDLCALL mixThread(void *ptr)
 	{
 		outOfMemory = true;
 		setMouseBusy(false);
-		editor.ui.sysReqShown = false;
+		ui.sysReqShown = false;
 		return true;
 	}
 
@@ -918,50 +966,56 @@ static int32_t SDLCALL mixThread(void *ptr)
 	{
 		outOfMemory = true;
 		setMouseBusy(false);
-		editor.ui.sysReqShown = false;
+		ui.sysReqShown = false;
 		return true;
 	}
 
 	pauseAudio();
 
 	restoreSample(&instr[destIns]->samp[destSmp]);
+
+	// restore source sample
 	if (instr[mixIns] != NULL)
 		restoreSample(&instr[mixIns]->samp[mixSmp]);
 
 	// scale value for faster math and suitable rounding for PCM waveforms (DIV -> arithmetic bitshift right)
-	mixMul1 = (int32_t)round((mix_Balance * 256) / 100.0);
-	if (mixMul1 > 256)
-		mixMul1 = 256;
+	mixMul1 = (mix_Balance * 256) / 100;
 	mixMul2 = 256 - mixMul1;
 
+	int8_t *destPek = p + SMP_DAT_OFFSET;
 	for (i = 0; i < max8Size; i++)
 	{
-		x1 = (i >= mix8Size) ? 0 : getSampleValueNr(mixPtr, mixTyp, (mixTyp & 16) ? (i << 1) : i);
-		x2 = (i >= dest8Size) ? 0 : getSampleValueNr(destPtr, destTyp, (destTyp & 16) ? (i << 1) : i);
+		int32_t index16 = i << 1;
 
-		if (!(mixTyp & 16)) x1 <<= 8;
-		if (!(destTyp & 16)) x2 <<= 8;
+		x1 = (i >= mix8Size) ? 0 : getSampleValueNr(mixPtr, mixTyp, src16Bits ? index16 : i);
+		x2 = (i >= dest8Size) ? 0 : getSampleValueNr(destPtr, destTyp, dst16Bits ? index16 : i);
 
-		smp32 = (int32_t)((((int64_t)x1 * mixMul1) + ((int64_t)x2 * mixMul2)) >> 8);
+		if (!src16Bits) x1 <<= 8;
+		if (!dst16Bits) x2 <<= 8;
+
+		smp32 = ((x1 * mixMul1) >> 8) + ((x2 * mixMul2) >> 8);
 		CLAMP16(smp32);
 
-		if (!(destTyp & 16))
+		if (!dst16Bits)
 			smp32 >>= 8;
 
-		putSampleValueNr(p, destTyp, (destTyp & 16) ? (i << 1) : i, (int16_t)smp32);
+		putSampleValueNr(destPek, destTyp, dst16Bits ? index16 : i, (int16_t)smp32);
 	}
 
-	if (instr[destIns]->samp[destSmp].pek != NULL)
-		free(instr[destIns]->samp[destSmp].pek);
+	if (instr[destIns]->samp[destSmp].origPek != NULL)
+		free(instr[destIns]->samp[destSmp].origPek);
 
-	instr[destIns]->samp[destSmp].pek = p;
+	instr[destIns]->samp[destSmp].origPek = p;
+	instr[destIns]->samp[destSmp].pek = instr[destIns]->samp[destSmp].origPek + SMP_DAT_OFFSET;
 	instr[destIns]->samp[destSmp].len = maxLen;
 	instr[destIns]->samp[destSmp].typ = destTyp;
 
-	if (destTyp & 16) // bugfix
+	if (dst16Bits)
 		instr[destIns]->samp[destSmp].len &= 0xFFFFFFFE;
 
 	fixSample(&instr[destIns]->samp[destSmp]);
+
+	// fix source sample
 	if (instr[mixIns] != NULL)
 		fixSample(&instr[mixIns]->samp[mixSmp]);
 
@@ -970,7 +1024,7 @@ static int32_t SDLCALL mixThread(void *ptr)
 	setSongModifiedFlag();
 	setMouseBusy(false);
 
-	editor.ui.sysReqShown = false;
+	ui.sysReqShown = false;
 	return true;
 }
 
@@ -1030,10 +1084,7 @@ static void drawMixSampleBox(void)
 	textOutShadow(198, 246, PAL_FORGRND, PAL_BUTTON2, "Mixing balance");
 
 	assert((mix_Balance >= 0) && (mix_Balance <= 100));
-
-	charOut(299 + (0 * 7), 246, PAL_FORGRND, '0' + ((mix_Balance / 100) % 10));
-	charOut(299 + (1 * 7), 246, PAL_FORGRND, '0' + ((mix_Balance / 10) % 10));
-	charOut(299 + (2 * 7), 246, PAL_FORGRND, '0' + (mix_Balance % 10));
+	textOutFixed(299, 246, PAL_FORGRND, PAL_BUTTONS, dec3StrTab[mix_Balance]);
 }
 
 static void setupMixBoxWidgets(void)
@@ -1115,10 +1166,10 @@ void pbSampleMix(void)
 	outOfMemory = false;
 
 	exitFlag = false;
-	while (editor.ui.sysReqShown)
+	while (ui.sysReqShown)
 	{
 		readInput();
-		if (editor.ui.sysReqEnterPressed)
+		if (ui.sysReqEnterPressed)
 			pbMix();
 
 		setSyncedReplayerVars();
@@ -1205,9 +1256,10 @@ static void pbSampEndVolUp(void)
 
 static int32_t SDLCALL applyVolumeThread(void *ptr)
 {
+#define AMP_BITS 15
 	int8_t *ptr8;
 	int16_t *ptr16;
-	int32_t vol1, vol2, tmp32, x1, x2, len, i;
+	int32_t amp32, x1, x2, len, i;
 	sampleTyp *s;
 
 	if (instr[editor.curInstr] == NULL)
@@ -1215,8 +1267,6 @@ static int32_t SDLCALL applyVolumeThread(void *ptr)
 
 	s = &instr[editor.curInstr]->samp[editor.curSmp];
 
-	(void)ptr;
-
 	if (smpEd_Rx1 < smpEd_Rx2)
 	{
 		x1 = smpEd_Rx1;
@@ -1245,30 +1295,36 @@ static int32_t SDLCALL applyVolumeThread(void *ptr)
 
 	if (s->typ & 16)
 	{
-		x1 /= 2;
-		x2 /= 2;
+		x1 >>= 1;
+		x2 >>= 1;
 	}
 
 	len = x2 - x1;
 	if (len <= 0)
 		goto applyVolumeExit;
 
+	const double dVol1 = vol_StartVol * ((1 << AMP_BITS) / 100.0);
+	const double dVol2 = vol_EndVol * ((1 << AMP_BITS) / 100.0);
+
+	/* 8bitbubsy: Rewritten to use 64-bit deltas instead, to
+	** prevent having to do a 64-bit mul for every output sample.
+	*/
+	const int64_t delta64 = (int64_t)round(((dVol2 - dVol1) * (UINT32_MAX+1.0)) / len);
+	int64_t pos64 = (int64_t)dVol1 << 32;
+
 	pauseAudio();
 	restoreSample(s);
-
-	// scale values for faster math and suitable rounding for PCM waveforms (DIV -> arithmetic bitshift right)
-	vol1 = (int32_t)round((vol_StartVol * 256) / 100.0);
-	vol2 = (int32_t)round((vol_EndVol * 256) / 100.0) - vol1;
-
 	if (s->typ & 16)
 	{
 		ptr16 = (int16_t *)s->pek;
 		for (i = x1; i < x2; i++)
 		{
-			tmp32 = vol1 + (int32_t)(((int64_t)(i - x1) * vol2) / len);
-			tmp32 = (ptr16[i] * tmp32) >> 8;
-			CLAMP16(tmp32);
-			ptr16[i] = (int16_t)tmp32;
+			amp32 = (int32_t)(pos64 >> 32);
+			pos64 += delta64;
+
+			amp32 = (ptr16[i] * amp32) >> AMP_BITS;
+			CLAMP16(amp32);
+			ptr16[i] = (int16_t)amp32;
 		}
 	}
 	else
@@ -1276,20 +1332,24 @@ static int32_t SDLCALL applyVolumeThread(void *ptr)
 		ptr8 = s->pek;
 		for (i = x1; i < x2; i++)
 		{
-			tmp32 = vol1 + (int32_t)(((int64_t)(i - x1) * vol2) / len);
-			tmp32 = (ptr8[i] * tmp32) >> 8;
-			CLAMP8(tmp32);
-			ptr8[i] = (int8_t)tmp32;
+			amp32 = (int32_t)(pos64 >> 32);
+			pos64 += delta64;
+
+			amp32 = (ptr8[i] * amp32) >> AMP_BITS;
+			CLAMP8(amp32);
+			ptr8[i] = (int8_t)amp32;
 		}
 	}
 	fixSample(s);
-
 	resumeAudio();
+
 	setSongModifiedFlag();
 
 applyVolumeExit:
 	setMouseBusy(false);
-	editor.ui.sysReqShown = false;
+	ui.sysReqShown = false;
+
+	(void)ptr;
 	return true;
 }
 
@@ -1297,7 +1357,7 @@ static void pbApplyVolume(void)
 {
 	if (vol_StartVol == 100 && vol_EndVol == 100)
 	{
-		editor.ui.sysReqShown = false;
+		ui.sysReqShown = false;
 		return; // no volume change to be done
 	}
 
@@ -1450,7 +1510,7 @@ static void drawSampleVolumeBox(void)
 	hLine(x + 2,     y + h - 3, w - 4, PAL_BUTTON1);
 
 	textOutShadow(172, 236, PAL_FORGRND, PAL_BUTTON2, "Start volume");
-	textOutShadow(172, 249, PAL_FORGRND, PAL_BUTTON2, "End volume");
+	textOutShadow(172, 250, PAL_FORGRND, PAL_BUTTON2, "End volume");
 	charOutShadow(282, 236, PAL_FORGRND, PAL_BUTTON2, '%');
 	charOutShadow(282, 250, PAL_FORGRND, PAL_BUTTON2, '%');
 
@@ -1462,20 +1522,20 @@ static void drawSampleVolumeBox(void)
 	if (val > 99)
 	{
 		charOut(253, 236, PAL_FORGRND, sign);
-		charOut(260, 236, PAL_FORGRND, '0' + ((val / 100) % 10));
+		charOut(260, 236, PAL_FORGRND, '0' + (char)(val / 100));
 		charOut(267, 236, PAL_FORGRND, '0' + ((val / 10) % 10));
 		charOut(274, 236, PAL_FORGRND, '0' + (val % 10));
 	}
 	else if (val > 9)
 	{
 		charOut(260, 236, PAL_FORGRND, sign);
-		charOut(267, 236, PAL_FORGRND, '0' + ((val / 10) % 10));
+		charOut(267, 236, PAL_FORGRND, '0' + (char)(val / 10));
 		charOut(274, 236, PAL_FORGRND, '0' + (val % 10));
 	}
 	else
 	{
 		charOut(267, 236, PAL_FORGRND, sign);
-		charOut(274, 236, PAL_FORGRND, '0' + (val % 10));
+		charOut(274, 236, PAL_FORGRND, '0' + (char)val);
 	}
 
 	     if (vol_EndVol == 0) sign = ' ';
@@ -1486,20 +1546,20 @@ static void drawSampleVolumeBox(void)
 	if (val > 99)
 	{
 		charOut(253, 250, PAL_FORGRND, sign);
-		charOut(260, 250, PAL_FORGRND, '0' + ((val / 100) % 10));
+		charOut(260, 250, PAL_FORGRND, '0' + (char)(val / 100));
 		charOut(267, 250, PAL_FORGRND, '0' + ((val / 10) % 10));
 		charOut(274, 250, PAL_FORGRND, '0' + (val % 10));
 	}
 	else if (val > 9)
 	{
 		charOut(260, 250, PAL_FORGRND, sign);
-		charOut(267, 250, PAL_FORGRND, '0' + ((val / 10) % 10));
+		charOut(267, 250, PAL_FORGRND, '0' + (char)(val / 10));
 		charOut(274, 250, PAL_FORGRND, '0' + (val % 10));
 	}
 	else
 	{
 		charOut(267, 250, PAL_FORGRND, sign);
-		charOut(274, 250, PAL_FORGRND, '0' + (val % 10));
+		charOut(274, 250, PAL_FORGRND, '0' + (char)val);
 	}
 }
 
@@ -1633,10 +1693,10 @@ void pbSampleVolume(void)
 	windowOpen();
 
 	exitFlag = false;
-	while (editor.ui.sysReqShown)
+	while (ui.sysReqShown)
 	{
 		readInput();
-		if (editor.ui.sysReqEnterPressed)
+		if (ui.sysReqEnterPressed)
 		{
 			pbApplyVolume();
 			keyb.ignoreCurrKeyUp = true; // don't handle key up event for this key release
@@ -1646,7 +1706,7 @@ void pbSampleVolume(void)
 		handleRedrawing();
 
 		// this is needed for the "Get maximum scale" button
-		if (editor.ui.setMouseIdle) mouseAnimOff();
+		if (ui.setMouseIdle) mouseAnimOff();
 
 		drawSampleVolumeBox();
 		setScrollBarPos(0, 500 + vol_StartVol, false);
