@@ -3,8 +3,12 @@
 #include <stdint.h>
 #include <stdbool.h>
 #include "ft2_header.h"
+#include "ft2_audio.h"
+
+#define SCOPE_FRAC_BITS 32
+#define SCOPE_FRAC_SCALE (1ULL << SCOPE_FRAC_BITS)
+#define SCOPE_FRAC_MASK (SCOPE_FRAC_SCALE-1)
 
-void resetOldScopeRates(void);
 int32_t getSamplePosition(uint8_t ch);
 void stopAllScopes(void);
 void refreshScopes(void);
@@ -20,10 +24,11 @@ typedef struct scope_t
 	const int8_t *sampleData8;
 	const int16_t *sampleData16;
 	int8_t SVol;
-	bool wasCleared, sample16Bit;
+	bool wasCleared, sample16Bit, backwards;
 	uint8_t loopType;
-	int32_t SPosDir, SRepS, SRepL, SLen, SPos;
-	uint32_t SFrq, SPosDec;
+	int32_t SRepS, SRepL, SLen, SPos;
+	uint32_t DFrq;
+	uint64_t SFrq, SPosDec;
 } scope_t;
 
 typedef struct lastChInstr_t
