@@ -11,8 +11,9 @@
 #include "ft2_config.h"
 #include "ft2_diskop.h"
 #include "ft2_keyboard.h"
-#include "ft2_gfxdata.h"
 #include "ft2_mouse.h"
+#include "ft2_bmp.h"
+#include "ft2_structs.h"
 
 textBox_t textBoxes[NUM_TEXTBOXES] =
 {
@@ -63,7 +64,8 @@ textBox_t textBoxes[NUM_TEXTBOXES] =
 };
 
 static int16_t markX1, markX2;
-static uint16_t oldCursorPos, oldMouseX;
+static uint16_t oldCursorPos;
+static int32_t oldMouseX;
 
 static void moveTextCursorLeft(int16_t i, bool updateTextBox);
 static void moveTextCursorRight(int16_t i, bool updateTextBox);
@@ -498,7 +500,7 @@ static void textOutBuf(uint8_t *dstBuffer, uint32_t dstWidth, uint8_t paletteInd
 
 		if (chr != ' ')
 		{
-			srcPtr = &font1Data[chr * FONT1_CHAR_W];
+			srcPtr = &bmp.font1[chr * FONT1_CHAR_W];
 			dstPtr = &dstBuffer[currX];
 
 			for (uint32_t y = 0; y < FONT1_CHAR_H; y++)
@@ -518,31 +520,6 @@ static void textOutBuf(uint8_t *dstBuffer, uint32_t dstWidth, uint8_t paletteInd
 	}
 }
 
-static void blitClipW(uint16_t xPos, uint16_t yPos, const uint8_t *srcPtr, uint16_t w, uint16_t h, uint16_t clipW)
-{
-	uint16_t blitW;
-	uint32_t *dstPtr;
-
-	blitW = w;
-	if (blitW > clipW)
-		blitW = clipW;
-
-	assert(xPos < SCREEN_W && yPos < SCREEN_H && xPos+blitW <= SCREEN_W && yPos+h <= SCREEN_H && srcPtr != NULL);
-
-	dstPtr = &video.frameBuffer[(yPos * SCREEN_W) + xPos];
-	for (uint32_t y = 0; y < h; y++)
-	{
-		for (uint32_t x = 0; x < blitW; x++)
-		{
-			if (srcPtr[x] != PAL_TRANSPR)
-				dstPtr[x] = video.palette[srcPtr[x]];
-		}
-
-		srcPtr += w;
-		dstPtr += SCREEN_W;
-	}
-}
-
  // a lot of filling here, but textboxes are small so no problem...
 void drawTextBox(uint16_t textBoxID)
 {
@@ -615,7 +592,7 @@ void drawTextBox(uint16_t textBoxID)
 	fillRect(t->x + t->tx, t->y + t->ty, t->renderW, 10, pal); // 10 = tallest possible glyph/char height
 
 	// render visible part of text render buffer to screen
-	blitClipW(t->x + t->tx, t->y + t->ty, &t->renderBuf[t->bufOffset], t->renderBufW, t->renderBufH, t->renderW);
+	blitClipX(t->x + t->tx, t->y + t->ty, &t->renderBuf[t->bufOffset], t->renderBufW, t->renderBufH, t->renderW);
 }
 
 void showTextBox(uint16_t textBoxID)
@@ -723,7 +700,7 @@ bool testTextBoxMouseDown(void)
 	oldMouseX = mouse.x;
 	oldCursorPos = 0;
 
-	if (editor.ui.sysReqShown)
+	if (ui.sysReqShown)
 	{
 		// if a system request is open, only test the first textbox (reserved)
 		start = 0;
