@@ -15,8 +15,9 @@
 #include "ft2_replayer.h"
 #include "ft2_audio.h"
 #include "ft2_mouse.h"
+#include "ft2_structs.h"
 
-// Messy But Works
+// this is truly a mess, but it works...
 
 static char byteFormatBuffer[64], tmpInstrName[1 + MAX_INST][22 + 1], tmpInstName[MAX_INST][22 + 1];
 static bool removePatt, removeInst, removeSamp, removeChans, removeSmpDataAfterLoop, convSmpsTo8Bit;
@@ -31,10 +32,12 @@ void pbTrimCalc(void);
 
 static void freeTmpInstruments(void)
 {
-	for (int16_t i = 0; i <= MAX_INST; i++)
+	for (int32_t i = 0; i <= MAX_INST; i++)
 	{
 		if (tmpInstr[i] != NULL)
 		{
+			// don't free samples, as the pointers are shared with main instruments...
+
 			free(tmpInstr[i]);
 			tmpInstr[i] = NULL;
 		}
@@ -45,7 +48,7 @@ static bool setTmpInstruments(void)
 {
 	freeTmpInstruments();
 
-	for (int16_t i = 0; i <= MAX_INST; i++)
+	for (int32_t i = 0; i <= MAX_INST; i++)
 	{
 		if (instr[i] != NULL)
 		{
@@ -56,19 +59,19 @@ static bool setTmpInstruments(void)
 				return false;
 			}
 
-			memcpy(tmpInstr[i], instr[i], sizeof (instrTyp));
+			*tmpInstr[i] = *instr[i];
 		}
 	}
 
 	return true;
 }
 
-static void remapInstrInSong(uint8_t src, uint8_t dst, int16_t ap)
+static void remapInstrInSong(uint8_t src, uint8_t dst, int32_t ap)
 {
 	int32_t readLen;
 	tonTyp *pattPtr, *note;
 
-	for (int16_t i = 0; i < ap; i++)
+	for (int32_t i = 0; i < ap; i++)
 	{
 		pattPtr = patt[i];
 		if (pattPtr == NULL)
@@ -99,7 +102,8 @@ static int16_t getUsedTempSamples(uint16_t nr)
 		i--;
 
 	/* Yes, 'i' can be -1 here, and will be set to at least 0
-	 * because of ins->ta values. Possibly an FT2 bug... */
+	** because of ins->ta values. Possibly an FT2 bug...
+	**/
 	for (j = 0; j < 96; j++)
 	{
 		if (ins->ta[j] > i)
@@ -146,10 +150,11 @@ static int64_t getTempInsAndSmpSize(void)
 	return currSize64;
 }
 
-static void wipeInstrUnused(bool testWipeSize, int16_t *ai, int16_t ap, uint8_t antChn)
+static void wipeInstrUnused(bool testWipeSize, int16_t *ai, int32_t ap, int32_t antChn)
 {
 	uint8_t newInst;
-	int16_t numInsts, newNumInsts, instToDel, i, j, k, pattLen;
+	int16_t newNumInsts, instToDel, pattLen;
+	int32_t numInsts, i, j, k;
 	tonTyp *pattPtr;
 
 	numInsts = *ai;
@@ -410,10 +415,11 @@ static void wipeSamplesUnused(bool testWipeSize, int16_t ai)
 				{
 					// sample is unused
 
-					if (s->pek != NULL && !testWipeSize)
-						free(s->pek);
+					if (s->origPek != NULL && !testWipeSize)
+						free(s->origPek);
 
 					memset(s, 0, sizeof (sampleTyp));
+					s->origPek = NULL;
 					s->pek = NULL;
 				}
 			}
@@ -484,7 +490,7 @@ static void wipeSmpDataAfterLoop(bool testWipeSize, int16_t ai)
 		for (int16_t j = 0; j < l; j++)
 		{
 			s = &ins->samp[j];
-			if (s->pek != NULL && s->typ & 3 && s->len > 0 && s->len > s->repS+s->repL)
+			if (s->origPek != NULL && s->typ & 3 && s->len > 0 && s->len > s->repS+s->repL)
 			{
 				if (!testWipeSize)
 					restoreSample(s);
@@ -495,14 +501,19 @@ static void wipeSmpDataAfterLoop(bool testWipeSize, int16_t ai)
 					if (s->len <= 0)
 					{
 						s->len = 0;
-						free(s->pek);
+
+						free(s->origPek);
+						s->origPek = NULL;
 						s->pek = NULL;
 					}
 					else
 					{
-						newPtr = (int8_t *)realloc(s->pek, s->len + LOOP_FIX_LEN);
+						newPtr = (int8_t *)realloc(s->origPek, s->len + LOOP_FIX_LEN);
 						if (newPtr != NULL)
-							s->pek = newPtr;
+						{
+							s->origPek = newPtr;
+							s->pek = s->origPek + SMP_DAT_OFFSET;
+						}
 					}
 				}
 
@@ -547,37 +558,41 @@ static void convertSamplesTo8bit(bool testWipeSize, int16_t ai)
 		for (int16_t j = 0; j < k; j++)
 		{
 			s = &ins->samp[j];
-			if (s->pek != NULL && (s->typ & 16) && s->len > 0)
+			if (s->origPek != NULL && (s->typ & 16) && s->len > 0)
 			{
 				if (testWipeSize)
 				{
 					s->typ &= ~16;
-					s->len /= 2;
-					s->repL /= 2;
-					s->repS /= 2;
+					s->len >>= 1;
+					s->repL >>= 1;
+					s->repS >>= 1;
 				}
 				else
 				{
 					restoreSample(s);
 
+					assert(s->pek != NULL);
 					src16 = (int16_t *)s->pek;
 					dst8 = s->pek;
 
-					newLen = s->len / 2;
+					newLen = s->len >> 1;
 					for (int32_t a = 0; a < newLen; a++)
 					{
 						smp8 = src16[a] >> 8;
 						dst8[a] = smp8;
 					}
 
-					s->repL /= 2;
-					s->repS /= 2;
-					s->len /= 2;
+					s->repL >>= 1;
+					s->repS >>= 1;
+					s->len >>= 1;
 					s->typ &= ~16;
 
-					newPtr = (int8_t *)realloc(s->pek, s->len + LOOP_FIX_LEN);
+					newPtr = (int8_t *)realloc(s->origPek, s->len + LOOP_FIX_LEN);
 					if (newPtr != NULL)
-						s->pek = newPtr;
+					{
+						s->origPek = newPtr;
+						s->pek = s->origPek + SMP_DAT_OFFSET;
+					}
 
 					fixSample(s);
 				}
@@ -586,7 +601,7 @@ static void convertSamplesTo8bit(bool testWipeSize, int16_t ai)
 	}
 }
 
-static uint16_t getPackedPattSize(tonTyp *pattern, uint16_t numRows, uint8_t antChn)
+static uint16_t getPackedPattSize(tonTyp *pattern, int32_t numRows, int32_t antChn)
 {
 	uint8_t bytes[sizeof (tonTyp)], packBits, *writePtr, *firstBytePtr, *pattPtr;
 	uint16_t totalPackLen;
@@ -596,9 +611,9 @@ static uint16_t getPackedPattSize(tonTyp *pattern, uint16_t numRows, uint8_t ant
 	pattPtr = (uint8_t *)pattern;
 
 	writePtr = pattPtr;
-	for (uint16_t row = 0; row < numRows; row++)
+	for (int32_t row = 0; row < numRows; row++)
 	{
-		for (uint16_t chn = 0; chn < antChn; chn++)
+		for (int32_t chn = 0; chn < antChn; chn++)
 		{
 			bytes[0] = *pattPtr++;
 			bytes[1] = *pattPtr++;
@@ -635,10 +650,10 @@ static uint16_t getPackedPattSize(tonTyp *pattern, uint16_t numRows, uint8_t ant
 	return totalPackLen;
 }
 
-static bool tmpPatternEmpty(uint16_t nr, uint8_t antChn)
+static bool tmpPatternEmpty(uint16_t nr, int32_t antChn)
 {
 	uint8_t *scanPtr;
-	uint32_t pattLen, scanLen;
+	int32_t pattLen, scanLen;
 
 	if (tmpPatt[nr] == NULL)
 		return true;
@@ -647,9 +662,9 @@ static bool tmpPatternEmpty(uint16_t nr, uint8_t antChn)
 	scanLen = antChn * sizeof (tonTyp);
 	pattLen = tmpPattLens[nr];
 
-	for (uint32_t i = 0; i < pattLen; i++)
+	for (int32_t i = 0; i < pattLen; i++)
 	{
-		for (uint32_t j = 0; j < scanLen; j++)
+		for (int32_t j = 0; j < scanLen; j++)
 		{
 			if (scanPtr[j] != 0)
 				return false;
@@ -721,9 +736,8 @@ static int64_t calculateXMSize(void)
 
 static int64_t calculateTrimSize(void)
 {
-	uint8_t antChn;
 	int16_t ap, i, j, k, ai, highestChan, pattLen;
-	int32_t pattDataLen, newPattDataLen;
+	int32_t antChn, pattDataLen, newPattDataLen;
 	int64_t bytes64, oldInstrSize64, newInstrSize64;
 	tonTyp *note, *pattPtr;
 
@@ -986,14 +1000,14 @@ void trimThreadDone(void)
 
 	if (removeChans)
 	{
-		if (editor.ui.patternEditorShown)
+		if (ui.patternEditorShown)
 		{
-			if (editor.ui.channelOffset > song.antChn-editor.ui.numChannelsShown)
-				setScrollBarPos(SB_CHAN_SCROLL, song.antChn - editor.ui.numChannelsShown, true);
+			if (ui.channelOffset > song.antChn-ui.numChannelsShown)
+				setScrollBarPos(SB_CHAN_SCROLL, song.antChn - ui.numChannelsShown, true);
 		}
 
-		if (editor.cursor.ch >= editor.ui.channelOffset+editor.ui.numChannelsShown)
-			editor.cursor.ch = editor.ui.channelOffset+editor.ui.numChannelsShown - 1;
+		if (cursor.ch >= ui.channelOffset+ui.numChannelsShown)
+			cursor.ch = ui.channelOffset+ui.numChannelsShown - 1;
 	}
 
 	checkMarkLimits();
@@ -1124,28 +1138,28 @@ void hideTrimScreen(void)
 	hidePushButton(PB_TRIM_CALC);
 	hidePushButton(PB_TRIM_TRIM);
 
-	editor.ui.trimScreenShown = false;
-	editor.ui.scopesShown = true;
+	ui.trimScreenShown = false;
+	ui.scopesShown = true;
 	drawScopeFramework();
 }
 
 void showTrimScreen(void)
 {
-	if (editor.ui.extended)
+	if (ui.extended)
 		exitPatternEditorExtended();
 
 	hideTopScreen();
 	showTopScreen(false);
 
-	editor.ui.trimScreenShown = true;
-	editor.ui.scopesShown = false;
+	ui.trimScreenShown = true;
+	ui.scopesShown = false;
 
 	drawTrimScreen();
 }
 
 void toggleTrimScreen(void)
 {
-	if (editor.ui.trimScreenShown)
+	if (ui.trimScreenShown)
 		hideTrimScreen();
 	else
 		showTrimScreen();
@@ -1207,7 +1221,7 @@ void pbTrimCalc(void)
 	if (xmAfterTrimSize64 < 0)
 		xmAfterTrimSize64 = 0;
 
-	if (editor.ui.trimScreenShown)
+	if (ui.trimScreenShown)
 		drawTrimScreen();
 }
 
@@ -1239,6 +1253,6 @@ void resetTrimSizes(void)
 	xmAfterTrimSize64 = -1;
 	spaceSaved64 = -1;
 
-	if (editor.ui.trimScreenShown)
+	if (ui.trimScreenShown)
 		drawTrimScreen();
 }
