@@ -16,7 +16,6 @@
 #endif
 #include "ft2_header.h"
 #include "ft2_config.h"
-#include "ft2_gfxdata.h"
 #include "ft2_gui.h"
 #include "ft2_video.h"
 #include "ft2_events.h"
@@ -33,6 +32,8 @@
 #include "ft2_sampling.h"
 #include "ft2_module_loader.h"
 #include "ft2_midi.h"
+#include "ft2_bmp.h"
+#include "ft2_structs.h"
 
 static const uint8_t textCursorData[12] =
 {
@@ -42,6 +43,8 @@ static const uint8_t textCursorData[12] =
 	PAL_FORGRND, PAL_FORGRND, PAL_FORGRND
 };
 
+video_t video; // globalized
+
 static bool songIsModified;
 static char wndTitle[128 + PATH_MAX];
 static uint64_t timeNext64, timeNext64Frac;
@@ -49,7 +52,7 @@ static sprite_t sprites[SPRITE_NUM];
 
 // for FPS counter
 #define FPS_SCAN_FRAMES 60
-#define FPS_RENDER_W 280
+#define FPS_RENDER_W 285
 #define FPS_RENDER_H (((FONT1_CHAR_H + 1) * 8) + 1)
 #define FPS_RENDER_X 2
 #define FPS_RENDER_Y 2
@@ -162,8 +165,6 @@ void endFPSCounter(void)
 
 void flipFrame(void)
 {
-	uint32_t windowFlags = SDL_GetWindowFlags(video.window);
-
 	renderSprites();
 
 	if (video.showFPSCounter)
@@ -181,6 +182,8 @@ void flipFrame(void)
 	}
 	else
 	{
+		uint32_t windowFlags = SDL_GetWindowFlags(video.window);
+
 		/* We have VSync, but it can unexpectedly get inactive in certain scenarios.
 		** We have to force thread sleeping (to ~60Hz) if so.
 		*/
@@ -260,8 +263,8 @@ static void updateRenderSizeVars(void)
 			if (dXUpscale != 0.0) video.renderW = (int32_t)(video.renderW / dXUpscale);
 			if (dYUpscale != 0.0) video.renderH = (int32_t)(video.renderH / dYUpscale);
 #endif
-			video.renderX = (video.displayW - video.renderW) / 2;
-			video.renderY = (video.displayH - video.renderH) / 2;
+			video.renderX = (video.displayW - video.renderW) >> 1;
+			video.renderY = (video.displayH - video.renderH) >> 1;
 		}
 	}
 	else
@@ -273,8 +276,8 @@ static void updateRenderSizeVars(void)
 	}
 
 	// for mouse cursor creation
-	video.xScale = (uint32_t)round(video.renderW / (double)SCREEN_W);
-	video.yScale = (uint32_t)round(video.renderH / (double)SCREEN_H);
+	video.xScale = (uint32_t)round(video.renderW * (1.0 / SCREEN_W));
+	video.yScale = (uint32_t)round(video.renderH * (1.0 / SCREEN_H));
 	createMouseCursors();
 }
 
@@ -282,8 +285,8 @@ void enterFullscreen(void)
 {
 	SDL_DisplayMode dm;
 
-	strcpy(editor.ui.fullscreenButtonText, "Go windowed");
-	if (editor.ui.configScreenShown && editor.currConfigScreen == CONFIG_SCREEN_MISCELLANEOUS)
+	strcpy(ui.fullscreenButtonText, "Go windowed");
+	if (ui.configScreenShown && editor.currConfigScreen == CONFIG_SCREEN_MISCELLANEOUS)
 		showConfigScreen(); // redraw so that we can see the new button text
 
 	if (config.windowFlags & FILTERING)
@@ -307,8 +310,8 @@ void enterFullscreen(void)
 
 void leaveFullScreen(void)
 {
-	strcpy(editor.ui.fullscreenButtonText, "Go fullscreen");
-	if (editor.ui.configScreenShown && editor.currConfigScreen == CONFIG_SCREEN_MISCELLANEOUS)
+	strcpy(ui.fullscreenButtonText, "Go fullscreen");
+	if (ui.configScreenShown && editor.currConfigScreen == CONFIG_SCREEN_MISCELLANEOUS)
 		showConfigScreen(); // redraw so that we can see the new button text
 
 	SDL_SetWindowFullscreen(video.window, 0);
@@ -341,17 +344,17 @@ bool setupSprites(void)
 	memset(sprites, 0, sizeof (sprites));
 
 	s = &sprites[SPRITE_MOUSE_POINTER];
-	s->data = mouseCursors;
+	s->data = bmp.mouseCursors;
 	s->w = MOUSE_CURSOR_W;
 	s->h = MOUSE_CURSOR_H;
 
 	s = &sprites[SPRITE_LEFT_LOOP_PIN];
-	s->data = leftLoopPinUnclicked;
+	s->data = &bmp.loopPins[0*(154*16)];
 	s->w = 16;
 	s->h = SAMPLE_AREA_HEIGHT;
 
 	s = &sprites[SPRITE_RIGHT_LOOP_PIN];
-	s->data = rightLoopPinUnclicked;
+	s->data = &bmp.loopPins[2*(154*16)];
 	s->w = 16;
 	s->h = SAMPLE_AREA_HEIGHT;
 
@@ -376,7 +379,7 @@ bool setupSprites(void)
 	return true;
 }
 
-void changeSpriteData(uint8_t sprite, const uint8_t *data)
+void changeSpriteData(int32_t sprite, const uint8_t *data)
 {
 	sprites[sprite].data = data;
 	memset(sprites[sprite].refreshBuffer, 0, sprites[sprite].w * sprites[sprite].h * sizeof (int32_t));
@@ -384,7 +387,7 @@ void changeSpriteData(uint8_t sprite, const uint8_t *data)
 
 void freeSprites(void)
 {
-	for (uint32_t i = 0; i < SPRITE_NUM; i++)
+	for (int32_t i = 0; i < SPRITE_NUM; i++)
 	{
 		if (sprites[i].refreshBuffer != NULL)
 		{
@@ -396,26 +399,26 @@ void freeSprites(void)
 
 void setLeftLoopPinState(bool clicked)
 {
-	changeSpriteData(SPRITE_LEFT_LOOP_PIN, clicked ? leftLoopPinClicked : leftLoopPinUnclicked);
+	changeSpriteData(SPRITE_LEFT_LOOP_PIN, clicked ? &bmp.loopPins[1*(154*16)] : &bmp.loopPins[0*(154*16)]);
 }
 
 void setRightLoopPinState(bool clicked)
 {
-	changeSpriteData(SPRITE_RIGHT_LOOP_PIN, clicked ? rightLoopPinClicked : rightLoopPinUnclicked);
+	changeSpriteData(SPRITE_RIGHT_LOOP_PIN, clicked ? &bmp.loopPins[3*(154*16)] : &bmp.loopPins[2*(154*16)]);
 }
 
-int32_t getSpritePosX(uint8_t sprite)
+int32_t getSpritePosX(int32_t sprite)
 {
 	return sprites[sprite].x;
 }
 
-void setSpritePos(uint8_t sprite, int16_t x, int16_t y)
+void setSpritePos(int32_t sprite, int32_t x, int32_t y)
 {
-	sprites[sprite].newX = x;
-	sprites[sprite].newY = y;
+	sprites[sprite].newX = (int16_t)x;
+	sprites[sprite].newY = (int16_t)y;
 }
 
-void hideSprite(uint8_t sprite)
+void hideSprite(int32_t sprite)
 {
 	sprites[sprite].newX = SCREEN_W;
 }
@@ -423,7 +426,7 @@ void hideSprite(uint8_t sprite)
 void eraseSprites(void)
 {
 	int8_t i;
-	register int32_t x, y, sw, sh, srcPitch, dstPitch;
+	int32_t sx, sy, x, y, sw, sh, srcPitch, dstPitch;
 	const uint32_t *src32;
 	uint32_t *dst32;
 	sprite_t *s;
@@ -431,28 +434,36 @@ void eraseSprites(void)
 	for (i = SPRITE_NUM-1; i >= 0; i--) // erasing must be done in reverse order
 	{
 		s = &sprites[i];
-		if (s->x >= SCREEN_W) // sprite is hidden, don't erase
+		if (s->x >= SCREEN_W || s->y >= SCREEN_H) // sprite is hidden, don't draw nor fill clear buffer
 			continue;
 
-		assert(s->y >= 0 && s->refreshBuffer != NULL);
+		assert(s->refreshBuffer != NULL);
 
 		sw = s->w;
 		sh = s->h;
-		x  = s->x;
-		y  = s->y;
+		sx = s->x;
+		sy = s->y;
 
-		// if x is negative, adjust variables (can only happen on loop pins in smp. ed.)
-		if (x < 0)
+		// if x is negative, adjust variables
+		if (sx < 0)
 		{
-			sw += x; // subtraction
-			x = 0;
+			sw += sx; // subtraction
+			sx = 0;
+		}
+
+		// if y is negative, adjust variables
+		if (sy < 0)
+		{
+			sh += sy; // subtraction
+			sy = 0;
 		}
 
 		src32 = s->refreshBuffer;
-		dst32 = &video.frameBuffer[(y * SCREEN_W) + x];
+		dst32 = &video.frameBuffer[(sy * SCREEN_W) + sx];
 
-		if (y+sh >= SCREEN_H) sh = SCREEN_H - y;
-		if (x+sw >= SCREEN_W) sw = SCREEN_W - x;
+		// handle x/y clipping
+		if (sx+sw >= SCREEN_W) sw = SCREEN_W - sx;
+		if (sy+sh >= SCREEN_H) sh = SCREEN_H - sy;
 
 		srcPitch = s->w - sw;
 		dstPitch = SCREEN_W - sw;
@@ -471,7 +482,7 @@ void eraseSprites(void)
 void renderSprites(void)
 {
 	const uint8_t *src8;
-	register int32_t x, y, sw, sh, srcPitch, dstPitch;
+	int32_t sx, sy, x, y, sw, sh, srcPitch, dstPitch;
 	uint32_t i, *clr32, *dst32, windowFlags;
 	sprite_t *s;
 
@@ -495,20 +506,42 @@ void renderSprites(void)
 		s->x = s->newX;
 		s->y = s->newY;
 
-		if (s->x >= SCREEN_W) // sprite is hidden, don't draw nor fill clear buffer
+		if (s->x >= SCREEN_W || s->y >= SCREEN_H) // sprite is hidden, don't draw nor fill clear buffer
 			continue;
 
-		assert(s->x >= 0 && s->y >= 0 && s->data != NULL && s->refreshBuffer != NULL);
+		assert(s->data != NULL && s->refreshBuffer != NULL);
 
 		sw = s->w;
 		sh = s->h;
+		sx = s->x;
+		sy = s->y;
 		src8 = s->data;
-		dst32 = &video.frameBuffer[(s->y * SCREEN_W) + s->x];
+
+		// if x is negative, adjust variables
+		if (sx < 0)
+		{
+			sw += sx; // subtraction
+			src8 -= sx; // addition
+			sx = 0;
+		}
+
+		// if y is negative, adjust variables
+		if (sy < 0)
+		{
+			sh += sy; // subtraction
+			src8 += (-sy * s->w); // addition
+			sy = 0;
+		}
+
+		if (sw <= 0 || sh <= 0) // sprite is hidden, don't draw nor fill clear buffer
+			continue;
+
+		dst32 = &video.frameBuffer[(sy * SCREEN_W) + sx];
 		clr32 = s->refreshBuffer;
 
-		// handle xy clipping
-		if (s->y+sh >= SCREEN_H) sh = SCREEN_H - s->y;
-		if (s->x+sw >= SCREEN_W) sw = SCREEN_W - s->x;
+		// handle x/y clipping
+		if (sx+sw >= SCREEN_W) sw = SCREEN_W - sx;
+		if (sy+sh >= SCREEN_H) sh = SCREEN_H - sy;
 
 		srcPitch = s->w - sw;
 		dstPitch = SCREEN_W - sw;
@@ -570,8 +603,7 @@ void renderLoopPins(void)
 {
 	uint8_t pal;
 	const uint8_t *src8;
-	int32_t sx;
-	register int32_t x, y, sw, sh, srcPitch, dstPitch;
+	int32_t sx, x, y, sw, sh, srcPitch, dstPitch;
 	uint32_t *clr32, *dst32;
 	sprite_t *s;
 
@@ -604,7 +636,7 @@ void renderLoopPins(void)
 		dst32 = &video.frameBuffer[(s->y * SCREEN_W) + sx];
 
 		// handle x clipping
-		if (s->x+sw >= SCREEN_W) sw = SCREEN_W - s->x;
+		if (sx+sw >= SCREEN_W) sw = SCREEN_W - sx;
 
 		srcPitch = s->w - sw;
 		dstPitch = SCREEN_W - sw;
@@ -663,7 +695,7 @@ void renderLoopPins(void)
 		dst32 = &video.frameBuffer[(s->y * SCREEN_W) + sx];
 
 		// handle x clipping
-		if (s->x+sw >= SCREEN_W) sw = SCREEN_W - s->x;
+		if (sx+sw >= SCREEN_W) sw = SCREEN_W - sx;
 
 		srcPitch = s->w - sw;
 		dstPitch = SCREEN_W - sw;
@@ -711,20 +743,20 @@ void setupWaitVBL(void)
 void waitVBL(void)
 {
 	// this routine almost never delays if we have 60Hz vsync, but it's still needed in some occasions
-	int32_t time32;
-	uint32_t diff32;
-	uint64_t time64;
 
-	time64 = SDL_GetPerformanceCounter();
+	uint64_t time64 = SDL_GetPerformanceCounter();
 	if (time64 < timeNext64)
 	{
-		assert(timeNext64-time64 <= 0xFFFFFFFFULL);
-		diff32 = (uint32_t)(timeNext64 - time64);
+		time64 = timeNext64 - time64;
+		if (time64 > UINT32_MAX)
+			time64 = UINT32_MAX;
+
+		const uint32_t diff32 = (uint32_t)time64;
 
 		// convert and round to microseconds
-		time32 = (int32_t)((diff32 * editor.dPerfFreqMulMicro) + 0.5);
+		const int32_t time32 = (int32_t)((diff32 * editor.dPerfFreqMulMicro) + 0.5);
 
-		// delay until we have reached next frame
+		// delay until we have reached the next frame
 		if (time32 > 0)
 			usleep(time32);
 	}
@@ -924,7 +956,7 @@ bool setupWindow(void)
 		return false;
 	}
 
-#ifdef __APPLE__ // for macOS we need to do this here for reasons I can't be bothered to explain
+#ifdef __APPLE__ // for macOS we need to do this here for reasons I have forgotten
 	SDL_PumpEvents();
 	SDL_ShowWindow(video.window);
 #endif
@@ -998,6 +1030,12 @@ bool setupRenderer(void)
 	else
 		SDL_ShowCursor(SDL_FALSE);
 
+	// Workaround: SDL_GetGlobalMouseState() doesn't work with KMSDRM
+	video.useDesktopMouseCoords = true;
+	const char *videoDriver = SDL_GetCurrentVideoDriver();
+	if (videoDriver != NULL && strcmp("KMSDRM", videoDriver) == 0)
+		video.useDesktopMouseCoords = false;
+
 	return true;
 }
 
@@ -1005,24 +1043,24 @@ void handleRedrawing(void)
 {
 	textBox_t *txt;
 
-	if (!editor.ui.configScreenShown && !editor.ui.helpScreenShown)
+	if (!ui.configScreenShown && !ui.helpScreenShown)
 	{
-		if (editor.ui.aboutScreenShown)
+		if (ui.aboutScreenShown)
 		{
 			aboutFrame();
 		}
-		else if (editor.ui.nibblesShown)
+		else if (ui.nibblesShown)
 		{
 			if (editor.NI_Play)
 				moveNibblePlayers();
 		}
 		else
 		{
-			if (editor.ui.updatePosSections)
+			if (ui.updatePosSections)
 			{
-				editor.ui.updatePosSections = false;
+				ui.updatePosSections = false;
 
-				if (!editor.ui.diskOpShown)
+				if (!ui.diskOpShown)
 				{
 					drawSongRepS();
 					drawSongLength();
@@ -1037,7 +1075,7 @@ void handleRedrawing(void)
 						setScrollBarPos(SB_POS_ED, editor.songPos, false);
 
 					// draw current mode text (not while in extended pattern editor mode)
-					if (!editor.ui.extended)
+					if (!ui.extended)
 					{
 						fillRect(115, 80, 74, 10, PAL_DESKTOP);
 
@@ -1049,21 +1087,21 @@ void handleRedrawing(void)
 				}
 			}
 
-			if (editor.ui.updatePosEdScrollBar)
+			if (ui.updatePosEdScrollBar)
 			{
-				editor.ui.updatePosEdScrollBar = false;
+				ui.updatePosEdScrollBar = false;
 				setScrollBarPos(SB_POS_ED, song.songPos, false);
 				setScrollBarEnd(SB_POS_ED, (song.len - 1) + 5);
 			}
 
-			if (!editor.ui.extended)
+			if (!ui.extended)
 			{
-				if (!editor.ui.diskOpShown)
+				if (!ui.diskOpShown)
 					drawPlaybackTime();
 
-				if (editor.ui.sampleEditorExtShown)
+				if (ui.sampleEditorExtShown)
 					handleSampleEditorExtRedrawing();
-				else if (editor.ui.scopesShown)
+				else if (ui.scopesShown)
 					drawScopes();
 			}
 		}
@@ -1071,9 +1109,9 @@ void handleRedrawing(void)
 
 	drawReplayerData();
 
-	if (editor.ui.instEditorShown)
+	if (ui.instEditorShown)
 		handleInstEditorRedrawing();
-	else if (editor.ui.sampleEditorShown)
+	else if (ui.sampleEditorShown)
 		handleSamplerRedrawing();
 
 	// blink text edit cursor
@@ -1102,69 +1140,66 @@ static void drawReplayerData(void)
 
 	if (songPlaying)
 	{
-		if (editor.ui.drawReplayerPianoFlag)
+		if (ui.drawReplayerPianoFlag)
 		{
-			editor.ui.drawReplayerPianoFlag = false;
-			if (editor.ui.instEditorShown)
-			{
-				if (chSyncEntry != NULL)
-					drawPianoReplayer(chSyncEntry);
-			}
+			ui.drawReplayerPianoFlag = false;
+			if (ui.instEditorShown && chSyncEntry != NULL)
+				drawPiano(chSyncEntry);
 		}
 
 		drawPosText = true;
-		if (editor.ui.configScreenShown || editor.ui.nibblesShown     ||
-			editor.ui.helpScreenShown   || editor.ui.aboutScreenShown ||
-			editor.ui.diskOpShown)
+		if (ui.configScreenShown || ui.nibblesShown     ||
+			ui.helpScreenShown   || ui.aboutScreenShown ||
+			ui.diskOpShown)
 		{
 			drawPosText = false;
 		}
 
 		if (drawPosText)
 		{
-			if (editor.ui.drawBPMFlag)
+			if (ui.drawBPMFlag)
 			{
-				editor.ui.drawBPMFlag = false;
+				ui.drawBPMFlag = false;
 				drawSongBPM(editor.speed);
 			}
 			
-			if (editor.ui.drawSpeedFlag)
+			if (ui.drawSpeedFlag)
 			{
-				editor.ui.drawSpeedFlag = false;
+				ui.drawSpeedFlag = false;
 				drawSongSpeed(editor.tempo);
 			}
 
-			if (editor.ui.drawGlobVolFlag)
+			if (ui.drawGlobVolFlag)
 			{
-				editor.ui.drawGlobVolFlag = false;
+				ui.drawGlobVolFlag = false;
 				drawGlobalVol(editor.globalVol);
 			}
 
-			if (editor.ui.drawPosEdFlag)
+			if (ui.drawPosEdFlag)
 			{
-				editor.ui.drawPosEdFlag = false;
+				ui.drawPosEdFlag = false;
 				drawPosEdNums(editor.songPos);
 				setScrollBarPos(SB_POS_ED, editor.songPos, false);
 			}
 
-			if (editor.ui.drawPattNumLenFlag)
+			if (ui.drawPattNumLenFlag)
 			{
-				editor.ui.drawPattNumLenFlag = false;
+				ui.drawPattNumLenFlag = false;
 				drawEditPattern(editor.editPattern);
 				drawPatternLength(editor.editPattern);
 			}
 		}
 	}
-	else if (editor.ui.instEditorShown)
+	else if (ui.instEditorShown)
 	{
-		drawPiano();
+		drawPiano(NULL);
 	}
 
 	// handle pattern data updates
-	if (editor.ui.updatePatternEditor)
+	if (ui.updatePatternEditor)
 	{
-		editor.ui.updatePatternEditor = false;
-		if (editor.ui.patternEditorShown)
+		ui.updatePatternEditor = false;
+		if (ui.patternEditorShown)
 			writePattern(editor.pattPos, editor.editPattern);
 	}
 }
