@@ -3,8 +3,18 @@
 #include <stdint.h>
 #include "ft2_header.h"
 
-#define MIN_WAV_RENDER_FREQ 8000
+#define MIN_WAV_RENDER_FREQ 44100
+
+#if defined __amd64__ || defined _WIN64
+
 #define MAX_WAV_RENDER_FREQ 192000
+
+#else
+
+#define MAX_WAV_RENDER_FREQ 48000
+
+#endif
+
 #define MAX_WAV_RENDER_SAMPLES_PER_TICK (((MAX_WAV_RENDER_FREQ * 5) / 2) / MIN_BPM)
 
 void updateWavRendererSettings(void);
